@@ -31,6 +31,17 @@
 #include <string.h>
 #include <stdlib.h>
 
+
+GQuark
+rb_daap_mdns_error_quark (void)
+{
+	static GQuark quark = 0;
+	if (!quark)
+		quark = g_quark_from_static_string ("rb_daap_mdns_error");
+
+	return quark;
+}
+
 #ifdef WITH_DAAP_SUPPORT
 static void mdns_error_dialog (const gchar *impl)
 {
@@ -40,11 +51,6 @@ static void mdns_error_dialog (const gchar *impl)
 			 impl);
 }
 
-typedef struct _CallbackAndData {
-	gpointer callback;
-	gpointer data;
-	gint port;
-} CallbackAndData;
 #endif
 
 #ifdef WITH_HOWL
@@ -53,8 +59,8 @@ typedef struct _CallbackAndData {
 #undef VERSION
 #include <howl.h>
 
-/* stupid hack cause howl sucks */
-static gchar *our_service_name = NULL;
+char *our_service_name; /* stupid hack cause howl sucks */
+
 
 static gboolean
 howl_in_cb (GIOChannel *io_channel,
@@ -87,7 +93,7 @@ setup_sw_discovery (sw_discovery discovery)
 }
 
 static sw_discovery
-get_sw_discovery ()
+get_sw_discovery (GError **error)
 {
 	sw_result result;
 	static sw_discovery discovery = NULL;
@@ -98,7 +104,10 @@ get_sw_discovery ()
 	
 		if (result != SW_OKAY) {
 			rb_debug ("Error initializing Howl");
-			mdns_error_dialog ("Howl");
+			g_set_error (error,
+				     RB_DAAP_MDNS_ERROR,
+				     RB_DAAP_MDNS_ERROR_FAILED,
+				     "Hown daemon was not running");
 			return NULL;
 		}
 
@@ -127,7 +136,12 @@ howl_strerror (sw_result result)
 
 	return "";
 }
-		
+
+typedef struct {
+	gpointer callback;
+	gpointer data;
+	sw_discovery_oid oid;
+} HowlBrowseData;
 		
 static sw_result
 browse_cb (sw_discovery discovery,
@@ -137,9 +151,8 @@ browse_cb (sw_discovery discovery,
 	   sw_const_string name,
 	   sw_const_string type,
 	   sw_const_string domain,
-	   sw_opaque extra)
+	   HowlBrowseData *browse_data)
 {
-	CallbackAndData *cd = (CallbackAndData *) extra;
 	RBDAAPmDNSBrowserStatus bstatus;
 
 	/* This sucks.  Howl sucks.  I can't wait til avahi gets used more
@@ -165,10 +178,10 @@ browse_cb (sw_discovery discovery,
 		return SW_OKAY;
 	}
 	
-	((RBDAAPmDNSBrowserCallback)cd->callback) ((RBDAAPmDNSBrowser) oid,
-						   bstatus,
-						   (const gchar *) name,
-						   cd->data);
+	((RBDAAPmDNSBrowserCallback)browse_data->callback) ((RBDAAPmDNSBrowser) browse_data,
+							    bstatus,
+							    (const gchar *) name,
+							    browse_data->data);
 	
 	return SW_OKAY;
 }
@@ -178,28 +191,34 @@ rb_daap_mdns_browse (RBDAAPmDNSBrowser *browser,
 		     RBDAAPmDNSBrowserCallback callback,
 		     gpointer user_data)
 {
-	sw_discovery discovery = get_sw_discovery ();
-	static CallbackAndData cd;
+	sw_discovery discovery;
+	HowlBrowseData *browse_data;
 	sw_result result;
+	GError *error = NULL;
 	
+	discovery = get_sw_discovery (&error);
 	if (!discovery) {
 		rb_debug ("Error initializing Howl for browsing");
-		mdns_error_dialog ("Howl");
+		if (!g_error_matches (error, RB_DAAP_MDNS_ERROR, RB_DAAP_MDNS_ERROR_NOT_RUNNING))
+			mdns_error_dialog ("Howl");
 		return FALSE;
 	}
-		
-	cd.callback = callback;
-	cd.data = user_data;
+
+	browse_data = g_new0 (HowlBrowseData, 1);
+	*browser = (RBDAAPmDNSBrowser) browse_data;
+	browse_data->callback = callback;
+	browse_data->data = user_data;
 		
        	result = sw_discovery_browse (discovery,
 				      0,
 				      "_daap._tcp", 
 				      "local", 
 				      (sw_discovery_browse_reply) browse_cb, 
-				      (sw_opaque) &cd, 
-				      (sw_discovery_oid *)browser);
+				      (sw_opaque) browse_data ,
+				      (sw_discovery_oid*) &browse_data->oid);
 
 	if (result != SW_OKAY) {
+		g_free (browse_data);
 		rb_debug ("Error starting mDNS browsing with Howl: %s", howl_strerror (result));
 		mdns_error_dialog ("Howl");
 		return FALSE;
@@ -211,17 +230,25 @@ rb_daap_mdns_browse (RBDAAPmDNSBrowser *browser,
 void
 rb_daap_mdns_browse_cancel (RBDAAPmDNSBrowser browser)
 {
+	HowlBrowseData *browse_data = (HowlBrowseData*) browser;
 	sw_discovery discovery;
 	
-	discovery = get_sw_discovery ();
+	discovery = get_sw_discovery (NULL);
 
 	if (discovery) {
-		sw_discovery_cancel (discovery, (sw_discovery_oid) browser);
+		sw_discovery_cancel (discovery, (sw_discovery_oid) browse_data->oid);
 	}
 
-	return;
+	g_free (browse_data);
 }
 
+typedef struct {
+	gpointer callback;
+	gpointer data;
+	sw_discovery_oid oid;
+} HowlResolveData;
+
+
 static sw_result
 resolve_cb (sw_discovery disc,
 	    sw_discovery_oid oid, 
@@ -233,10 +260,9 @@ resolve_cb (sw_discovery disc,
 	    sw_port port, 
 	    sw_octets text_record, 
 	    sw_ulong text_record_length, 
-	    sw_opaque extra)
+	    HowlResolveData *res_data)
 {
 	gchar *host = g_malloc (16);
-	CallbackAndData *cd = (CallbackAndData *) extra;
 	sw_text_record_iterator it;
 	gboolean pp = FALSE;
 	gchar *name = NULL;
@@ -255,21 +281,26 @@ resolve_cb (sw_discovery disc,
 				}
 			}
 			if (strcmp ((char *)key, "Machine Name") == 0) {
+				if (name != NULL)
+					g_free (name);
 				name = g_strdup ((char *)val);
 			}
 		}
 		
 		sw_text_record_iterator_fina (it);
 	}
-	
-	((RBDAAPmDNSResolverCallback)cd->callback) ((RBDAAPmDNSResolver) oid,
-						    RB_DAAP_MDNS_RESOLVER_FOUND,
-						    service_name,
-						    name,
-						    host,
-						    (guint) port,
-						    pp,
-						    cd->data);
+
+	if (name == NULL)
+		name = g_strdup (service_name);
+
+	((RBDAAPmDNSResolverCallback)res_data->callback) ((RBDAAPmDNSResolver) res_data,
+							  RB_DAAP_MDNS_RESOLVER_FOUND,
+							  service_name,
+							  name,
+							  host,
+							  (guint) port,
+							  pp,
+							  res_data->data);
 	g_free (host);
 	g_free (name);
 	
@@ -283,18 +314,23 @@ rb_daap_mdns_resolve (RBDAAPmDNSResolver *resolver,
 		      RBDAAPmDNSResolverCallback callback,
 		      gpointer user_data)
 {
-	sw_discovery discovery = get_sw_discovery ();
-	static CallbackAndData cd;
+	sw_discovery discovery;
+	HowlResolveData *res_data;
 	sw_result result;
+	GError *error = NULL;
 
+	discovery = get_sw_discovery (&error);
 	if (!discovery) {
 		rb_debug ("Error initializing Howl for resolving");
-		mdns_error_dialog ("Howl");
+		if (!g_error_matches (error, RB_DAAP_MDNS_ERROR, RB_DAAP_MDNS_ERROR_NOT_RUNNING))
+			mdns_error_dialog ("Howl");
 		return FALSE;
 	}
 
-	cd.callback = callback;
-	cd.data = user_data;
+	res_data = g_new0 (HowlResolveData, 1);
+	*resolver = (RBDAAPmDNSResolver)res_data;
+	res_data->callback = callback;
+	res_data->data = user_data;
 		
        	result = sw_discovery_resolve (discovery,
 				       0,
@@ -302,10 +338,11 @@ rb_daap_mdns_resolve (RBDAAPmDNSResolver *resolver,
 				       "_daap._tcp", 
 				       "local",
 				       (sw_discovery_resolve_reply) resolve_cb,
-	       			       (sw_opaque) &cd,
-				       (sw_discovery_oid *)resolver);
+	       			       (sw_opaque) res_data,
+				       (sw_discovery_oid*) &res_data->oid);
 		
 	if (result != SW_OKAY) {
+		g_free (res_data);
 		rb_debug ("Error starting mDNS resolving with Howl: %s", howl_strerror (result));
 		mdns_error_dialog ("Howl");
 		return FALSE;
@@ -317,33 +354,42 @@ rb_daap_mdns_resolve (RBDAAPmDNSResolver *resolver,
 void
 rb_daap_mdns_resolve_cancel (RBDAAPmDNSResolver resolver)
 {
-	sw_discovery discovery = get_sw_discovery ();
+	HowlResolveData *res_data = (HowlResolveData*) resolver;
+	sw_discovery discovery = get_sw_discovery (NULL);
 
 	if (discovery) {
-		sw_discovery_cancel (discovery, (sw_discovery_oid) resolver);
+		sw_discovery_cancel (discovery, (sw_discovery_oid) res_data->oid);
 	}
+
+	g_free (res_data);
 }
 
+
+typedef struct {
+	gpointer callback;
+	gpointer data;
+	gint port;
+	sw_discovery_oid oid;
+} HowlPublishData;
+
 static sw_result
 publish_cb (sw_discovery discovery,
 	    sw_discovery_oid oid,
 	    sw_discovery_publish_status status,
-	    sw_opaque extra)
+	    HowlPublishData *pub_data)
 {
-	CallbackAndData *cd = (CallbackAndData *) extra;
-
 	if (status == SW_DISCOVERY_PUBLISH_STARTED) {
-		((RBDAAPmDNSPublisherCallback)cd->callback) ((RBDAAPmDNSPublisher) oid,
-							     RB_DAAP_MDNS_PUBLISHER_STARTED,
-							     cd->data);
+		((RBDAAPmDNSPublisherCallback)pub_data->callback) ((RBDAAPmDNSPublisher) pub_data,
+								   RB_DAAP_MDNS_PUBLISHER_STARTED,
+								   pub_data->data);
 	} else if (status == SW_DISCOVERY_PUBLISH_NAME_COLLISION) {
 	/* This is all well and good, but howl won't report a name collision.
 	 * http://lists.porchdogsoft.com/pipermail/howl-users/Week-of-Mon-20041206/000487.html
 	 * So.  Fuck.
 	 */
-		((RBDAAPmDNSPublisherCallback)cd->callback) ((RBDAAPmDNSPublisher) oid,
-							     RB_DAAP_MDNS_PUBLISHER_COLLISION,
-							     cd->data);
+		((RBDAAPmDNSPublisherCallback)pub_data->callback) ((RBDAAPmDNSPublisher) pub_data,
+								   RB_DAAP_MDNS_PUBLISHER_COLLISION,
+								   pub_data->data);
 	}
 	
 	return SW_OKAY;
@@ -356,24 +402,26 @@ rb_daap_mdns_publish (RBDAAPmDNSPublisher *publisher,
 		      RBDAAPmDNSPublisherCallback callback,
 		      gpointer user_data)
 {
-	sw_discovery discovery = get_sw_discovery ();
-	static CallbackAndData cd;
+	sw_discovery discovery;
+	HowlPublishData *pub_data;
 	sw_result result;
+	GError *error = NULL;
 
+	discovery = get_sw_discovery (&error);
 	if (!discovery) {
 		rb_debug ("Error initializing Howl for resolving");
-		mdns_error_dialog ("Howl");
+		if (!g_error_matches (error, RB_DAAP_MDNS_ERROR, RB_DAAP_MDNS_ERROR_NOT_RUNNING))
+			mdns_error_dialog ("Howl");
 		return FALSE;
 	}
 
-	cd.callback = callback;
-	cd.port = port;
-	cd.data = user_data;
+	pub_data = g_new0 (HowlPublishData, 1);
+	*publisher = (RBDAAPmDNSPublisher)pub_data;
+	pub_data->callback = callback;
+	pub_data->port = port;
+	pub_data->data = user_data;
 
-	if (our_service_name) {
-		g_free (our_service_name);
-	}
-	
+	g_assert (our_service_name == NULL);
 	our_service_name = g_strdup (name);
 
 	result = sw_discovery_publish (discovery,
@@ -386,10 +434,11 @@ rb_daap_mdns_publish (RBDAAPmDNSPublisher *publisher,
 				       NULL,
 				       0,
 				       (sw_discovery_publish_reply) publish_cb,
-	       			       (sw_opaque) &cd,
-				       (sw_discovery_oid *)publisher);
+	       			       (sw_opaque) pub_data,
+				       (sw_discovery_oid*) &pub_data->oid);
 		
 	if (result != SW_OKAY) {
+		g_free (pub_data);
 		rb_debug ("Error starting mDNS pubilsh with Howl: %s", howl_strerror (result));
 		mdns_error_dialog ("Howl");
 		return FALSE;
@@ -401,28 +450,38 @@ rb_daap_mdns_publish (RBDAAPmDNSPublisher *publisher,
 void
 rb_daap_mdns_publish_cancel (RBDAAPmDNSPublisher publisher)
 {
+	HowlPublishData *pub_data = (HowlPublishData*) publisher;
 	sw_discovery discovery;
 	
-	discovery = get_sw_discovery ();
+	discovery = get_sw_discovery (NULL);
 
 	if (discovery) {
-		sw_discovery_cancel (discovery, (sw_discovery_oid) publisher);
+		sw_discovery_cancel (discovery, (sw_discovery_oid) pub_data->oid);
 	}
 
-	if (our_service_name) {
-		g_free (our_service_name);
-		our_service_name = NULL;
-	}
+	g_free (pub_data);
+	g_free (our_service_name);
 }
 #endif
 
 #ifdef WITH_AVAHI
 
+#ifdef HAVE_AVAHI_0_6
+#include <avahi-client/lookup.h>
+#include <avahi-client/publish.h>
+#endif
 #include <avahi-client/client.h>
 #include <avahi-common/error.h>
 #include <avahi-glib/glib-malloc.h>
 #include <avahi-glib/glib-watch.h>
 
+typedef struct {
+	AvahiServiceBrowser *sb;
+	RBDAAPmDNSBrowserCallback callback;
+	gpointer data;
+} RBDAAPmDNSBrowserData;
+
+
 static void
 client_cb (AvahiClient *client,
 	   AvahiClientState state,
@@ -434,7 +493,8 @@ client_cb (AvahiClient *client,
  */
 }
 
-static AvahiClient * get_avahi_client ()
+static AvahiClient *
+get_avahi_client (GError **err)
 {
 	static gboolean initialized = FALSE;
 	static AvahiClient *client = NULL;
@@ -449,12 +509,37 @@ static AvahiClient * get_avahi_client ()
 
 		if (!poll) {
 			rb_debug ("Unable to create AvahiGlibPoll object for mDNS");
+			g_set_error (err,
+				     RB_DAAP_MDNS_ERROR,
+				     RB_DAAP_MDNS_ERROR_FAILED,
+				     "Could not create AvahiGlibPoll object");
 			return NULL;
 		}
 
+#ifdef HAVE_AVAHI_0_5
 		client = avahi_client_new (avahi_glib_poll_get (poll), client_cb, NULL, &error);
+#endif
+#ifdef HAVE_AVAHI_0_6
+		{
+			AvahiClientFlags flags;
+			flags = AVAHI_CLIENT_NO_FAIL;
+
+			client = avahi_client_new (avahi_glib_poll_get (poll), flags, client_cb, NULL, &error);
+		}
+#endif
 		if (client == NULL) {
 			rb_debug ("Unable to create AvahiClient: %s", avahi_strerror (error));
+			if (error == AVAHI_ERR_NO_DAEMON) {
+				g_set_error (err,
+					     RB_DAAP_MDNS_ERROR,
+					     RB_DAAP_MDNS_ERROR_NOT_RUNNING,
+					     "Avahi daemon was noty running");
+			} else {
+				g_set_error (err,
+					     RB_DAAP_MDNS_ERROR,
+					     RB_DAAP_MDNS_ERROR_FAILED,
+					     "Error creating avahi client: %s", avahi_strerror (error));
+			}
 			return NULL;
 		}
 
@@ -472,12 +557,21 @@ browse_cb (AvahiServiceBrowser *browser,
 	   const char *name,
 	   const char *type,
 	   const char *domain,
-	   void *data)
+#ifdef HAVE_AVAHI_0_6
+	   AvahiLookupResultFlags flags,
+#endif
+	   RBDAAPmDNSBrowserData *browse_data)
 {
-	CallbackAndData *cd = (CallbackAndData *) data;
 	RBDAAPmDNSBrowserStatus bstatus;
+	gboolean local;
 
-	if (avahi_client_is_service_local (get_avahi_client (), interface, protocol, name, type, domain)) {
+#ifdef HAVE_AVAHI_0_5
+	local = avahi_client_is_service_local (get_avahi_client (NULL), interface, protocol, name, type, domain);
+#endif
+#ifdef HAVE_AVAHI_0_6
+	local = ((flags & AVAHI_LOOKUP_RESULT_LOCAL) != 0);
+#endif
+	if (local) {
 		rb_debug ("Ignoring local service %s", name);
 		return;
 	}
@@ -490,10 +584,10 @@ browse_cb (AvahiServiceBrowser *browser,
 		return;
 	}
 
-	((RBDAAPmDNSBrowserCallback)cd->callback) ((RBDAAPmDNSBrowser) browser,
-						   bstatus,
-						   (const gchar *) name,
-						   cd->data);
+	(browse_data->callback) ((RBDAAPmDNSBrowser) browse_data,
+				 bstatus,
+				 name,
+				 browse_data->data);
 }
 
 gboolean
@@ -502,26 +596,34 @@ rb_daap_mdns_browse (RBDAAPmDNSBrowser *browser,
 		     gpointer user_data)
 {
 	AvahiClient *client;
-	static CallbackAndData cd;
+	RBDAAPmDNSBrowserData *browse_data;
+	GError *error = NULL;
 	
-	client = get_avahi_client ();
+	client = get_avahi_client (&error);
 	if (!client) {
 		rb_debug ("Error initializing Avahi for browsing");
-		mdns_error_dialog ("Avahi");
+		if (!g_error_matches (error, RB_DAAP_MDNS_ERROR, RB_DAAP_MDNS_ERROR_NOT_RUNNING))
+			mdns_error_dialog ("Avahi");
 		return FALSE;
 	}
 
-	cd.callback = callback;
-	cd.data = user_data;
+	browse_data = g_new0(RBDAAPmDNSBrowserData, 1);
+	*browser = (RBDAAPmDNSBrowser)browse_data;
+	browse_data->callback = callback;
+	browse_data->data = user_data;
 	
-	*browser = (gpointer) avahi_service_browser_new (client,
-							 AVAHI_IF_UNSPEC,
-							 AVAHI_PROTO_INET,
-							 "_daap._tcp",
-							 "local",
-							 (AvahiServiceBrowserCallback)browse_cb,
-							 &cd);
-	if (*browser == NULL) {
+	browse_data->sb = avahi_service_browser_new (client,
+						     AVAHI_IF_UNSPEC,
+						     AVAHI_PROTO_INET,
+						     "_daap._tcp",
+						     "local",
+#ifdef HAVE_AVAHI_0_6
+						     0,
+#endif
+						     (AvahiServiceBrowserCallback)browse_cb,
+						     browse_data);
+	if (browse_data->sb == NULL) {
+		g_free (browse_data);
 		rb_debug ("Error starting mDNS discovery using AvahiServiceBrowser");
 		mdns_error_dialog ("Avahi");
 		return FALSE;
@@ -533,12 +635,20 @@ rb_daap_mdns_browse (RBDAAPmDNSBrowser *browser,
 void
 rb_daap_mdns_browse_cancel (RBDAAPmDNSBrowser browser)
 {
-	AvahiServiceBrowser *sb = (AvahiServiceBrowser *) browser;
+	RBDAAPmDNSBrowserData *browse_data = (RBDAAPmDNSBrowserData*)browser;
 
-	avahi_service_browser_free (sb);
-	browser = NULL;
+	avahi_service_browser_free (browse_data->sb);
+	g_free (browse_data);
 }
 
+
+
+typedef struct {
+	AvahiServiceResolver *sr;
+	RBDAAPmDNSResolverCallback callback;
+	gpointer data;
+} RBDAAPmDNSResolverData;
+
 static void
 resolve_cb (AvahiServiceResolver *resolver,
 	    AvahiIfIndex interface,
@@ -551,9 +661,11 @@ resolve_cb (AvahiServiceResolver *resolver,
 	    const AvahiAddress *address,
 	    uint16_t port,
 	    AvahiStringList *text,
-	    void *data)
+#ifdef HAVE_AVAHI_0_6
+	    AvahiLookupResultFlags flags,
+#endif
+	    RBDAAPmDNSResolverData *res_data)
 {
-	CallbackAndData *cd = (CallbackAndData *) data;
 
 	if (event == AVAHI_RESOLVER_FOUND) {
 		gchar *host;
@@ -602,26 +714,26 @@ resolve_cb (AvahiServiceResolver *resolver,
 		avahi_address_snprint (host, 16, address);
 
 
-		((RBDAAPmDNSResolverCallback)cd->callback) ((RBDAAPmDNSResolver) resolver,
-							    RB_DAAP_MDNS_RESOLVER_FOUND,
-							    service_name,
-							    name,
-							    host,
-							    (guint16) port,
-							    pp,
-							    cd->data);
+		(res_data->callback) ((RBDAAPmDNSResolver) res_data,
+				      RB_DAAP_MDNS_RESOLVER_FOUND,
+				      service_name,
+				      name,
+				      host,
+				      port,
+				      pp,
+				      res_data->data);
 
 		g_free (host);
 		g_free (name);
 	} else if (event == RB_DAAP_MDNS_RESOLVER_TIMEOUT) {
-		((RBDAAPmDNSResolverCallback)cd->callback) ((RBDAAPmDNSResolver) resolver,
-							    RB_DAAP_MDNS_RESOLVER_TIMEOUT,
-							    service_name,
-							    NULL,
-							    NULL,
-							    0,
-							    FALSE,
-							    cd->data);
+		(res_data->callback) ((RBDAAPmDNSResolver) res_data,
+				      RB_DAAP_MDNS_RESOLVER_TIMEOUT,
+				      service_name,
+				      NULL,
+				      NULL,
+				      0,
+				      FALSE,
+				      res_data->data);
 	}
 }
 
@@ -632,28 +744,36 @@ rb_daap_mdns_resolve (RBDAAPmDNSResolver *resolver,
 		      gpointer user_data)
 {
 	AvahiClient *client;
-	static CallbackAndData cd;
+	GError *error = NULL;
+	RBDAAPmDNSResolverData *res_data;
 	
-	client = get_avahi_client ();
+	client = get_avahi_client (&error);
 	if (!client) {
 		rb_debug ("Error initializing Avahi for resolving");
-		mdns_error_dialog ("Avahi");
+		if (!g_error_matches (error, RB_DAAP_MDNS_ERROR, RB_DAAP_MDNS_ERROR_NOT_RUNNING))
+			mdns_error_dialog ("Avahi");
 		return FALSE;
 	}
 
-	cd.callback = callback;
-	cd.data = user_data;
+	res_data = g_new0 (RBDAAPmDNSResolverData, 1);
+	*resolver = (RBDAAPmDNSResolver) res_data;
+	res_data->callback = callback;
+	res_data->data = user_data;
 	
-	*resolver = (gpointer) avahi_service_resolver_new (client,
-							   AVAHI_IF_UNSPEC,
-							   AVAHI_PROTO_INET,
-							   name,
-							   "_daap._tcp",
-							   "local",
-							   AVAHI_PROTO_UNSPEC,
-							   (AvahiServiceResolverCallback)resolve_cb,
-							   &cd);
-	if (!*resolver) {
+	res_data->sr = avahi_service_resolver_new (client,
+						   AVAHI_IF_UNSPEC,
+						   AVAHI_PROTO_INET,
+						   name,
+						   "_daap._tcp",
+						   "local",
+						   AVAHI_PROTO_UNSPEC,
+#ifdef HAVE_AVAHI_0_6
+						   AVAHI_LOOKUP_USE_MULTICAST,
+#endif
+						   (AvahiServiceResolverCallback)resolve_cb,
+						   res_data);
+	if (res_data->sr == NULL) {
+		g_free (res_data);
 		rb_debug ("Error starting mDNS resolving using AvahiServiceResolver");
 		mdns_error_dialog ("Avahi");
 		return FALSE;
@@ -665,22 +785,34 @@ rb_daap_mdns_resolve (RBDAAPmDNSResolver *resolver,
 void
 rb_daap_mdns_resolve_cancel (RBDAAPmDNSResolver resolver)
 {
-	AvahiServiceResolver *sr = (AvahiServiceResolver *) resolver;
+	RBDAAPmDNSResolverData *res_data = (RBDAAPmDNSResolverData*) resolver;
 
-	avahi_service_resolver_free (sr);
-	resolver = NULL;
+	avahi_service_resolver_free (res_data->sr);
+	g_free (res_data);
 }
 
+
+
+typedef struct {
+	AvahiEntryGroup *eg;
+	RBDAAPmDNSPublisherCallback callback;
+	gpointer data;
+	guint port;
+} RBDAAPmDNSPublisherData;
+
 static gint
 add_service (AvahiEntryGroup *group,
 	     const gchar *name,
-	     gint port)
+	     guint port)
 {
 	gint ret;
 	
 	ret = avahi_entry_group_add_service (group,
 					     AVAHI_IF_UNSPEC,
 					     AVAHI_PROTO_INET,
+#ifdef HAVE_AVAHI_0_6
+					     AVAHI_PUBLISH_USE_MULTICAST,
+#endif
 					     name,
 					     "_daap._tcp",
 					     NULL,
@@ -701,24 +833,22 @@ add_service (AvahiEntryGroup *group,
 static void
 entry_group_cb (AvahiEntryGroup *group,
 		AvahiEntryGroupState state,
-		void *data)
+		RBDAAPmDNSPublisherData *pub_data)
 {
-	CallbackAndData *cd = (CallbackAndData *)data;
-
 	if (state == AVAHI_ENTRY_GROUP_ESTABLISHED) {
-		((RBDAAPmDNSPublisherCallback)cd->callback) ((RBDAAPmDNSPublisher) group,
-							     RB_DAAP_MDNS_PUBLISHER_STARTED,
-							     cd->data);
+		(pub_data->callback) ((RBDAAPmDNSPublisher)pub_data,
+				      RB_DAAP_MDNS_PUBLISHER_STARTED,
+				      pub_data->data);
 	} else if (state == AVAHI_ENTRY_GROUP_COLLISION) {
 		gchar *new_name = NULL;
 		gint ret;
 		
-		new_name = ((RBDAAPmDNSPublisherCallback)cd->callback) (
-				(RBDAAPmDNSPublisher) group,
+		new_name = (pub_data->callback) (
+				(RBDAAPmDNSPublisher)pub_data,
 				RB_DAAP_MDNS_PUBLISHER_COLLISION, 
-				cd->data);
+				pub_data->data);
 		
-		ret = add_service (group, new_name, cd->port);
+		ret = add_service (group, new_name, pub_data->port);
 
 		g_free (new_name);
 
@@ -738,32 +868,38 @@ rb_daap_mdns_publish (RBDAAPmDNSPublisher *publisher,
 		      gpointer user_data)
 {
 	AvahiClient *client;
-	static CallbackAndData cd;
+	RBDAAPmDNSPublisherData *pub_data;
 	gint ret;
+	GError *error = NULL;
 	
-	client = get_avahi_client ();
+	client = get_avahi_client (&error);
 	if (!client) {
 		rb_debug ("Error initializing Avahi for publishing");
-		mdns_error_dialog ("Avahi");
+		if (!g_error_matches (error, RB_DAAP_MDNS_ERROR, RB_DAAP_MDNS_ERROR_NOT_RUNNING))
+			mdns_error_dialog ("Avahi");
 		return FALSE;
 	}
 
-	cd.callback = callback;
-	cd.port = port;
-	cd.data = user_data;
+	pub_data = g_new0 (RBDAAPmDNSPublisherData, 1);
+	*publisher = (RBDAAPmDNSPublisher) pub_data;
+	pub_data->callback = callback;
+	pub_data->port = port;
+	pub_data->data = user_data;
 	
-	*publisher = (RBDAAPmDNSPublisher) avahi_entry_group_new (client,
-						       entry_group_cb,
-						       &cd);
-	if (!*publisher) {
+	pub_data->eg = avahi_entry_group_new (client,
+					      (AvahiEntryGroupCallback)entry_group_cb,
+					      pub_data);
+	if (pub_data->eg == NULL) {
+		g_free (pub_data);
 		rb_debug ("Could not create AvahiEntryGroup for publishing");
 		mdns_error_dialog ("Avahi");
 		return FALSE;
 	}
 
-	ret = add_service ((AvahiEntryGroup *)*publisher, name, port);
+	ret = add_service (pub_data->eg, name, port);
 	
 	if (ret < 0) {
+		g_free (pub_data);
 		rb_debug ("Error adding service to AvahiEntryGroup: %s", avahi_strerror (ret));
 		mdns_error_dialog ("Avahi");
 		return FALSE;
@@ -775,9 +911,9 @@ rb_daap_mdns_publish (RBDAAPmDNSPublisher *publisher,
 void
 rb_daap_mdns_publish_cancel (RBDAAPmDNSPublisher publisher)
 {
-	AvahiEntryGroup *eg = (AvahiEntryGroup *)publisher;
+	RBDAAPmDNSPublisherData *pub_data = (RBDAAPmDNSPublisherData*)publisher;
 
-	avahi_entry_group_free (eg);
-	publisher = NULL;
+	avahi_entry_group_free (pub_data->eg);
+	g_free (pub_data);
 }
 #endif
