@@ -55,10 +55,23 @@ static void rb_daap_share_dispose	(GObject *object);
 
 static void rb_daap_share_start_publish (RBDAAPShare *share);
 static void rb_daap_share_stop_publish  (RBDAAPShare *share);
+static void rb_daap_share_playlist_created (RBPlaylistManager *mgr,
+					    RBSource *playlist,
+					    RBDAAPShare *share);
+static void rb_daap_share_process_playlist (RBSource *playlist,
+					    RBDAAPShare *share);
+static void rb_daap_share_playlist_destroyed (RBDAAPShare *share,
+					      RBSource *source);
+static void rb_daap_share_forget_playlist (gpointer data,
+					   RBDAAPShare *share);
+
 
 #define CONF_NAME CONF_PREFIX "/sharing/share_name"
 #define STANDARD_DAAP_PORT 3689
 
+/* HTTP chunk size used to send files to clients */
+#define DAAP_SHARE_CHUNK_SIZE	16384
+
 struct RBDAAPSharePrivate {
 	gchar *name;
 	guint port;
@@ -81,8 +94,16 @@ struct RBDAAPSharePrivate {
 
 	/* playlist things */
 	RBPlaylistManager *playlist_manager;
+	guint next_playlist_id;
+	GList *playlist_ids;	/* contains RBPlaylistIDs */
 };
 
+typedef struct {
+	RBSource *source;
+	gint32 id;
+} RBPlaylistID;
+
+
 enum {
 	PROP_0,
 	PROP_NAME,
@@ -144,6 +165,13 @@ rb_daap_share_set_property (GObject *object,
 	switch (prop_id) {
 	case PROP_NAME: {
 		gboolean restart_publish = FALSE;
+		const char *name = g_value_get_string (value);
+
+		/* check if the name hasn't really changed */
+		if (share->priv->name && name &&
+		    strcmp (name, share->priv->name) == 0) {
+			return;
+		}
 	
 		if (share->priv->name) {
 			g_free (share->priv->name);
@@ -154,7 +182,7 @@ rb_daap_share_set_property (GObject *object,
 			}
 		}
 
-		share->priv->name = g_value_dup_string (value);
+		share->priv->name = g_strdup (name);
 
 		if (restart_publish) {
 			rb_daap_share_start_publish (share);
@@ -166,7 +194,21 @@ rb_daap_share_set_property (GObject *object,
 		share->priv->db = g_value_get_object (value);
 		break;
 	case PROP_PLAYLIST_MANAGER:
-		share->priv->playlist_manager = g_value_get_object (value);
+		{
+			GList *playlists;
+			share->priv->playlist_manager = g_value_get_object (value);
+			g_signal_connect_object (G_OBJECT (share->priv->playlist_manager),
+						 "playlist_added",
+						 G_CALLBACK (rb_daap_share_playlist_created),
+						 share, 0);
+
+			/* Currently, there are no playlists when this object is created, but in
+			 * case it changes..
+			 */
+			playlists = rb_playlist_manager_get_playlists (share->priv->playlist_manager);
+			g_list_foreach (playlists, (GFunc) rb_daap_share_process_playlist, share);
+			g_list_free (playlists);
+		}
 		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -198,6 +240,77 @@ rb_daap_share_get_property (GObject *object,
 	}
 }
 
+static gint
+_find_by_id (gconstpointer a, gconstpointer b)
+{
+	RBPlaylistID *ai = (RBPlaylistID *)a;
+	gint bv = GPOINTER_TO_INT (b);
+	return (ai->id - bv);
+}
+
+static gint
+_find_by_source (gconstpointer a, gconstpointer b)
+{
+	RBPlaylistID *ai = (RBPlaylistID *)a;
+	RBSource *bs = (RBSource *)b;
+	return (ai->source - bs);
+}
+
+static void
+rb_daap_share_playlist_created (RBPlaylistManager *manager,
+				RBSource *source,
+				RBDAAPShare *share)
+{
+	rb_daap_share_process_playlist (source, share);
+}
+
+static void
+rb_daap_share_process_playlist (RBSource *source,
+				RBDAAPShare *share)
+{
+	RBPlaylistID *id;
+	
+	/* make sure we're not going insane.. */
+	g_assert (g_list_find_custom (share->priv->playlist_ids, 
+				      source, 
+				      _find_by_source) == NULL);
+
+	g_object_weak_ref (G_OBJECT (source),
+			   (GWeakNotify) rb_daap_share_playlist_destroyed,
+			   share);
+	id = g_new0 (RBPlaylistID, 1);
+	id->source = source;
+	id->id = share->priv->next_playlist_id++;
+	share->priv->playlist_ids = g_list_append (share->priv->playlist_ids, id);
+
+	/* if we knew how to send updates to clients, we'd probably do something here */
+}
+
+static void
+rb_daap_share_playlist_destroyed (RBDAAPShare *share,
+				  RBSource *source)
+{
+	GList *id;
+
+	id = g_list_find_custom (share->priv->playlist_ids, source, _find_by_source);
+	if (!id)
+		return;
+
+	share->priv->playlist_ids = g_list_remove_link (share->priv->playlist_ids, id);
+	g_free (id->data);
+	g_list_free_1 (id);
+}
+
+static void
+rb_daap_share_forget_playlist (gpointer data,
+			       RBDAAPShare *share)
+{
+	RBPlaylistID *id = (RBPlaylistID *)data;
+	g_object_weak_unref (G_OBJECT (id->source),
+			     (GWeakNotify) rb_daap_share_playlist_destroyed,
+			     share);
+}
+
 
 static void
 rb_daap_share_dispose (GObject *object)
@@ -212,6 +325,10 @@ rb_daap_share_dispose (GObject *object)
 		g_free (share->priv->name);
 		g_object_unref (share->priv->db);
 		g_object_unref (share->priv->playlist_manager);
+		
+		g_list_foreach (share->priv->playlist_ids, (GFunc) rb_daap_share_forget_playlist, share);
+		g_list_foreach (share->priv->playlist_ids, (GFunc) g_free, NULL);
+		
 		g_free (share->priv);
 		share->priv = NULL;
 	}
@@ -656,7 +773,7 @@ add_entry_to_mlcl (RhythmDBEntry *entry,
 }
 
 static void 
-add_playlist_to_mlcl (RBSource *source, 
+add_playlist_to_mlcl (RBPlaylistID *playlist_id, 
 		      GNode *mlcl)
 {
  /* 	MLIT listing item
@@ -670,14 +787,15 @@ add_playlist_to_mlcl (RBSource *source,
 	RBEntryView *ev;
 	guint num_songs;
 	
-	g_object_get (G_OBJECT (source), "name", &name, NULL);
+	g_object_get (G_OBJECT (playlist_id->source), "name", &name, NULL);
 	
-	ev = rb_source_get_entry_view (source);
+	ev = rb_source_get_entry_view (playlist_id->source);
 	num_songs = rb_entry_view_get_num_entries (ev);
 	
 	mlit = rb_daap_structure_add (mlcl, RB_DAAP_CC_MLIT);
-	rb_daap_structure_add (mlit, RB_DAAP_CC_MIID, (gint32) source);
-	rb_daap_structure_add (mlit, RB_DAAP_CC_MPER, (gint64)(gint32) source);
+	rb_daap_structure_add (mlit, RB_DAAP_CC_MIID, playlist_id->id);
+	/* we don't have a persistant ID for playlists, unfortunately */
+	rb_daap_structure_add (mlit, RB_DAAP_CC_MPER, (gint64) playlist_id->id);
 	rb_daap_structure_add (mlit, RB_DAAP_CC_MINM, name);
 	rb_daap_structure_add (mlit, RB_DAAP_CC_MIMC, (gint32) num_songs);
 	
@@ -753,6 +871,28 @@ parse_meta (const gchar *s)
 	return bits;
 }
 
+static void
+write_next_chunk (SoupMessage *message, GnomeVFSHandle *handle)
+{
+	GnomeVFSFileSize read_size;
+	GnomeVFSResult result;
+	gchar *chunk = g_malloc (DAAP_SHARE_CHUNK_SIZE);
+
+	result = gnome_vfs_read (handle, chunk, DAAP_SHARE_CHUNK_SIZE, &read_size);
+	if (result == GNOME_VFS_OK && read_size > 0) {
+		soup_message_add_chunk (message, SOUP_BUFFER_SYSTEM_OWNED, chunk, read_size);
+	} else {
+		g_free (chunk);
+		soup_message_add_final_chunk (message);
+	}
+}
+
+static void
+message_finished (SoupMessage *message, GnomeVFSHandle *handle)
+{
+	gnome_vfs_close (handle);
+}
+
 static void 
 databases_cb (RBDAAPShare *share, 
 	      SoupMessage *message)
@@ -846,7 +986,6 @@ databases_cb (RBDAAPShare *share,
 		GNode *aply;
 		GNode *mlcl;
 		GNode *mlit;
-		GList *playlists;
 
 		aply = rb_daap_structure_add (NULL, RB_DAAP_CC_APLY);
 		rb_daap_structure_add (aply, RB_DAAP_CC_MSTT, (gint32) DMAP_STATUS_OK);
@@ -861,9 +1000,7 @@ databases_cb (RBDAAPShare *share,
 		rb_daap_structure_add (mlit, RB_DAAP_CC_MIMC, (gint32) share->priv->num_songs);
 		rb_daap_structure_add (mlit, RB_DAAP_CC_ABPL, (gchar) 1); /* base playlist */
 
-		playlists = rb_playlist_manager_get_playlists (share->priv->playlist_manager);
-		g_list_foreach (playlists, (GFunc)add_playlist_to_mlcl, mlcl);
-		g_list_free (playlists);
+		g_list_foreach (share->priv->playlist_ids, (GFunc) add_playlist_to_mlcl, mlcl);
 	
 		message_set_from_rb_daap_structure (message, aply);
 		rb_daap_structure_destroy (aply);
@@ -898,25 +1035,37 @@ databases_cb (RBDAAPShare *share,
 		
 			g_hash_table_foreach (share->priv->entry_to_id, (GHFunc) add_entry_to_mlcl, &mb);
 		} else {
-			RBSource *source = GINT_TO_POINTER (pl_id);
+			RBPlaylistID *id;
+			GList *idl;
 			RBEntryView *ev;
 			guint num_songs;
 			RhythmDBQueryModel *model;
 
+			idl = g_list_find_custom (share->priv->playlist_ids, 
+						  GINT_TO_POINTER (pl_id), 
+						  _find_by_id);
+			if (idl == NULL) {
+				soup_message_set_status (message, SOUP_STATUS_NOT_FOUND);
+				soup_server_message_set_encoding (SOUP_SERVER_MESSAGE (message), 
+								  SOUP_TRANSFER_CONTENT_LENGTH);
+				soup_message_set_response (message, "text/plain", SOUP_BUFFER_USER_OWNED, "", 0);
+				goto out;
+			}
+			id = (RBPlaylistID *)idl->data;
+
 			mb.mlcl = rb_daap_structure_add (apso, RB_DAAP_CC_MLCL);
 
 			mb.pointer = share->priv->entry_to_id;
 			
-			ev = rb_source_get_entry_view (source);
+			ev = rb_source_get_entry_view (id->source);
 			num_songs = rb_entry_view_get_num_entries (ev);
 				
 			rb_daap_structure_add (apso, RB_DAAP_CC_MTCO, (gint32) num_songs);
 			rb_daap_structure_add (apso, RB_DAAP_CC_MRCO, (gint32) num_songs);
 
-			g_object_get (G_OBJECT (source), "query-model", &model, NULL);
+			g_object_get (G_OBJECT (id->source), "query-model", &model, NULL);
 			gtk_tree_model_foreach (GTK_TREE_MODEL (model), (GtkTreeModelForeachFunc) add_playlist_entry_to_mlcl, &mb);
 			g_object_unref (model);
-			
 		}
 		
 		message_set_from_rb_daap_structure (message, apso);
@@ -931,7 +1080,6 @@ databases_cb (RBDAAPShare *share,
 		GnomeVFSResult result;
 		GnomeVFSHandle *handle;
 		const gchar *range_header;
-		gchar *buf;
 		guint status_code = SOUP_STATUS_OK;
 		
 		id_str = rest_of_path + 9;
@@ -972,35 +1120,13 @@ databases_cb (RBDAAPShare *share,
 
 			file_size -= range;
 		}
-		
-		/* FIXME FIXME FIXME
-		 * Ideally, it seems that an mmap type solution should be used
-		 * here.  However, this works for now.
-		 */
-		buf = g_try_malloc (file_size);
-		if (buf == NULL) {
-			g_warning ("Unable to malloc %"G_GUINT64_FORMAT" bytes to transfer file", file_size);
-			soup_message_set_status (message, SOUP_STATUS_INTERNAL_SERVER_ERROR);
-			goto out;
-		}
 
-		result = gnome_vfs_read (handle, buf, file_size, NULL);
-		if (result != GNOME_VFS_OK) {
-			soup_message_set_status (message, SOUP_STATUS_INTERNAL_SERVER_ERROR);
-			g_free (buf);
-			goto out;
-		}
-		
-		message_add_standard_headers (message);
-		soup_message_add_header (message->response_headers, "Accept-Ranges", "bytes");
-		message->response.owner = SOUP_BUFFER_SYSTEM_OWNED;
-		message->response.length = file_size;
-		message->response.body = buf;
-
-		gnome_vfs_close (handle);
-	
+		g_signal_connect (message, "wrote_chunk", G_CALLBACK (write_next_chunk), handle);
+		g_signal_connect (message, "finished", G_CALLBACK (message_finished), handle);
+		write_next_chunk (message, handle);
+		 
 		soup_message_set_status (message, status_code);
-		soup_server_message_set_encoding (SOUP_SERVER_MESSAGE (message), SOUP_TRANSFER_CONTENT_LENGTH);
+		soup_server_message_set_encoding (SOUP_SERVER_MESSAGE (message), SOUP_TRANSFER_CHUNKED);
 
 	} else {
 		g_print ("unhandled: %s\n", path);
@@ -1068,8 +1194,9 @@ db_entry_added_cb (RhythmDB *db,
 		   RBDAAPShare *share)
 {
 	RhythmDBEntryType type = rhythmdb_entry_get_ulong (entry, RHYTHMDB_PROP_TYPE);
+	gboolean hidden = rhythmdb_entry_get_boolean (entry, RHYTHMDB_PROP_HIDDEN);
 
-	if (type == rhythmdb_entry_song_get_type ()) {
+	if (type == rhythmdb_entry_song_get_type () && !hidden) {
 		share->priv->num_songs++;
 
 		g_hash_table_insert (share->priv->id_to_entry, GINT_TO_POINTER (share->priv->num_songs), entry);
@@ -1082,14 +1209,11 @@ db_entry_deleted_cb (RhythmDB *db,
 		     RhythmDBEntry *entry,
 		     RBDAAPShare *share)
 {
-	RhythmDBEntryType type = rhythmdb_entry_get_ulong (entry, RHYTHMDB_PROP_TYPE);
+	gpointer id;
 
-	if (type == rhythmdb_entry_song_get_type ()) {
-		gpointer id;
-
-		id = g_hash_table_lookup (share->priv->entry_to_id, entry);
+	id = g_hash_table_lookup (share->priv->entry_to_id, entry);
+	if (id) {
 		g_hash_table_remove (share->priv->entry_to_id, entry);
-		g_hash_table_remove (share->priv->id_to_entry, id);
 
 		share->priv->num_songs--;
 	}
@@ -1164,6 +1288,7 @@ rb_daap_share_start_publish (RBDAAPShare *share)
 	share->priv->id_to_entry = g_hash_table_new (NULL, NULL);
 	share->priv->entry_to_id = g_hash_table_new (NULL, NULL);
 	share->priv->num_songs = 0;
+	share->priv->next_playlist_id = 2;		/* 1 already used */
 
 	rhythmdb_entry_foreach (share->priv->db, (GFunc)add_db_entry, share);
 
