@@ -71,10 +71,17 @@ static void rb_station_properties_dialog_update_play_count (RBStationPropertiesD
 static void rb_station_properties_dialog_update_bitrate (RBStationPropertiesDialog *dialog);
 static void rb_station_properties_dialog_update_last_played (RBStationPropertiesDialog *dialog);
 static void rb_station_properties_dialog_update_rating (RBStationPropertiesDialog *dialog);
+static void rb_station_properties_dialog_update_playback_error (RBStationPropertiesDialog *dialog);
 static void rb_station_properties_dialog_rated_cb (RBRating *rating,
 						   double score,
 						   RBStationPropertiesDialog *dialog);
 static void rb_station_properties_dialog_sync_entries (RBStationPropertiesDialog *dialog);
+static GtkWidget * boldify_label (GtkWidget *label);
+static void rb_station_properties_dialog_show (GtkWidget *widget);
+static void rb_station_properties_dialog_create_station (RBStationPropertiesDialog *dialog);
+static void rb_station_properties_dialog_location_changed_cb (GtkEntry *entry,
+							      RBStationPropertiesDialog *dialog);
+						  
 
 struct RBStationPropertiesDialogPrivate
 {
@@ -89,6 +96,8 @@ struct RBStationPropertiesDialogPrivate
 	GtkWidget   *playcount;
 	GtkWidget   *bitrate;
 	GtkWidget   *rating;
+	GtkWidget   *playback_error;
+	GtkWidget   *playback_error_box;
 	GtkWidget   *okbutton;
 	GtkWidget   *cancelbutton;
 };
@@ -96,50 +105,24 @@ struct RBStationPropertiesDialogPrivate
 enum 
 {
 	PROP_0,
-	PROP_ENTRY_VIEW,
-	PROP_BACKEND
+	PROP_ENTRY_VIEW
 };
 
-static GObjectClass *parent_class = NULL;
-
-GType
-rb_station_properties_dialog_get_type (void)
-{
-	static GType rb_station_properties_dialog_type = 0;
-
-	if (rb_station_properties_dialog_type == 0)
-	{
-		static const GTypeInfo our_info =
-		{
-			sizeof (RBStationPropertiesDialogClass),
-			NULL,
-			NULL,
-			(GClassInitFunc) rb_station_properties_dialog_class_init,
-			NULL,
-			NULL,
-			sizeof (RBStationPropertiesDialog),
-			0,
-			(GInstanceInitFunc) rb_station_properties_dialog_init
-		};
-		
-		rb_station_properties_dialog_type = g_type_register_static (GTK_TYPE_DIALOG,
-									    "RBStationPropertiesDialog",
-									    &our_info, 0);
-	}
-
-	return rb_station_properties_dialog_type;
-}
+G_DEFINE_TYPE (RBStationPropertiesDialog, 
+	       rb_station_properties_dialog, 
+	       GTK_TYPE_DIALOG)
 
 static void
 rb_station_properties_dialog_class_init (RBStationPropertiesDialogClass *klass)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
-
-	parent_class = g_type_class_peek_parent (klass);
+	GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (klass);
 
 	object_class->set_property = rb_station_properties_dialog_set_property;
 	object_class->get_property = rb_station_properties_dialog_get_property;
 
+	widget_class->show = rb_station_properties_dialog_show;
+
 	g_object_class_install_property (object_class,
 					 PROP_ENTRY_VIEW,
 					 g_param_spec_object ("entry-view",
@@ -169,7 +152,6 @@ rb_station_properties_dialog_init (RBStationPropertiesDialog *dialog)
 
 	gtk_dialog_set_default_response (GTK_DIALOG (dialog),
 					 GTK_RESPONSE_OK);
-	gtk_window_set_title (GTK_WINDOW (dialog), _("%s Properties"));
 
 	xml = rb_glade_xml_new ("station-properties.glade",
 				"stationproperties",
@@ -194,6 +176,21 @@ rb_station_properties_dialog_init (RBStationPropertiesDialog *dialog)
 	dialog->priv->lastplayed = glade_xml_get_widget (xml, "lastplayedLabel");
 	dialog->priv->playcount = glade_xml_get_widget (xml, "playcountLabel");
 	dialog->priv->bitrate = glade_xml_get_widget (xml, "bitrateLabel");
+	dialog->priv->playback_error = glade_xml_get_widget (xml, "errorLabel");
+	dialog->priv->playback_error_box = glade_xml_get_widget (xml, "errorBox");
+
+	boldify_label (glade_xml_get_widget (xml, "titleLabel"));
+	boldify_label (glade_xml_get_widget (xml, "genreLabel"));
+	boldify_label (glade_xml_get_widget (xml, "locationLabel"));
+	boldify_label (glade_xml_get_widget (xml, "ratingLabel"));
+	boldify_label (glade_xml_get_widget (xml, "lastplayedDescLabel"));
+	boldify_label (glade_xml_get_widget (xml, "playcountDescLabel"));
+	boldify_label (glade_xml_get_widget (xml, "bitrateDescLabel"));
+
+	g_signal_connect_object (G_OBJECT (dialog->priv->location),
+				 "changed",
+				 G_CALLBACK (rb_station_properties_dialog_location_changed_cb),
+				 dialog, 0);
 
 	dialog->priv->rating = GTK_WIDGET (rb_rating_new ());
 	g_signal_connect_object (dialog->priv->rating, 
@@ -219,7 +216,7 @@ rb_station_properties_dialog_finalize (GObject *object)
 
 	g_free (dialog->priv);
 
-	G_OBJECT_CLASS (parent_class)->finalize (object);
+	G_OBJECT_CLASS (rb_station_properties_dialog_parent_class)->finalize (object);
 }
 
 static void
@@ -263,24 +260,38 @@ rb_station_properties_dialog_get_property (GObject *object,
 }
 
 GtkWidget *
-rb_station_properties_dialog_new (RBEntryView *entry_view)
+rb_station_properties_dialog_new (RBEntryView *entry_view, gboolean new_station)
 {
 	RBStationPropertiesDialog *dialog;
 
 	g_return_val_if_fail (RB_IS_ENTRY_VIEW (entry_view), NULL);
 
 	dialog = g_object_new (RB_TYPE_STATION_PROPERTIES_DIALOG,
-			       "entry-view", entry_view, NULL);
+			       "entry-view", entry_view, 
+			       NULL);
 
-	if (!rb_station_properties_dialog_get_current_entry (dialog)) {
-		g_object_unref (G_OBJECT (dialog));
-		return NULL;
+	if (!new_station) {
+		if (!rb_station_properties_dialog_get_current_entry (dialog)) {
+			g_object_unref (G_OBJECT (dialog));
+			return NULL;
+		}
 	}
 	rb_station_properties_dialog_update (dialog);
 
 	return GTK_WIDGET (dialog);
 }
 
+static GtkWidget *
+boldify_label (GtkWidget *label)
+{
+	gchar *str_final;
+	str_final = g_strdup_printf ("<b>%s</b>",
+				     gtk_label_get_label (GTK_LABEL (label)));
+	gtk_label_set_markup_with_mnemonic (GTK_LABEL (label), str_final);
+	g_free (str_final);
+	return label;
+}
+
 static void
 rb_station_properties_dialog_response_cb (GtkDialog *gtkdialog,
 					  int response_id,
@@ -288,7 +299,12 @@ rb_station_properties_dialog_response_cb (GtkDialog *gtkdialog,
 {
 	if (response_id != GTK_RESPONSE_OK)
 		goto cleanup;
-	rb_station_properties_dialog_sync_entries (dialog);
+
+	if (dialog->priv->current_entry)
+		rb_station_properties_dialog_sync_entries (dialog);
+	else
+		rb_station_properties_dialog_create_station (dialog);
+	
 cleanup:
 	gtk_widget_destroy (GTK_WIDGET (dialog));
 }
@@ -314,72 +330,58 @@ rb_station_properties_dialog_get_current_entry (RBStationPropertiesDialog *dialo
 static void
 rb_station_properties_dialog_update (RBStationPropertiesDialog *dialog)
 {
-	g_return_if_fail (dialog->priv->current_entry != NULL);
-	rb_station_properties_dialog_update_location (dialog);
 	rb_station_properties_dialog_update_title (dialog);
-	rb_station_properties_dialog_update_title_entry (dialog);
-	rb_station_properties_dialog_update_genre (dialog);
+
+	if (dialog->priv->current_entry) {
+		rb_station_properties_dialog_update_location (dialog);
+		rb_station_properties_dialog_update_title_entry (dialog);
+		rb_station_properties_dialog_update_genre (dialog);
+	}
+
 	rb_station_properties_dialog_update_play_count (dialog);
 	rb_station_properties_dialog_update_bitrate (dialog);
 	rb_station_properties_dialog_update_last_played (dialog);
 	rb_station_properties_dialog_update_rating (dialog);
+	rb_station_properties_dialog_location_changed_cb (GTK_ENTRY (dialog->priv->location), dialog);
 }
 
 static void
 rb_station_properties_dialog_update_title (RBStationPropertiesDialog *dialog)
 {
 	const char *name;
-	char *tmp;	
-	rhythmdb_read_lock (dialog->priv->db);
-	name = rhythmdb_entry_get_string (dialog->priv->db,
-					  dialog->priv->current_entry,
-					  RHYTHMDB_PROP_TITLE);
-	rhythmdb_read_unlock (dialog->priv->db);
-	tmp = g_strdup_printf (_("Properties for %s"), name);
-	gtk_window_set_title (GTK_WINDOW (dialog), tmp);
-	g_free (tmp);
+	char *tmp;
+
+	if (dialog->priv->current_entry) {
+		name = rb_refstring_get (dialog->priv->current_entry->title);
+		tmp = g_strdup_printf (_("Properties for %s"), name);
+		gtk_window_set_title (GTK_WINDOW (dialog), tmp);
+		g_free (tmp);
+	} else {
+		gtk_window_set_title (GTK_WINDOW (dialog), _("New Internet Radio Station"));
+	}
 }
 
 static void
 rb_station_properties_dialog_update_title_entry (RBStationPropertiesDialog *dialog)
 {
-	const char *tmp;
-
-	rhythmdb_read_lock (dialog->priv->db);
-	tmp = rhythmdb_entry_get_string (dialog->priv->db,
-					 dialog->priv->current_entry,
-					 RHYTHMDB_PROP_TITLE);
-	rhythmdb_read_unlock (dialog->priv->db);
-	
-	gtk_entry_set_text (GTK_ENTRY (dialog->priv->title), tmp);
+	gtk_entry_set_text (GTK_ENTRY (dialog->priv->title),
+			    rb_refstring_get (dialog->priv->current_entry->title));
 }
 
 static void
 rb_station_properties_dialog_update_genre (RBStationPropertiesDialog *dialog)
 {
-	const char *tmp;
-
-	rhythmdb_read_lock (dialog->priv->db);
-	tmp = rhythmdb_entry_get_string (dialog->priv->db,
-					 dialog->priv->current_entry,
-					 RHYTHMDB_PROP_GENRE);
-	rhythmdb_read_unlock (dialog->priv->db);
-	
-	gtk_entry_set_text (GTK_ENTRY (dialog->priv->genre), tmp);
+	gtk_entry_set_text (GTK_ENTRY (dialog->priv->genre),
+			    rb_refstring_get (dialog->priv->current_entry->genre));
 }
 
 static void
 rb_station_properties_dialog_update_location (RBStationPropertiesDialog *dialog)
 {
-	const char *tmp;
-
-	rhythmdb_read_lock (dialog->priv->db);
-	tmp = rhythmdb_entry_get_string (dialog->priv->db,
-					 dialog->priv->current_entry,
-					 RHYTHMDB_PROP_LOCATION);
-	rhythmdb_read_unlock (dialog->priv->db);
-	
-	gtk_entry_set_text (GTK_ENTRY (dialog->priv->location), tmp);
+	char *unescaped;
+	unescaped = gnome_vfs_unescape_string_for_display (dialog->priv->current_entry->location);
+	gtk_entry_set_text (GTK_ENTRY (dialog->priv->location), unescaped);
+	g_free (unescaped);
 }
 
 static void
@@ -387,40 +389,39 @@ rb_station_properties_dialog_rated_cb (RBRating *rating,
 				       double score,
 				       RBStationPropertiesDialog *dialog)
 {
-	GValue value = { 0, };
-
+	GValue value = {0, };
+	
 	g_return_if_fail (RB_IS_RATING (rating));
 	g_return_if_fail (RB_IS_STATION_PROPERTIES_DIALOG (dialog));
 	g_return_if_fail (score >= 0 && score <= 5 );
 
-	/* set the new value for the song */
+	if (!dialog->priv->current_entry)
+		return;
+
 	g_value_init (&value, G_TYPE_DOUBLE);
 	g_value_set_double (&value, score);
-	rhythmdb_write_lock (dialog->priv->db);
+
+	/* set the new value for the song */
 	rhythmdb_entry_set (dialog->priv->db,
 			    dialog->priv->current_entry,
 			    RHYTHMDB_PROP_RATING,
 			    &value);
-	rhythmdb_write_unlock (dialog->priv->db);
 	g_value_unset (&value);
+	rhythmdb_commit (dialog->priv->db);
 
-	g_object_set (G_OBJECT (dialog->priv->rating),
-		      "score", score,
-		      NULL);
+	g_object_set (G_OBJECT (dialog->priv->rating), "rating", score, NULL);
 }
 
 static void
 rb_station_properties_dialog_update_play_count (RBStationPropertiesDialog *dialog)
 {
-	int val;
 	char *text;
+	long int count = 0;
 
-	rhythmdb_read_lock (dialog->priv->db);
-	val = rhythmdb_entry_get_int (dialog->priv->db,
-				      dialog->priv->current_entry,
-				      RHYTHMDB_PROP_PLAY_COUNT);
-	rhythmdb_read_unlock (dialog->priv->db);
-	text = g_strdup_printf ("%d", val);
+	if (dialog->priv->current_entry)
+		count = dialog->priv->current_entry->play_count;
+
+	text = g_strdup_printf ("%ld", count);
 	gtk_label_set_text (GTK_LABEL (dialog->priv->playcount), text);
 	g_free (text);
 }
@@ -428,18 +429,16 @@ rb_station_properties_dialog_update_play_count (RBStationPropertiesDialog *dialo
 static void
 rb_station_properties_dialog_update_bitrate (RBStationPropertiesDialog *dialog)
 {
-	int val;
+	gulong val = 0;
 	char *text;
 
-	rhythmdb_read_lock (dialog->priv->db);
-	val = rhythmdb_entry_get_int (dialog->priv->db,
-				      dialog->priv->current_entry,
-				      RHYTHMDB_PROP_BITRATE);
-	rhythmdb_read_unlock (dialog->priv->db);
+	if (dialog->priv->current_entry)
+		val = dialog->priv->current_entry->bitrate;
+
 	if (val == 0)
 		text = g_strdup (_("Unknown"));
 	else
-		text = g_strdup_printf ("%d", val);
+		text = g_strdup_printf (_("%lu kbps"), val);
 
 	gtk_label_set_text (GTK_LABEL (dialog->priv->bitrate), text);
 	g_free (text);
@@ -448,32 +447,38 @@ rb_station_properties_dialog_update_bitrate (RBStationPropertiesDialog *dialog)
 static void
 rb_station_properties_dialog_update_last_played (RBStationPropertiesDialog *dialog)
 {
-	const char *tmp;
+	const char *last_played = _("Never");
 
-	rhythmdb_read_lock (dialog->priv->db);
-	tmp = rhythmdb_entry_get_string (dialog->priv->db,
-					 dialog->priv->current_entry,
-					 RHYTHMDB_PROP_LAST_PLAYED_STR);
-	rhythmdb_read_unlock (dialog->priv->db);
-	
-	gtk_label_set (GTK_LABEL (dialog->priv->lastplayed), tmp);
+	if (dialog->priv->current_entry)
+		last_played = rb_refstring_get (dialog->priv->current_entry->last_played_str);
+	gtk_label_set (GTK_LABEL (dialog->priv->lastplayed), last_played);
 }
 
 static void
 rb_station_properties_dialog_update_rating (RBStationPropertiesDialog *dialog)
 {
-	double rating;
-
+	gdouble rating = 0.0;
 	g_return_if_fail (RB_IS_STATION_PROPERTIES_DIALOG (dialog));
 
-	rhythmdb_read_lock (dialog->priv->db);
-	rating = rhythmdb_entry_get_double (dialog->priv->db,
-					    dialog->priv->current_entry,
-					    RHYTHMDB_PROP_RATING);
-	rhythmdb_read_unlock (dialog->priv->db);
+	if (dialog->priv->current_entry)
+		rating = dialog->priv->current_entry->rating;
+	
+	g_object_set (G_OBJECT (dialog->priv->rating), "rating", rating, NULL);
+}
 
-	g_object_set (G_OBJECT (dialog->priv->rating),
-		      "score", rating, NULL);
+static void
+rb_station_properties_dialog_update_playback_error (RBStationPropertiesDialog *dialog)
+{
+	g_return_if_fail (RB_IS_STATION_PROPERTIES_DIALOG (dialog));
+
+	if (dialog->priv->current_entry && dialog->priv->current_entry->playback_error) {
+		gtk_label_set_text (GTK_LABEL (dialog->priv->playback_error),
+				    dialog->priv->current_entry->playback_error);
+		gtk_widget_show (dialog->priv->playback_error_box);
+	} else {
+		gtk_label_set_text (GTK_LABEL (dialog->priv->playback_error), "");
+		gtk_widget_hide (dialog->priv->playback_error_box);
+	}
 }
 
 static void
@@ -483,24 +488,98 @@ rb_station_properties_dialog_sync_entries (RBStationPropertiesDialog *dialog)
 	const char *genre = gtk_entry_get_text (GTK_ENTRY (dialog->priv->genre));
 	const char *location = gtk_entry_get_text (GTK_ENTRY (dialog->priv->location));
 	GValue val = {0,};
+	gboolean changed = FALSE;
+	RhythmDBEntry *entry = dialog->priv->current_entry;
 
-	rhythmdb_write_lock (dialog->priv->db);
-	g_value_init (&val, G_TYPE_STRING);
-	g_value_set_string (&val, title);
-	rhythmdb_entry_set (dialog->priv->db,
-			    dialog->priv->current_entry, RHYTHMDB_PROP_TITLE, &val);
-	g_value_unset (&val);
+	if (strcmp (title, rb_refstring_get (entry->title))) {
+		g_value_init (&val, G_TYPE_STRING);
+		g_value_set_string (&val, title);
+		rhythmdb_entry_set (dialog->priv->db, entry, 
+				    RHYTHMDB_PROP_TITLE,
+				    &val);
+		g_value_unset (&val);
+		changed = TRUE;
+	}
 
-	g_value_init (&val, G_TYPE_STRING);
-	g_value_set_string (&val, genre);
-	rhythmdb_entry_set (dialog->priv->db,
-			    dialog->priv->current_entry, RHYTHMDB_PROP_GENRE, &val);
-	g_value_unset (&val);
+	if (strcmp (genre, rb_refstring_get (entry->genre))) {
+		g_value_init (&val, G_TYPE_STRING);
+		g_value_set_string (&val, genre);
+		rhythmdb_entry_set (dialog->priv->db, entry, 
+				    RHYTHMDB_PROP_GENRE, &val);
+		g_value_unset (&val);
+		changed = TRUE;
+	}
 
-	g_value_init (&val, G_TYPE_STRING);
-	g_value_set_string (&val, location);
-	rhythmdb_entry_set (dialog->priv->db,
-			    dialog->priv->current_entry, RHYTHMDB_PROP_LOCATION, &val);
-	g_value_unset (&val);
-	rhythmdb_write_unlock (dialog->priv->db);
+	if (strcmp (location, entry->location)) {
+		g_value_init (&val, G_TYPE_STRING);
+		g_value_set_string (&val, location);
+		rhythmdb_entry_set (dialog->priv->db, entry,
+				    RHYTHMDB_PROP_LOCATION, &val);
+		g_value_unset (&val);
+		changed = TRUE;
+	}
+
+	if (changed)
+		rhythmdb_commit (dialog->priv->db);
+}
+
+static void
+rb_station_properties_dialog_create_station (RBStationPropertiesDialog *dialog)
+{
+	GValue title_val = { 0, };
+	GValue genre_val = { 0, };
+	GValue rating_val = { 0, };
+	RhythmDBEntry *entry;
+	gdouble rating;
+	const char *location = gtk_entry_get_text (GTK_ENTRY (dialog->priv->location));
+	char *trimmed_location = g_strstrip (g_strdup (location));
+
+	g_value_init (&title_val, G_TYPE_STRING);
+	g_value_set_static_string (&title_val,
+				   gtk_entry_get_text (GTK_ENTRY (dialog->priv->title)));
+	g_value_init (&genre_val, G_TYPE_STRING);
+	g_value_set_static_string (&genre_val,
+				   gtk_entry_get_text (GTK_ENTRY (dialog->priv->genre)));
+	g_object_get (G_OBJECT (dialog->priv->rating), "rating", &rating, NULL);
+	g_value_init (&rating_val, G_TYPE_DOUBLE);
+	g_value_set_double (&rating_val, rating);
+
+	entry = rhythmdb_entry_lookup_by_location (dialog->priv->db, trimmed_location);
+	if (entry) {
+		g_free (trimmed_location);
+		return;
+	}
+
+	entry = rhythmdb_entry_new (dialog->priv->db,
+				    RHYTHMDB_ENTRY_TYPE_IRADIO_STATION,
+				    trimmed_location);
+
+	rhythmdb_entry_set_uninserted (dialog->priv->db, entry, RHYTHMDB_PROP_TITLE, &title_val);
+	g_value_unset (&title_val);
+	rhythmdb_entry_set_uninserted (dialog->priv->db, entry, RHYTHMDB_PROP_GENRE, &genre_val);
+	g_value_unset (&genre_val);
+	rhythmdb_entry_set_uninserted (dialog->priv->db, entry, RHYTHMDB_PROP_RATING, &rating_val);
+	g_value_unset (&rating_val);
+	rhythmdb_commit (dialog->priv->db);
+
+	g_free (trimmed_location);
+}
+
+static void
+rb_station_properties_dialog_show (GtkWidget *widget)
+{
+	if (GTK_WIDGET_CLASS (rb_station_properties_dialog_parent_class)->show)
+		GTK_WIDGET_CLASS (rb_station_properties_dialog_parent_class)->show (widget);
+
+	rb_station_properties_dialog_update_playback_error (
+			RB_STATION_PROPERTIES_DIALOG (widget));
+}
+
+static void
+rb_station_properties_dialog_location_changed_cb (GtkEntry *entry,
+						  RBStationPropertiesDialog *dialog)
+{
+	/* only enable OK button if there's a location */
+	gtk_widget_set_sensitive (dialog->priv->okbutton,
+				  g_utf8_strlen (gtk_entry_get_text (entry), -1) > 0);
 }
