@@ -28,10 +28,17 @@
 #include "rhythmdb-property-model.h"
 #include "rb-debug.h"
 #include "gsequence.h"
+#include "rb-tree-dnd.h"
 
-static void rhythmdb_property_model_class_init (RhythmDBPropertyModelClass *klass);
 static void rhythmdb_property_model_tree_model_init (GtkTreeModelIface *iface);
-static void rhythmdb_property_model_init (RhythmDBPropertyModel *shell_player);
+static void rhythmdb_property_model_drag_source_init (RbTreeDragSourceIface *iface);
+
+G_DEFINE_TYPE_WITH_CODE(RhythmDBPropertyModel, rhythmdb_property_model, G_TYPE_OBJECT,
+			G_IMPLEMENT_INTERFACE(GTK_TYPE_TREE_MODEL,
+					      rhythmdb_property_model_tree_model_init)
+			G_IMPLEMENT_INTERFACE(RB_TYPE_TREE_DRAG_SOURCE,
+					      rhythmdb_property_model_drag_source_init))
+
 static void rhythmdb_property_model_finalize (GObject *object);
 static void rhythmdb_property_model_set_property (GObject *object,
 					       guint prop_id,
@@ -41,16 +48,23 @@ static void rhythmdb_property_model_get_property (GObject *object,
 					       guint prop_id,
 					       GValue *value,
 					       GParamSpec *pspec);
+static void rhythmdb_property_model_sync (RhythmDBPropertyModel *model);
 static void rhythmdb_property_model_row_inserted_cb (GtkTreeModel *model,
 						     GtkTreePath *path,
 						     GtkTreeIter *iter,
 						     RhythmDBPropertyModel *propmodel);
-static void rhythmdb_property_model_row_deleted_cb (GtkTreeModel *model,
-						    GtkTreePath *path,
-						    RhythmDBPropertyModel *propmodel);
+static void rhythmdb_property_model_prop_changed_cb (RhythmDB *db, RhythmDBEntry *entry,
+						     RhythmDBPropType prop, const GValue *old,
+						     const GValue *new,
+						     RhythmDBPropertyModel *propmodel);
+static void rhythmdb_property_model_entry_removed_cb (RhythmDBQueryModel *model,
+						      RhythmDBEntry *entry,
+						      RhythmDBPropertyModel *propmodel);
 static void rhythmdb_property_model_insert (RhythmDBPropertyModel *model, RhythmDBEntry *entry);
-static gboolean rhythmdb_property_model_delete (RhythmDBPropertyModel *model,
-						RhythmDBEntry *entry);
+static void rhythmdb_property_model_delete (RhythmDBPropertyModel *model,
+					    RhythmDBEntry *entry);
+static void rhythmdb_property_model_delete_prop (RhythmDBPropertyModel *model,
+						 const char *propstr);
 static GtkTreeModelFlags rhythmdb_property_model_get_flags (GtkTreeModel *model);
 static gint rhythmdb_property_model_get_n_columns (GtkTreeModel *tree_model);
 static GType rhythmdb_property_model_get_column_type (GtkTreeModel *tree_model, int index);
@@ -76,9 +90,46 @@ static gboolean rhythmdb_property_model_iter_parent (GtkTreeModel *tree_model,
 						  GtkTreeIter  *iter,
 						  GtkTreeIter  *child);
 
+static gboolean rhythmdb_property_model_drag_data_get (RbTreeDragSource *dragsource,
+						       GList *paths,
+						       GtkSelectionData *selection_data);
+static gboolean rhythmdb_property_model_drag_data_delete (RbTreeDragSource *dragsource,
+							  GList *paths);
+static gboolean rhythmdb_property_model_row_draggable (RbTreeDragSource *dragsource,
+						       GList *paths);
+enum {
+	TARGET_ALBUMS,
+	TARGET_GENRE,
+	TARGET_ARTISTS,
+	TARGET_LOCATION,
+	TARGET_URIS,
+};
+
+static const GtkTargetEntry targets_album  [] = {
+	{ "text/x-rhythmbox-album",  0, TARGET_ALBUMS },
+	{ "text/uri-list", 0, TARGET_URIS },
+};
+static const GtkTargetEntry targets_genre  [] = {
+	{ "text/x-rhythmbox-genre",  0, TARGET_GENRE },
+	{ "text/uri-list", 0, TARGET_URIS },
+};
+static const GtkTargetEntry targets_artist [] = {
+	{ "text/x-rhythmbox-artist", 0, TARGET_ARTISTS },
+	{ "text/uri-list", 0, TARGET_URIS },
+};
+static const GtkTargetEntry targets_location [] = {
+	{ "text/x-rhythmbox-location", 0, TARGET_LOCATION },
+	{ "text/uri-list", 0, TARGET_URIS },
+};
+
+
+static GtkTargetList *rhythmdb_property_model_album_drag_target_list = NULL;
+static GtkTargetList *rhythmdb_property_model_artist_drag_target_list = NULL;
+static GtkTargetList *rhythmdb_property_model_genre_drag_target_list = NULL;
+
 typedef struct {
-	char *name;
-	char *sort_key;
+	RBRefString *string;
+	RBRefString *sort_string;
 	guint refcount;
 } RhythmDBPropertyModelEntry;
 
@@ -102,6 +153,7 @@ struct RhythmDBPropertyModelPrivate
 	RhythmDBPropertyModelEntry *all;
 	
 	gboolean complete;
+	guint syncing_id;
 };
 
 enum
@@ -119,54 +171,25 @@ enum
 	PROP_QUERY_MODEL,
 };
 
-static GObjectClass *parent_class = NULL;
-
 static guint rhythmdb_property_model_signals[LAST_SIGNAL] = { 0 };
 
-GType
-rhythmdb_property_model_get_type (void)
-{
-	static GType rhythmdb_property_model_type = 0;
-	static const GInterfaceInfo tree_model_info =
-	{
-		(GInterfaceInitFunc) rhythmdb_property_model_tree_model_init,
-		NULL,
-		NULL
-	};
-
-	if (rhythmdb_property_model_type == 0)
-	{
-		static const GTypeInfo our_info =
-		{
-			sizeof (RhythmDBPropertyModelClass),
-			NULL,
-			NULL,
-			(GClassInitFunc) rhythmdb_property_model_class_init,
-			NULL,
-			NULL,
-			sizeof (RhythmDBPropertyModel),
-			0,
-			(GInstanceInitFunc) rhythmdb_property_model_init
-		};
-
-		rhythmdb_property_model_type = g_type_register_static (G_TYPE_OBJECT,
-								       "RhythmDBPropertyModel",
-								       &our_info, 0);
-
-		g_type_add_interface_static (rhythmdb_property_model_type,
-					     GTK_TYPE_TREE_MODEL,
-					     &tree_model_info);
-	}
-
-	return rhythmdb_property_model_type;
-}
-
 static void
 rhythmdb_property_model_class_init (RhythmDBPropertyModelClass *klass)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
-	parent_class = g_type_class_peek_parent (klass);
+	if (!rhythmdb_property_model_artist_drag_target_list)
+		rhythmdb_property_model_artist_drag_target_list =
+			gtk_target_list_new (targets_artist,
+					     G_N_ELEMENTS (targets_artist));
+	if (!rhythmdb_property_model_album_drag_target_list)
+		rhythmdb_property_model_album_drag_target_list =
+			gtk_target_list_new (targets_album,
+					     G_N_ELEMENTS (targets_album));
+	if (!rhythmdb_property_model_genre_drag_target_list)
+		rhythmdb_property_model_genre_drag_target_list =
+			gtk_target_list_new (targets_genre,
+					     G_N_ELEMENTS (targets_genre));
 
 	object_class->set_property = rhythmdb_property_model_set_property;
 	object_class->get_property = rhythmdb_property_model_get_property;
@@ -233,6 +256,14 @@ rhythmdb_property_model_tree_model_init (GtkTreeModelIface *iface)
 	iface->iter_parent = rhythmdb_property_model_iter_parent;
 }
 
+static void
+rhythmdb_property_model_drag_source_init (RbTreeDragSourceIface *iface)
+{
+	iface->row_draggable = rhythmdb_property_model_row_draggable;
+	iface->drag_data_delete = rhythmdb_property_model_drag_data_delete;
+	iface->drag_data_get = rhythmdb_property_model_drag_data_get;
+}
+
 static void
 rhythmdb_property_model_set_property (GObject *object,
 				   guint prop_id,
@@ -258,16 +289,17 @@ rhythmdb_property_model_set_property (GObject *object,
 		switch (model->priv->propid)
 		{
 		case RHYTHMDB_PROP_GENRE:
-			model->priv->sort_propid = RHYTHMDB_PROP_GENRE_SORT_KEY;
+			model->priv->sort_propid = RHYTHMDB_PROP_GENRE;
 			break;
 		case RHYTHMDB_PROP_ARTIST:
-			model->priv->sort_propid = RHYTHMDB_PROP_ARTIST_SORT_KEY;
+			model->priv->sort_propid = RHYTHMDB_PROP_ARTIST;
 			break;
 		case RHYTHMDB_PROP_ALBUM:
-			model->priv->sort_propid = RHYTHMDB_PROP_ALBUM_SORT_KEY;
+			model->priv->sort_propid = RHYTHMDB_PROP_ALBUM;
 			break;
 		case RHYTHMDB_PROP_TITLE:
-			model->priv->sort_propid = RHYTHMDB_PROP_TITLE_SORT_KEY;
+		case RHYTHMDB_PROP_LOCATION:
+			model->priv->sort_propid = RHYTHMDB_PROP_TITLE;
 			break;
 		default:
 			g_assert_not_reached ();
@@ -281,11 +313,11 @@ rhythmdb_property_model_set_property (GObject *object,
 							      G_CALLBACK (rhythmdb_property_model_row_inserted_cb),
 							      model);
 			g_signal_handlers_disconnect_by_func (G_OBJECT (model->priv->query_model),
-							      G_CALLBACK (rhythmdb_property_model_row_deleted_cb),
+							      G_CALLBACK (rhythmdb_property_model_entry_removed_cb),
+							      model);
+			g_signal_handlers_disconnect_by_func (G_OBJECT (model->priv->query_model),
+							      G_CALLBACK (rhythmdb_property_model_prop_changed_cb),
 							      model);
-/* 			g_signal_handlers_disconnect_by_func (G_OBJECT (model->priv->query_model), */
-/* 							      G_CALLBACK (rhythmdb_property_model_row_changed_cb), */
-/* 							      model); */
 			g_object_unref (G_OBJECT (model->priv->query_model));
 		}
 		
@@ -298,15 +330,15 @@ rhythmdb_property_model_set_property (GObject *object,
 						 model,
 						 0);
 			g_signal_connect_object (G_OBJECT (model->priv->query_model),
-						 "row_deleted",
-						 G_CALLBACK (rhythmdb_property_model_row_deleted_cb),
+						 "entry-removed",
+						 G_CALLBACK (rhythmdb_property_model_entry_removed_cb),
+						 model,
+						 0);
+			g_signal_connect_object (G_OBJECT (model->priv->query_model),
+						 "entry-prop-changed",
+						 G_CALLBACK (rhythmdb_property_model_prop_changed_cb),
 						 model,
 						 0);
-/* 			g_signal_connect_object (G_OBJECT (model->priv->query_model), */
-/* 						 "row_changed", */
-/* 						 G_CALLBACK (rhythmdb_property_model_row_changed_cb), */
-/* 						 model, */
-/* 						 0); */
 			g_object_ref (G_OBJECT (model->priv->query_model));
 		}
 		break;
@@ -360,8 +392,7 @@ rhythmdb_property_model_init (RhythmDBPropertyModel *model)
 	model->priv->reverse_map = g_hash_table_new (g_str_hash, g_str_equal);
 
 	model->priv->all = g_mem_chunk_alloc (model->priv->property_memchunk);
-	model->priv->all->name = g_strdup (_("All"));
-	model->priv->all->sort_key = NULL;
+	model->priv->all->string = rb_refstring_new (_("All"));
 }
 
 static void
@@ -380,21 +411,27 @@ rhythmdb_property_model_finalize (GObject *object)
 
 	g_return_if_fail (model->priv != NULL);
 
+	if (model->priv->syncing_id != 0)
+		g_source_remove (model->priv->syncing_id);
+
 	end_ptr = g_sequence_get_end_ptr (model->priv->properties);
 	for (ptr = g_sequence_get_begin_ptr (model->priv->properties); ptr != end_ptr;
 	     ptr = g_sequence_ptr_next (ptr)) {
 		RhythmDBPropertyModelEntry *prop = g_sequence_ptr_get_data (ptr);
-		g_free (prop->name);
-		g_free (prop->sort_key);
+		rb_refstring_unref (prop->string);
+		rb_refstring_unref (prop->sort_string);
 	}
 
 	g_mem_chunk_destroy (model->priv->property_memchunk);
 	g_hash_table_destroy (model->priv->reverse_map);
 	g_sequence_free (model->priv->properties);
 
+	if (model->priv->query_model)
+		g_object_unref (G_OBJECT (model->priv->query_model));
+
 	g_free (model->priv);
 
-	G_OBJECT_CLASS (parent_class)->finalize (object);
+	G_OBJECT_CLASS (rhythmdb_property_model_parent_class)->finalize (object);
 }
 
 RhythmDBPropertyModel *
@@ -411,18 +448,6 @@ entry_from_tree_iter (GtkTreeModel *model, GtkTreeIter *iter)
 	return entry;
 }
 
-static RhythmDBEntry *
-entry_from_tree_path (GtkTreeModel *model, GtkTreePath *path)
-{
-	GtkTreeIter entry_iter;
-	RhythmDBEntry *entry;
-
-	gtk_tree_model_get_iter (model, &entry_iter, path);
-	gtk_tree_model_get (model, &entry_iter, 0,
-			    &entry, -1);
-	return entry;
-}
-
 static void
 rhythmdb_property_model_row_inserted_cb (GtkTreeModel *model,
 					 GtkTreePath *path,
@@ -431,21 +456,44 @@ rhythmdb_property_model_row_inserted_cb (GtkTreeModel *model,
 {
 	RhythmDBEntry *entry = entry_from_tree_iter (model, iter);
 	rhythmdb_property_model_insert (propmodel, entry);
+
+	rhythmdb_property_model_sync (propmodel);
 }
 
 static void
-rhythmdb_property_model_row_deleted_cb (GtkTreeModel *model,
-					GtkTreePath *path,
-					RhythmDBPropertyModel *propmodel)
+rhythmdb_property_model_prop_changed_cb (RhythmDB *db, RhythmDBEntry *entry,
+					 RhythmDBPropType prop, const GValue *old,
+					 const GValue *new,
+					 RhythmDBPropertyModel *propmodel)
+{
+	if (prop != propmodel->priv->propid)
+		return;
+
+	rhythmdb_property_model_delete_prop (propmodel, g_value_get_string (old));
+	rhythmdb_property_model_insert (propmodel, entry);
+
+	rhythmdb_property_model_sync (propmodel);
+}
+
+static void
+rhythmdb_property_model_entry_removed_cb (RhythmDBQueryModel *model,
+					  RhythmDBEntry *entry,
+					  RhythmDBPropertyModel *propmodel)
 {
-	RhythmDBEntry *entry = entry_from_tree_path (model, path);
 	rhythmdb_property_model_delete (propmodel, entry);
+
+	rhythmdb_property_model_sync (propmodel);
 }
 
 static gint
 rhythmdb_property_model_compare (RhythmDBPropertyModelEntry *a, RhythmDBPropertyModelEntry *b, RhythmDBPropertyModel *model)
 {
-	return strcmp (a->sort_key, b->sort_key);
+	const char *a_str, *b_str;
+
+	a_str = rb_refstring_get_sort_key (a->sort_string);
+	b_str = rb_refstring_get_sort_key (b->sort_string);
+	
+	return strcmp (a_str, b_str);
 }
 
 static void
@@ -456,60 +504,61 @@ rhythmdb_property_model_insert (RhythmDBPropertyModel *model, RhythmDBEntry *ent
 	GtkTreePath *path;
 	GSequencePtr ptr;
 	const char *propstr;
-	const char *sort_key;	
 
-	rhythmdb_read_lock (model->priv->db);
-	propstr = rhythmdb_entry_get_string (model->priv->db, entry, model->priv->propid);
-	rhythmdb_read_unlock (model->priv->db);
+	propstr = rhythmdb_entry_get_string (entry, model->priv->propid);
 
 	if ((ptr = g_hash_table_lookup (model->priv->reverse_map, propstr))) {
 		prop = g_sequence_ptr_get_data (ptr);
 		prop->refcount++;
+		rb_debug ("adding \"%s\": refcount %d", propstr, prop->refcount);
 		return;
 	}
 
-	rhythmdb_read_lock (model->priv->db);
-	sort_key = rhythmdb_entry_get_string (model->priv->db, entry, model->priv->sort_propid);
-	rhythmdb_read_unlock (model->priv->db);
-
 	prop = g_mem_chunk_alloc (model->priv->property_memchunk);
-	prop->name = g_strdup (propstr);
-	prop->sort_key = g_strdup (sort_key);
+	prop->string = rb_refstring_new (propstr);
+	prop->sort_string = rb_refstring_new (rhythmdb_entry_get_string (entry, model->priv->sort_propid));
 	prop->refcount = 1;
 
 	iter.stamp = model->priv->stamp;
 	ptr = g_sequence_insert_sorted (model->priv->properties, prop,
 					(GCompareDataFunc) rhythmdb_property_model_compare,
 					model);
-	g_hash_table_insert (model->priv->reverse_map, prop->name, ptr);
-	
+	g_hash_table_insert (model->priv->reverse_map,
+			     (gpointer)rb_refstring_get (prop->string),
+			     ptr);
+
 	iter.user_data = ptr;
 	path = rhythmdb_property_model_get_path (GTK_TREE_MODEL (model), &iter);
 	gtk_tree_model_row_inserted (GTK_TREE_MODEL (model), path, &iter);
 	gtk_tree_path_free (path);
 }
 
-gboolean
+void
 rhythmdb_property_model_delete (RhythmDBPropertyModel *model,
 				RhythmDBEntry *entry)
 {
-	GSequencePtr ptr;
 	const char *propstr;
+
+	propstr = rhythmdb_entry_get_string (entry, model->priv->propid);
+	rhythmdb_property_model_delete_prop (model, propstr);
+}
+	
+static void
+rhythmdb_property_model_delete_prop (RhythmDBPropertyModel *model,
+				     const char *propstr)
+{
+	GSequencePtr ptr;
 	RhythmDBPropertyModelEntry *prop;
 	GtkTreePath *path;
 	GtkTreeIter iter;
 
-	rhythmdb_read_lock (model->priv->db);
-	propstr = rhythmdb_entry_get_string (model->priv->db, entry, model->priv->propid);
-	rhythmdb_read_unlock (model->priv->db);
-
 	g_assert ((ptr = g_hash_table_lookup (model->priv->reverse_map, propstr)));
 
 	prop = g_sequence_ptr_get_data (ptr);
 	rb_debug ("deleting \"%s\": refcount: %d", propstr, prop->refcount);
 	prop->refcount--;
 	if (prop->refcount > 0)
-		return FALSE;
+		return;
 
 	iter.stamp = model->priv->stamp;
 	iter.user_data = ptr;
@@ -519,10 +568,27 @@ rhythmdb_property_model_delete (RhythmDBPropertyModel *model,
 	gtk_tree_model_row_deleted (GTK_TREE_MODEL (model), path);
 	gtk_tree_path_free (path);
 	g_sequence_remove (ptr);
-	g_hash_table_remove (model->priv->reverse_map, prop->name);
-	g_free (prop->name);
-	g_free (prop->sort_key);
+	g_hash_table_remove (model->priv->reverse_map, propstr);
+	rb_refstring_unref (prop->string);
+	rb_refstring_unref (prop->sort_string);
 	g_mem_chunk_free (model->priv->property_memchunk, prop);
+	return;
+}
+
+gboolean
+rhythmdb_property_model_iter_from_string (RhythmDBPropertyModel *model,
+					  const char *name, GtkTreeIter *iter)
+{
+	GSequencePtr ptr;
+
+	ptr = g_hash_table_lookup (model->priv->reverse_map, name);
+
+	if (!ptr)
+		return FALSE;
+
+	iter->stamp = model->priv->stamp;
+	iter->user_data = ptr;
+
 	return TRUE;
 }
 
@@ -606,7 +672,6 @@ rhythmdb_property_model_get_value (GtkTreeModel *tree_model, GtkTreeIter *iter,
 				   gint column, GValue *value)
 {
 	RhythmDBPropertyModel *model = RHYTHMDB_PROPERTY_MODEL (tree_model);
-	RhythmDBPropertyModelEntry *prop;
 
 	g_return_if_fail (!g_sequence_ptr_is_end (iter->user_data));
 	g_return_if_fail (model->priv->stamp == iter->stamp);
@@ -616,7 +681,7 @@ rhythmdb_property_model_get_value (GtkTreeModel *tree_model, GtkTreeIter *iter,
 		{
 		case 0:
 			g_value_init (value, G_TYPE_STRING);
-			g_value_set_string (value, model->priv->all->name);
+			g_value_set_string (value, rb_refstring_get (model->priv->all->string));
 			break;
 		case 1:
 			g_value_init (value, G_TYPE_BOOLEAN);
@@ -626,13 +691,13 @@ rhythmdb_property_model_get_value (GtkTreeModel *tree_model, GtkTreeIter *iter,
 			g_assert_not_reached ();
 		}
 	} else {
-		prop = g_sequence_ptr_get_data (iter->user_data);
+		RhythmDBPropertyModelEntry *prop = g_sequence_ptr_get_data (iter->user_data);
 		
 		switch (column)
 		{
 		case 0:
 			g_value_init (value, G_TYPE_STRING);
-			g_value_set_string (value, prop->name);
+			g_value_set_string (value, rb_refstring_get (prop->string));
 			break;
 		case 1:
 			g_value_init (value, G_TYPE_BOOLEAN);
@@ -731,3 +796,248 @@ rhythmdb_property_model_iter_parent (GtkTreeModel *tree_model,
 {
 	return FALSE;
 }
+
+static gboolean
+rhythmdb_property_model_row_draggable (RbTreeDragSource *dragsource,
+				       GList *paths)
+{
+	return TRUE;
+}
+
+static gboolean
+rhythmdb_property_model_drag_data_delete (RbTreeDragSource *dragsource, 
+					  GList *paths)
+{
+	/* not supported */
+	return TRUE;
+}
+
+/*Going through hoops to avoid nested functions*/
+struct QueryModelCbStruct { 
+	RhythmDB* db; 
+	GString* reply; 
+}; 
+
+static gboolean 
+query_model_cb (GtkTreeModel *query_model,
+ 		GtkTreePath *path, GtkTreeIter *iter,
+ 		struct QueryModelCbStruct *data) 
+{
+ 	const char* uri;
+ 	RhythmDBEntry* entry;
+	
+ 	gtk_tree_model_get (query_model, iter, 0, &entry, -1);
+ 	uri = rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_LOCATION);
+ 	g_string_append (data->reply, uri);
+ 	g_string_append (data->reply, "\r\n");
+	
+ 	rhythmdb_entry_unref (data->db, entry);
+ 	return FALSE;
+}
+
+static gboolean
+rhythmdb_property_model_drag_data_get (RbTreeDragSource *dragsource,
+				       GList *paths,
+				       GtkSelectionData *selection_data)
+{
+	RhythmDBPropertyModel *model = RHYTHMDB_PROPERTY_MODEL (dragsource);
+	guint target;
+	GtkTargetList *drag_target_list;
+
+	switch (model->priv->propid) {
+	case RHYTHMDB_PROP_GENRE:
+		drag_target_list = rhythmdb_property_model_genre_drag_target_list;
+		break;
+	case RHYTHMDB_PROP_ALBUM:
+		drag_target_list = rhythmdb_property_model_album_drag_target_list;
+		break;
+	case RHYTHMDB_PROP_ARTIST:
+		drag_target_list = rhythmdb_property_model_artist_drag_target_list;
+		break;
+	default:
+		g_assert_not_reached ();
+	}
+
+	if (!gtk_target_list_find (drag_target_list,
+				   selection_data->target, 
+				   &target)) {
+		return FALSE;
+	}
+
+	if (target == TARGET_URIS) {
+		RhythmDB *db = model->priv->db;
+ 		GtkTreeModel* query_model = GTK_TREE_MODEL (rhythmdb_query_model_new_empty(db));
+ 		GString* reply = g_string_new ("");
+ 		GtkTreeIter iter;
+ 		gboolean is_all;
+ 		struct QueryModelCbStruct tmp;
+		GtkTreePath *path;
+
+		rb_debug ("getting drag data as uri list");
+		/* check if first selected row is 'All' */
+		path = gtk_tree_row_reference_get_path (paths->data);
+		gtk_tree_model_get_iter (GTK_TREE_MODEL (model), &iter, path);
+		gtk_tree_model_get (GTK_TREE_MODEL (model), &iter,
+				    RHYTHMDB_PROPERTY_MODEL_COLUMN_PRIORITY,
+				    &is_all, -1);
+		gtk_tree_path_free (path);
+		if (is_all) {
+ 			/*filter out radios*/
+ 			rhythmdb_do_full_query (db, query_model,
+ 						RHYTHMDB_QUERY_PROP_EQUALS,
+ 						RHYTHMDB_PROP_TYPE, 
+ 						RHYTHMDB_ENTRY_TYPE_SONG,
+ 						RHYTHMDB_QUERY_END);
+		} else {
+ 			GList *row;
+			GPtrArray* subquery = g_ptr_array_new();
+ 
+ 			for (row = paths; row; row = row->next) {
+ 				char* name;
+				path = gtk_tree_row_reference_get_path (row->data);
+ 				gtk_tree_model_get_iter (GTK_TREE_MODEL (model), &iter, path);
+ 				gtk_tree_model_get (GTK_TREE_MODEL (model), &iter,
+ 						    RHYTHMDB_PROPERTY_MODEL_COLUMN_TITLE, 
+						    &name, -1);
+ 				if (row->next) {
+ 					rhythmdb_query_append (db, subquery,
+ 							       RHYTHMDB_QUERY_PROP_EQUALS,
+ 							       model->priv->propid, name,
+ 							       RHYTHMDB_QUERY_DISJUNCTION,
+ 							       RHYTHMDB_QUERY_END);
+ 				} else {
+ 					rhythmdb_query_append (db, subquery,
+ 							       RHYTHMDB_QUERY_PROP_EQUALS,
+ 							       model->priv->propid, name,
+							       RHYTHMDB_QUERY_END);
+ 				}
+ 
+				gtk_tree_path_free (path);
+ 				g_free (name);
+ 			}
+ 			/*filter out radios*/
+ 			rhythmdb_do_full_query (db, query_model,
+ 						RHYTHMDB_QUERY_PROP_EQUALS,
+ 						RHYTHMDB_PROP_TYPE, 
+ 						RHYTHMDB_ENTRY_TYPE_SONG,
+ 						RHYTHMDB_QUERY_SUBQUERY,
+ 						subquery,
+ 						RHYTHMDB_QUERY_END);
+		}
+ 		
+		tmp.db = db; 
+ 		tmp.reply = reply;
+ 		/* Too bad that we're on the main thread. Why doesn't gtk call us async?
+ 		 * How does file-roller manage? - it seems it refuses the drop when it isn't 
+		 * done unpacking. In which case, we should tweak the drop acknowledgement, 
+		 * and prepare the query using do_full_query_async. The query would be 
+		 * hooked to the drag context.
+		 */
+ 		gtk_tree_model_foreach (query_model,
+ 					(GtkTreeModelForeachFunc)query_model_cb,
+ 					&tmp);
+ 		
+ 		g_object_unref (query_model);
+ 
+ 		gtk_selection_data_set (selection_data,
+ 		                        selection_data->target,
+ 		                        8, (guchar *)reply->str,
+ 		                        reply->len);
+ 		g_string_free (reply, TRUE);
+
+	} else {
+		char* title;
+		GList *p;
+		GString* reply = g_string_new ("");
+ 
+		rb_debug ("getting drag data as list of property values");
+
+		for (p = paths; p; p = p->next) {
+			GtkTreeIter iter;
+			GtkTreePath *path;
+
+			path = gtk_tree_row_reference_get_path (p->data);
+			gtk_tree_model_get_iter (GTK_TREE_MODEL (model), &iter, path);
+			gtk_tree_model_get (GTK_TREE_MODEL (model), &iter,
+					    RHYTHMDB_PROPERTY_MODEL_COLUMN_TITLE, &title, -1);
+			g_string_append (reply, title);
+			if (p->next)
+				g_string_append (reply, "\r\n");
+			g_free (title);
+			gtk_tree_path_free (path);
+		}
+		gtk_selection_data_set (selection_data,
+					selection_data->target,
+					8, (guchar *)reply->str,
+					reply->len);
+		g_string_free (reply, TRUE);
+	}
+
+	return TRUE;
+}
+
+void			
+rhythmdb_property_model_enable_drag (RhythmDBPropertyModel *model, GtkTreeView *view)
+{
+	const GtkTargetEntry *targets;
+	gint n_elements;
+
+	switch (model->priv->propid) {
+	case RHYTHMDB_PROP_GENRE:
+		targets = targets_genre;
+		n_elements = G_N_ELEMENTS (targets_genre);
+		break;
+	case RHYTHMDB_PROP_ALBUM:
+		targets = targets_album;
+		n_elements = G_N_ELEMENTS (targets_album);
+		break;
+	case RHYTHMDB_PROP_ARTIST:
+		targets = targets_artist;
+		n_elements = G_N_ELEMENTS (targets_artist);
+		break;
+	case RHYTHMDB_PROP_LOCATION:
+		targets = targets_location;
+		n_elements = G_N_ELEMENTS (targets_location);		
+		break;
+	default:
+		g_assert_not_reached ();
+	}
+	
+	rb_tree_dnd_add_drag_source_support (view,
+					     GDK_BUTTON1_MASK,
+					     targets, n_elements,
+					     GDK_ACTION_COPY);
+}
+
+static gboolean
+rhythmdb_property_model_perform_sync (RhythmDBPropertyModel *model)
+{
+	int count = g_sequence_get_length (model->priv->properties);
+	GtkTreeIter iter;
+	GtkTreePath *path;
+	char *s;
+
+	rb_refstring_unref (model->priv->all->string);
+	s = g_strdup_printf (_("All (%d)"), count);
+	model->priv->all->string = rb_refstring_new (s);
+	g_free (s);
+
+	iter.stamp = model->priv->stamp;
+	iter.user_data = model->priv->all;
+	path = rhythmdb_property_model_get_path (GTK_TREE_MODEL (model), &iter);
+	gtk_tree_model_row_changed (GTK_TREE_MODEL (model), path, &iter);
+	gtk_tree_path_free (path);
+
+	model->priv->syncing_id = 0;
+	return FALSE;
+}
+
+static void
+rhythmdb_property_model_sync (RhythmDBPropertyModel *model)
+{
+	if (model->priv->syncing_id != 0)
+		return;
+
+	model->priv->syncing_id = g_idle_add ((GSourceFunc)rhythmdb_property_model_perform_sync, model);
+}
+
