@@ -72,7 +72,7 @@ RhythmDBQueryModel *	rhythmdb_query_model_new_empty	(RhythmDB *db);
 void			rhythmdb_query_model_add_entry	(RhythmDBQueryModel *model, RhythmDBEntry *entry);
 void			rhythmdb_query_model_add_entries(RhythmDBQueryModel *model, GPtrArray *entries);
 
-void			rhythmdb_query_model_remove_entry	(RhythmDBQueryModel *model, RhythmDBEntry *entry);
+gboolean		rhythmdb_query_model_remove_entry	(RhythmDBQueryModel *model, RhythmDBEntry *entry);
 
 void			rhythmdb_query_model_set_connected	(RhythmDBQueryModel *model, gboolean connected);
 
@@ -82,14 +82,10 @@ GnomeVFSFileSize	rhythmdb_query_model_get_size	(RhythmDBQueryModel *model);
 
 long			rhythmdb_query_model_get_duration(RhythmDBQueryModel *model);
 
-gboolean		rhythmdb_query_model_poll	(RhythmDBQueryModel *model,
-							 GTimeVal *timeout);
-
 gboolean		rhythmdb_query_model_entry_to_iter(RhythmDBQueryModel *model,
 							   RhythmDBEntry *entry,
 							   GtkTreeIter *iter);
 
-void			rhythmdb_query_model_cancel	(RhythmDBQueryModel *model);
 gboolean		rhythmdb_query_model_has_pending_changes (RhythmDBQueryModel *model);
 
 G_END_DECLS
