@@ -64,8 +64,8 @@ static void rhythmdb_tree_finalize (GObject *object);
 static void rhythmdb_tree_load (RhythmDB *rdb, gboolean *die);
 static void rhythmdb_tree_save (RhythmDB *rdb);
 static void rhythmdb_tree_entry_new (RhythmDB *db, RhythmDBEntry *entry);
-static void rhythmdb_tree_entry_set (RhythmDB *db, RhythmDBEntry *entry,
-				     guint propid, const GValue *value);
+static gboolean rhythmdb_tree_entry_set (RhythmDB *db, RhythmDBEntry *entry,
+					 guint propid, const GValue *value);
 
 static void rhythmdb_tree_entry_delete (RhythmDB *db, RhythmDBEntry *entry);
 static void rhythmdb_tree_entry_delete_by_type (RhythmDB *adb, RhythmDBEntryType type);
@@ -212,6 +212,8 @@ struct RhythmDBTreeLoadContext
 	RhythmDBEntry *entry;
 	GString *buf;
 	RhythmDBPropType propid;
+
+	gboolean has_date;
 };
 
 static void
@@ -248,6 +250,10 @@ rhythmdb_tree_parser_start_element (struct RhythmDBTreeLoadContext *ctx,
 						type = RHYTHMDB_ENTRY_TYPE_SONG;
 					else if (!strcmp (typename, "iradio"))
 						type = RHYTHMDB_ENTRY_TYPE_IRADIO_STATION;
+					else if (!strcmp (typename, "podcast-post"))
+						type = RHYTHMDB_ENTRY_TYPE_PODCAST_POST;
+					else if (!strcmp (typename, "podcast-feed"))
+						type = RHYTHMDB_ENTRY_TYPE_PODCAST_FEED;
 					else
 						return;
 					type_set = TRUE;
@@ -257,6 +263,7 @@ rhythmdb_tree_parser_start_element (struct RhythmDBTreeLoadContext *ctx,
 			g_assert (type_set);
 			ctx->state = RHYTHMDB_TREE_PARSER_STATE_ENTRY;
 			ctx->entry = rhythmdb_entry_allocate (RHYTHMDB (ctx->db), type);
+			ctx->has_date = FALSE;
 		} else
 			ctx->in_unknown_elt = TRUE;
 		break;
@@ -312,6 +319,13 @@ rhythmdb_tree_parser_end_element (struct RhythmDBTreeLoadContext *ctx, const cha
 		break;
 	case RHYTHMDB_TREE_PARSER_STATE_ENTRY:
 	{
+		if (!ctx->has_date) {
+			/* there is no date metadata, so this is from an old version
+			 * reset the last-modified timestamp, so that the file is re-read
+			 */
+			rb_debug ("pre-Date entry found, causing re-read");
+			ctx->entry->mtime = 0;
+		}
 		rhythmdb_tree_entry_new (RHYTHMDB (ctx->db), ctx->entry);
 		rhythmdb_entry_insert (RHYTHMDB (ctx->db), ctx->entry);
 		rhythmdb_commit (RHYTHMDB (ctx->db));
@@ -344,6 +358,17 @@ rhythmdb_tree_parser_end_element (struct RhythmDBTreeLoadContext *ctx, const cha
 		case RHYTHMDB_PROP_DISC_NUMBER:
 			ctx->entry->discnum = parse_ulong (ctx->buf->str);
 			break;
+		case RHYTHMDB_PROP_DATE:
+		{
+			gulong value = parse_ulong (ctx->buf->str);
+			
+			if (value > 0)
+				ctx->entry->date = g_date_new_julian (value);
+			else
+				;
+			ctx->has_date = TRUE;
+			break;
+		}
 		case RHYTHMDB_PROP_DURATION:
 			ctx->entry->duration = parse_ulong (ctx->buf->str);
 			break;
@@ -354,7 +379,9 @@ rhythmdb_tree_parser_end_element (struct RhythmDBTreeLoadContext *ctx, const cha
 			ctx->entry->location = g_strdup (ctx->buf->str);
 			break;
 		case RHYTHMDB_PROP_MOUNTPOINT:
-			ctx->entry->mountpoint = rb_refstring_new (ctx->buf->str);
+			/* remove this from old podcast-post entries */
+			if (!g_str_has_prefix (ctx->buf->str, "http://"))
+				ctx->entry->mountpoint = rb_refstring_new (ctx->buf->str);
 			break;
 		case RHYTHMDB_PROP_MTIME:
 			ctx->entry->mtime = parse_ulong (ctx->buf->str);
@@ -392,6 +419,30 @@ rhythmdb_tree_parser_end_element (struct RhythmDBTreeLoadContext *ctx, const cha
 		case RHYTHMDB_PROP_MIMETYPE:
 			ctx->entry->mimetype = rb_refstring_new (ctx->buf->str);
 			break;
+		case RHYTHMDB_PROP_STATUS:
+			ctx->entry->podcast->status = parse_ulong (ctx->buf->str);
+			break;			
+		case RHYTHMDB_PROP_DESCRIPTION:
+			ctx->entry->podcast->description = rb_refstring_new (ctx->buf->str);
+			break;
+		case RHYTHMDB_PROP_SUBTITLE:
+			ctx->entry->podcast->subtitle = rb_refstring_new (ctx->buf->str);
+			break;
+		case RHYTHMDB_PROP_SUMMARY:
+			ctx->entry->podcast->summary = rb_refstring_new (ctx->buf->str);
+			break;
+		case RHYTHMDB_PROP_LANG:
+			ctx->entry->podcast->lang = rb_refstring_new (ctx->buf->str);
+			break;
+		case RHYTHMDB_PROP_COPYRIGHT:
+			ctx->entry->podcast->copyright = rb_refstring_new (ctx->buf->str);
+			break;
+		case RHYTHMDB_PROP_IMAGE:
+			ctx->entry->podcast->image = rb_refstring_new (ctx->buf->str);
+			break;
+		case RHYTHMDB_PROP_POST_TIME:			
+			ctx->entry->podcast->post_time = parse_ulong (ctx->buf->str);
+			break;
 		case RHYTHMDB_PROP_TITLE_SORT_KEY:
 		case RHYTHMDB_PROP_GENRE_SORT_KEY:
 		case RHYTHMDB_PROP_ARTIST_SORT_KEY:
@@ -404,6 +455,7 @@ rhythmdb_tree_parser_end_element (struct RhythmDBTreeLoadContext *ctx, const cha
 		case RHYTHMDB_PROP_HIDDEN:
 		case RHYTHMDB_PROP_PLAYBACK_ERROR:
 		case RHYTHMDB_PROP_FIRST_SEEN_STR:
+		case RHYTHMDB_PROP_SEARCH_MATCH:
 		case RHYTHMDB_NUM_PROPERTIES:
 			g_assert_not_reached ();
 			break;
@@ -557,10 +609,10 @@ save_entry_int (struct RhythmDBTreeSaveContext *ctx,
 
 static void
 save_entry_ulong (struct RhythmDBTreeSaveContext *ctx,
-		  const xmlChar *elt_name, gulong num)
+		  const xmlChar *elt_name, gulong num, gboolean save_zeroes)
 {
 	char buf[92];
-	if (num == 0)
+	if (num == 0 && !save_zeroes)
 		return;
 	write_elt_name_open (ctx, elt_name);
 	g_snprintf (buf, sizeof (buf), "%lu", num);
@@ -615,6 +667,10 @@ save_entry (RhythmDBTree *db, RhythmDBEntry *entry, struct RhythmDBTreeSaveConte
 		RHYTHMDB_FWRITE_STATICSTR ("song", ctx->handle, ctx->error);
 	} else if (entry->type == RHYTHMDB_ENTRY_TYPE_IRADIO_STATION) {
 		RHYTHMDB_FWRITE_STATICSTR ("iradio", ctx->handle, ctx->error);
+	} else if (entry->type == RHYTHMDB_ENTRY_TYPE_PODCAST_POST) {
+		RHYTHMDB_FWRITE_STATICSTR ("podcast-post", ctx->handle, ctx->error);
+	} else if (entry->type == RHYTHMDB_ENTRY_TYPE_PODCAST_FEED) {
+		RHYTHMDB_FWRITE_STATICSTR ("podcast-feed", ctx->handle, ctx->error);
 	} else
 		g_assert_not_reached ();
 
@@ -646,13 +702,19 @@ save_entry (RhythmDBTree *db, RhythmDBEntry *entry, struct RhythmDBTreeSaveConte
 			save_entry_string(ctx, elt_name, rb_refstring_get (entry->genre));
 			break;
 		case RHYTHMDB_PROP_TRACK_NUMBER:
-			save_entry_ulong(ctx, elt_name, entry->tracknum);
+			save_entry_ulong (ctx, elt_name, entry->tracknum, FALSE);
 			break;
 		case RHYTHMDB_PROP_DISC_NUMBER:
-			save_entry_ulong(ctx, elt_name, entry->discnum);
+			save_entry_ulong (ctx, elt_name, entry->discnum, FALSE);
+			break;
+		case RHYTHMDB_PROP_DATE:
+			if (entry->date)
+				save_entry_ulong (ctx, elt_name, g_date_get_julian (entry->date), TRUE);
+			else
+				save_entry_ulong (ctx, elt_name, 0, TRUE);
 			break;
 		case RHYTHMDB_PROP_DURATION:
-			save_entry_ulong(ctx, elt_name, entry->duration);
+			save_entry_ulong (ctx, elt_name, entry->duration, FALSE);
 			break;
 		case RHYTHMDB_PROP_BITRATE:
 			save_entry_int(ctx, elt_name, entry->bitrate);
@@ -686,23 +748,55 @@ save_entry (RhythmDBTree *db, RhythmDBEntry *entry, struct RhythmDBTreeSaveConte
 			save_entry_string(ctx, elt_name, rb_refstring_get (entry->mimetype));
 			break;
 		case RHYTHMDB_PROP_MTIME:
-			save_entry_ulong(ctx, elt_name, entry->mtime);
+			save_entry_ulong (ctx, elt_name, entry->mtime, FALSE);
 			break;
 		case RHYTHMDB_PROP_FIRST_SEEN:
-			save_entry_ulong(ctx, elt_name, entry->first_seen);
+			save_entry_ulong (ctx, elt_name, entry->first_seen, FALSE);
 			break;
 		case RHYTHMDB_PROP_LAST_SEEN:
-			save_entry_ulong(ctx, elt_name, entry->last_seen);
+			save_entry_ulong (ctx, elt_name, entry->last_seen, FALSE);
 			break;
 		case RHYTHMDB_PROP_RATING:
 			save_entry_double(ctx, elt_name, entry->rating);
 			break;
 		case RHYTHMDB_PROP_PLAY_COUNT:
-			save_entry_ulong(ctx, elt_name, entry->play_count);
+			save_entry_ulong (ctx, elt_name, entry->play_count, FALSE);
 			break;
 		case RHYTHMDB_PROP_LAST_PLAYED:
-			save_entry_ulong(ctx, elt_name, entry->last_played);
+			save_entry_ulong (ctx, elt_name, entry->last_played, FALSE);
 			break;
+		case RHYTHMDB_PROP_STATUS:
+			if (entry->podcast)
+				save_entry_ulong (ctx, elt_name, entry->podcast->status, FALSE);
+			break;
+		case RHYTHMDB_PROP_DESCRIPTION:
+			if (entry->podcast && entry->podcast->description)
+				save_entry_string(ctx, elt_name, rb_refstring_get (entry->podcast->description));
+			break;
+		case RHYTHMDB_PROP_SUBTITLE:
+			if (entry->podcast && entry->podcast->subtitle)
+				save_entry_string(ctx, elt_name, rb_refstring_get (entry->podcast->subtitle));
+			break;
+		case RHYTHMDB_PROP_SUMMARY:
+			if (entry->podcast && entry->podcast->summary)
+				save_entry_string(ctx, elt_name, rb_refstring_get (entry->podcast->summary));
+			break;
+		case RHYTHMDB_PROP_LANG:
+			if (entry->podcast && entry->podcast->lang)
+				save_entry_string(ctx, elt_name, rb_refstring_get (entry->podcast->lang));
+			break;
+		case RHYTHMDB_PROP_COPYRIGHT:
+			if (entry->podcast && entry->podcast->copyright)
+				save_entry_string(ctx, elt_name, rb_refstring_get (entry->podcast->copyright));
+			break;
+		case RHYTHMDB_PROP_IMAGE:
+			if (entry->podcast && entry->podcast->image)
+				save_entry_string(ctx, elt_name, rb_refstring_get (entry->podcast->image));
+			break;
+		case RHYTHMDB_PROP_POST_TIME:
+			if (entry->podcast)
+				save_entry_ulong (ctx, elt_name, entry->podcast->post_time, FALSE);
+			break;			
 		case RHYTHMDB_PROP_TITLE_SORT_KEY:
 		case RHYTHMDB_PROP_GENRE_SORT_KEY:
 		case RHYTHMDB_PROP_ARTIST_SORT_KEY:
@@ -715,6 +809,7 @@ save_entry (RhythmDBTree *db, RhythmDBEntry *entry, struct RhythmDBTreeSaveConte
 		case RHYTHMDB_PROP_HIDDEN:
 		case RHYTHMDB_PROP_PLAYBACK_ERROR:
 		case RHYTHMDB_PROP_FIRST_SEEN_STR:
+		case RHYTHMDB_PROP_SEARCH_MATCH:
 		case RHYTHMDB_NUM_PROPERTIES:
 			break;
 		}
@@ -757,6 +852,12 @@ rhythmdb_tree_save (RhythmDB *rdb)
 	rhythmdb_hash_tree_foreach (rdb, RHYTHMDB_ENTRY_TYPE_IRADIO_STATION, 
 				    (RBTreeEntryItFunc)save_entry, 
 				    NULL, NULL, NULL, &ctx);
+	rhythmdb_hash_tree_foreach (rdb, RHYTHMDB_ENTRY_TYPE_PODCAST_POST, 
+				    (RBTreeEntryItFunc)save_entry, 
+				    NULL, NULL, NULL, &ctx);
+	rhythmdb_hash_tree_foreach (rdb, RHYTHMDB_ENTRY_TYPE_PODCAST_FEED, 
+				    (RBTreeEntryItFunc)save_entry, 
+				    NULL, NULL, NULL, &ctx);
 
 	RHYTHMDB_FWRITE_STATICSTR ("</rhythmdb>\n", ctx.handle, ctx.error);
 
@@ -1015,7 +1116,7 @@ remove_entry_from_album (RhythmDBTree *db, RhythmDBEntry *entry)
 	rb_refstring_unref (entry->album);
 }
 
-static void
+static gboolean
 rhythmdb_tree_entry_set (RhythmDB *adb, RhythmDBEntry *entry,
 			 guint propid, const GValue *value)
 {
@@ -1027,6 +1128,23 @@ rhythmdb_tree_entry_set (RhythmDB *adb, RhythmDBEntry *entry,
 	/* Handle special properties */
 	switch (propid)
 	{
+	case RHYTHMDB_PROP_LOCATION:
+	{
+		/* We have to use the string in the entry itself as the hash key,
+		 * otherwise either we leak it, or the string vanishes when the
+		 * GValue is freed; this means we have to do the entry modification
+		 * here, rather than letting rhythmdb_entry_set_internal do it.
+		 */
+		g_assert (g_hash_table_lookup (db->priv->entries, entry->location) != NULL);
+
+		g_hash_table_remove (db->priv->entries, entry->location);
+
+		g_free (entry->location);
+		entry->location = g_strdup (g_value_get_string (value));
+		g_hash_table_insert (db->priv->entries, entry->location, entry);
+
+		return TRUE;
+	}
 	case RHYTHMDB_PROP_ALBUM:
 	{
 		const char *albumname = g_value_get_string (value);
@@ -1104,6 +1222,8 @@ rhythmdb_tree_entry_set (RhythmDB *adb, RhythmDBEntry *entry,
 	default:
 		break;
 	}
+
+	return FALSE;
 }
 
 static void
@@ -1226,6 +1346,40 @@ rhythmdb_tree_evaluate_query (RhythmDB *adb, GPtrArray *query,
 				g_assert_not_reached (); \
 			}
 
+static gboolean
+search_match_properties (RhythmDB *db, RhythmDBEntry *entry, gchar **words)
+{
+	const RhythmDBPropType props[] = {
+		RHYTHMDB_PROP_TITLE_FOLDED,
+		RHYTHMDB_PROP_ALBUM_FOLDED,
+		RHYTHMDB_PROP_ARTIST_FOLDED,
+		RHYTHMDB_PROP_GENRE_FOLDED
+	};
+	gboolean islike = TRUE;
+	gchar **current;
+	int i;
+	
+	for (current = words; *current != NULL; current++) {
+		gboolean word_found = FALSE;
+
+		for (i = 0; i < G_N_ELEMENTS (props); i++) {
+			const char *entry_string = rhythmdb_entry_get_string (entry, props[i]);
+			if (entry_string && (strstr (entry_string, *current) != NULL)) {
+				/* the word was found, go to the next one */	
+				word_found = TRUE;
+				break;
+			}
+		}
+		if (!word_found) {
+			/* the word wasn't in any of the properties*/
+			islike = FALSE;
+			break;
+		}
+	}
+
+	return islike;
+}
+
 static gboolean
 evaluate_conjunctive_subquery (RhythmDBTree *dbtree, GPtrArray *query,
 			       guint base, guint max, RhythmDBEntry *entry)
@@ -1286,25 +1440,27 @@ evaluate_conjunctive_subquery (RhythmDBTree *dbtree, GPtrArray *query,
 		case RHYTHMDB_QUERY_PROP_NOT_LIKE:
 		{
 			if (rhythmdb_get_property_type (db, data->propid) == G_TYPE_STRING) {
-				const char *entry_string = rhythmdb_entry_get_string (entry, data->propid);
-				gboolean islike = FALSE;
+				gboolean islike;
 
-				/* check in case the property is NULL, the value should never be NULL */
-				if (entry_string == NULL)
-					return FALSE;
+				if (data->propid == RHYTHMDB_PROP_SEARCH_MATCH) {
+					/* this is a special property, that should match several things */
+					islike = search_match_properties (db, entry, g_value_get_boxed (data->val));
+					
+				} else {
+					const gchar *value_string = g_value_get_string (data->val);
+					const char *entry_string = rhythmdb_entry_get_string (entry, data->propid);
 
-				islike = (strstr (entry_string, g_value_get_string (data->val)) != NULL);
-				if (data->type == RHYTHMDB_QUERY_PROP_LIKE) {
-					if (!islike)
+					/* check in case the property is NULL, the value should never be NULL */
+					if (entry_string == NULL)
 						return FALSE;
-					else
-						continue;
-				} else if (data->type == RHYTHMDB_QUERY_PROP_NOT_LIKE) {
-					if (islike)
-						return FALSE;
-					else
-						continue;
+
+					islike = (strstr (entry_string, value_string) != NULL);
 				}
+
+				if ((data->type == RHYTHMDB_QUERY_PROP_LIKE) ^ islike)
+					return FALSE;
+				else
+					continue;
 				break;
 			} 
 			/* Fall through */
