@@ -1,4 +1,3 @@
-/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
  /*
  *  arch-tag: Header for RhythmDB - Rhythmbox backend queryable database
  *
@@ -48,6 +47,8 @@ typedef gint32 RhythmDBEntryType;
 
 #define RHYTHMDB_ENTRY_TYPE_SONG (rhythmdb_entry_song_get_type ())
 #define RHYTHMDB_ENTRY_TYPE_IRADIO_STATION (rhythmdb_entry_iradio_get_type ())
+#define RHYTHMDB_ENTRY_TYPE_PODCAST_POST (rhythmdb_entry_podcast_post_get_type ())
+#define RHYTHMDB_ENTRY_TYPE_PODCAST_FEED (rhythmdb_entry_podcast_feed_get_type ())
 
 typedef enum
 {
@@ -84,6 +85,7 @@ typedef enum
 	RHYTHMDB_PROP_PLAY_COUNT,
 	RHYTHMDB_PROP_LAST_PLAYED,
 	RHYTHMDB_PROP_BITRATE,
+	RHYTHMDB_PROP_DATE,
 	RHYTHMDB_PROP_TRACK_GAIN,
 	RHYTHMDB_PROP_TRACK_PEAK,
 	RHYTHMDB_PROP_ALBUM_GAIN,
@@ -101,10 +103,27 @@ typedef enum
 	RHYTHMDB_PROP_HIDDEN,
 	RHYTHMDB_PROP_PLAYBACK_ERROR,
 	RHYTHMDB_PROP_FIRST_SEEN_STR,
+	RHYTHMDB_PROP_SEARCH_MATCH,
+
+//podcast propriets
+	RHYTHMDB_PROP_STATUS,
+	RHYTHMDB_PROP_DESCRIPTION,
+	RHYTHMDB_PROP_SUBTITLE,
+	RHYTHMDB_PROP_SUMMARY,
+	RHYTHMDB_PROP_LANG,
+	RHYTHMDB_PROP_COPYRIGHT,
+	RHYTHMDB_PROP_IMAGE,
+	RHYTHMDB_PROP_POST_TIME,
 	
 	RHYTHMDB_NUM_PROPERTIES
 } RhythmDBPropType;
 
+enum {
+	RHYTHMDB_PODCAST_STATUS_COMPLETE = 100,
+	RHYTHMDB_PODCAST_STATUS_ERROR = 101,
+	RHYTHMDB_PODCAST_STATUS_WAITING = 102,
+	RHYTHMDB_PODCAST_STATUS_PAUSED = 103,
+};
 
 GType rhythmdb_query_get_type (void);
 GType rhythmdb_prop_get_type (void);
@@ -119,6 +138,19 @@ typedef struct {
 	GPtrArray *subquery;
 } RhythmDBQueryData;
 
+typedef struct {
+	/* podcast */
+	RBRefString *description;
+	RBRefString *subtitle;
+	RBRefString *summary;
+	RBRefString *lang;
+	RBRefString *copyright;
+	RBRefString *image;
+	gulong status;   //0-99: downloading; 100: Conplete; 101: Error; 102: wait; 103: pause;
+	gulong post_time;
+} RhythmDBPodcastFields;
+
+
 typedef struct {
 	/* internal bits */
 #ifndef G_DISABLE_ASSERT
@@ -142,6 +174,7 @@ typedef struct {
 	double track_peak;
 	double album_gain;
 	double album_peak;
+	GDate *date;
 
 	/* filesystem */
 	char *location;
@@ -170,6 +203,9 @@ typedef struct {
 
 	/* visibility (to hide entries on unmounted volumes) */
 	gboolean hidden;
+
+	/*Podcast*/
+	RhythmDBPodcastFields *podcast;
 } RhythmDBEntry;
 
 typedef struct {
@@ -228,6 +264,38 @@ rhythmdb_entry_get_string (RhythmDBEntry *entry, RhythmDBPropType propid)
 		return entry->playback_error;
 	case RHYTHMDB_PROP_FIRST_SEEN_STR:
 		return rb_refstring_get (entry->first_seen_str);
+//podcast propriets
+	case RHYTHMDB_PROP_DESCRIPTION:
+		if (entry->podcast)
+			return rb_refstring_get (entry->podcast->description);
+		else
+			return NULL;
+	case RHYTHMDB_PROP_SUBTITLE:
+		if (entry->podcast)
+			return rb_refstring_get (entry->podcast->subtitle);
+		else
+			return NULL;
+	case RHYTHMDB_PROP_SUMMARY: 
+		if (entry->podcast)
+			return rb_refstring_get (entry->podcast->summary);
+		else
+			return NULL;
+	case RHYTHMDB_PROP_LANG:
+		if (entry->podcast)
+			return rb_refstring_get (entry->podcast->lang);
+		else
+			return NULL;
+	case RHYTHMDB_PROP_COPYRIGHT:
+		if (entry->podcast)
+			return rb_refstring_get (entry->podcast->copyright);
+		else
+			return NULL;
+	case RHYTHMDB_PROP_IMAGE:
+		if (entry->podcast)
+			return rb_refstring_get (entry->podcast->image);
+		else
+			return NULL;
+
 	default:
 		g_assert_not_reached ();
 		return NULL;
@@ -282,6 +350,21 @@ rhythmdb_entry_get_ulong (RhythmDBEntry *entry, RhythmDBPropType propid)
 		return entry->play_count;
 	case RHYTHMDB_PROP_BITRATE:
 		return entry->bitrate;		
+	case RHYTHMDB_PROP_DATE:
+		if (entry->date)
+			return g_date_get_julian (entry->date);
+		else
+			return 0;
+	case RHYTHMDB_PROP_POST_TIME:
+		if (entry->podcast)
+			return entry->podcast->post_time;
+		else
+			return 0;
+	case RHYTHMDB_PROP_STATUS:
+		if (entry->podcast)
+			return entry->podcast->status;		
+		else
+			return 0;
 	default:
 		g_assert_not_reached ();
 		return 0;
@@ -348,7 +431,7 @@ typedef struct
 	
 	void		(*impl_entry_new)	(RhythmDB *db, RhythmDBEntry *entry);
 
-	void		(*impl_entry_set)	(RhythmDB *db, RhythmDBEntry *entry,
+	gboolean	(*impl_entry_set)	(RhythmDB *db, RhythmDBEntry *entry,
 	                                 guint propid, const GValue *value);
 
 	void		(*impl_entry_get)	(RhythmDB *db, RhythmDBEntry *entry,
@@ -359,7 +442,7 @@ typedef struct
 	void            (*impl_entry_delete_by_type) (RhythmDB *db, RhythmDBEntryType type);
 
 	RhythmDBEntry *	(*impl_lookup_by_location)(RhythmDB *db, const char *uri);
-
+	
 	gboolean 	(*impl_evaluate_query)	(RhythmDB *db, GPtrArray *query, RhythmDBEntry *entry);
 
 	void		(*impl_entry_foreach)	(RhythmDB *db, GFunc func, gpointer data);
@@ -399,8 +482,6 @@ void		rhythmdb_entry_delete	(RhythmDB *db, RhythmDBEntry *entry);
 void            rhythmdb_entry_delete_by_type (RhythmDB *db, 
 					       RhythmDBEntryType type);
 
-void		rhythmdb_entry_destroy	(RhythmDB *db, RhythmDBEntry *entry);
-
 
 RhythmDBEntry *	rhythmdb_entry_lookup_by_location (RhythmDB *db, const char *uri);
 
@@ -468,6 +549,8 @@ RhythmDBEntryType rhythmdb_entry_register_type          (void);
 
 RhythmDBEntryType rhythmdb_entry_song_get_type          (void);
 RhythmDBEntryType rhythmdb_entry_iradio_get_type        (void);
+RhythmDBEntryType rhythmdb_entry_podcast_post_get_type  (void);
+RhythmDBEntryType rhythmdb_entry_podcast_feed_get_type  (void);
 RhythmDBEntryType rhythmdb_entry_icecast_get_type        (void);
 
 extern GType rhythmdb_property_type_map[RHYTHMDB_NUM_PROPERTIES];
