@@ -32,9 +32,7 @@
 
 #include "rb-playlist-manager.h"
 #include "rb-playlist-source.h"
-#if defined(WITH_CD_BURNER_SUPPORT)
 #include "rb-recorder.h"
-#endif
 #include "rb-sourcelist.h"
 #include "rb-sourcelist-model.h"
 #include "rb-query-creator.h"
@@ -141,10 +139,10 @@ static GtkActionEntry rb_playlist_manager_actions [] =
 	/* Submenu of Music */
 	{ "Playlist", NULL, N_("_Playlist") },
 
-	{ "MusicPlaylistNewPlaylist", "gtk-media-playlist", N_("_New Playlist..."), "<control>N",
+	{ "MusicPlaylistNewPlaylist", GNOME_MEDIA_PLAYLIST, N_("_New Playlist..."), "<control>N",
 	  N_("Create a new playlist"),
 	  G_CALLBACK (rb_playlist_manager_cmd_new_playlist) },
-	{ "MusicPlaylistNewAutomaticPlaylist", "gtk-media-automatic-playlist", N_("New _Automatic Playlist..."), NULL,
+	{ "MusicPlaylistNewAutomaticPlaylist", GNOME_MEDIA_AUTO_PLAYLIST, N_("New _Automatic Playlist..."), NULL,
 	  N_("Create a new automatically updating playlist"),
 	  G_CALLBACK (rb_playlist_manager_cmd_new_automatic_playlist) },
 	{ "MusicPlaylistLoadPlaylist", NULL, N_("_Load from File..."), NULL,
@@ -373,6 +371,8 @@ rb_playlist_manager_set_uimanager (RBPlaylistManager *mgr,
 
 	if (mgr->priv->actiongroup == NULL) {
 		mgr->priv->actiongroup = gtk_action_group_new ("PlaylistActions");
+		gtk_action_group_set_translation_domain (mgr->priv->actiongroup,
+							 GETTEXT_PACKAGE);
 		gtk_action_group_add_actions (mgr->priv->actiongroup,
 					      rb_playlist_manager_actions,
 					      rb_playlist_manager_n_actions,
@@ -397,6 +397,8 @@ rb_playlist_manager_set_property (GObject *object,
 	case PROP_SOURCE:
 	{
 		gboolean playlist_active;
+		gboolean playlist_automatic = FALSE;
+		gboolean playlist_local = FALSE;
 		GtkAction *action;
 
 		if (mgr->priv->selected_source != NULL)
@@ -406,14 +408,23 @@ rb_playlist_manager_set_property (GObject *object,
 
 		mgr->priv->selected_source = g_value_get_object (value);
 		playlist_active = RB_IS_PLAYLIST_SOURCE (mgr->priv->selected_source);
+		if (playlist_active) {
+			g_object_get (G_OBJECT (mgr->priv->selected_source), "automatic", &playlist_automatic, NULL);
+			g_object_get (G_OBJECT (mgr->priv->selected_source), "is-local", &playlist_local, NULL);
+		}
 
 		action = gtk_action_group_get_action (mgr->priv->actiongroup,
 						      "MusicPlaylistSavePlaylist");
-		g_object_set (G_OBJECT (action), "sensitive", playlist_active, NULL);
+		g_object_set (G_OBJECT (action), "sensitive", playlist_local, NULL);
 		action = gtk_action_group_get_action (mgr->priv->actiongroup,
 						      "MusicPlaylistDeletePlaylist");
-		g_object_set (G_OBJECT (action), "sensitive", playlist_active, NULL);
-#if defined(WITH_CD_BURNER_SUPPORT)
+		g_object_set (G_OBJECT (action), "sensitive", playlist_local, NULL);
+		action = gtk_action_group_get_action (mgr->priv->actiongroup,
+ 						      "EditAutomaticPlaylist");
+		g_object_set (G_OBJECT (action), "sensitive", playlist_automatic, NULL);
+		action = gtk_action_group_get_action (mgr->priv->actiongroup,
+ 						      "MusicPlaylistRenamePlaylist");
+		g_object_set (G_OBJECT (action), "sensitive", playlist_local, NULL);
 		{
 			gboolean recorder_active;
 			int num_tracks = rb_entry_view_get_num_entries (rb_source_get_entry_view (mgr->priv->selected_source));
@@ -431,7 +442,6 @@ rb_playlist_manager_set_property (GObject *object,
 							 "entry-deleted", G_CALLBACK (rb_playlist_manager_playlist_entries_changed), mgr, 0);
 			}
 		}
-#endif
 		break;
 	}
 	case PROP_SHELL:
@@ -817,7 +827,10 @@ rb_playlist_manager_new_playlist (RBPlaylistManager *mgr,
 				  const char *suggested_name, 
 				  gboolean automatic)
 {
-	RBSource *playlist = RB_SOURCE (rb_playlist_source_new (mgr->priv->shell, automatic, TRUE));
+	RBSource *playlist = RB_SOURCE (rb_playlist_source_new (mgr->priv->shell, 
+								automatic, 
+								TRUE, 
+								RHYTHMDB_ENTRY_TYPE_SONG));
 	
 	g_object_set (G_OBJECT (playlist),
 		      "name", suggested_name ? suggested_name : "",
