@@ -37,6 +37,15 @@ G_BEGIN_DECLS
 #define RB_IS_PLAYLIST_MANAGER_CLASS(k)  (G_TYPE_CHECK_CLASS_TYPE ((k), RB_TYPE_PLAYLIST_MANAGER))
 #define RB_PLAYLIST_MANAGER_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), RB_TYPE_PLAYLIST_MANAGER, RBPlaylistManagerClass))
 
+typedef enum
+{
+	RB_PLAYLIST_MANAGER_ERROR_PARSE
+} RBPlaylistManagerError;
+
+#define RB_PLAYLIST_MANAGER_ERROR rb_playlist_manager_error_quark ()
+
+GQuark rb_playlist_manager_error_quark (void);
+
 typedef struct RBPlaylistManagerPrivate RBPlaylistManagerPrivate;
 
 typedef struct
@@ -52,6 +61,7 @@ typedef struct
 
 	/* signals */
 	void	(*playlist_added) (RBSource *source);
+	void	(*playlist_created) (RBSource *source);
 	void	(*load_start) (void);
 	void	(*load_finish) (void);
 } RBPlaylistManagerClass;
@@ -63,8 +73,9 @@ RBPlaylistManager *	rb_playlist_manager_new		(RBShell *shell,
 							 RBLibrarySource *libsource,
 							 RBIRadioSource *iradio_source);
 
-const char *		rb_playlist_manager_parse_file	(RBPlaylistManager *mgr,
-							 const char *uri);
+gboolean 		rb_playlist_manager_parse_file	(RBPlaylistManager *mgr,
+							 const char *uri,
+							 GError **error);
 
 void			rb_playlist_manager_set_source	(RBPlaylistManager *mgr,
 							 RBSource *player);
@@ -78,6 +89,7 @@ void			rb_playlist_manager_save_playlists (RBPlaylistManager *mgr, gboolean forc
 RBSource *		rb_playlist_manager_new_playlist (RBPlaylistManager *mgr,
 							  const char *suggested_name,
 							  gboolean automatic);
+GList *			rb_playlist_manager_get_playlists (RBPlaylistManager *manager);
 
 G_END_DECLS
 
