@@ -59,6 +59,7 @@
 #include "rb-util.h"
 #include "rb-play-order.h"
 #include "rb-statusbar.h"
+#include "rhythmdb.h"
 
 #ifdef HAVE_XIDLE_EXTENSION
 #include <X11/extensions/xidle.h>
@@ -250,7 +251,7 @@ enum
 enum
 {
 	WINDOW_TITLE_CHANGED,
-	DURATION_CHANGED,
+	ELAPSED_CHANGED,
 	PLAYING_SOURCE_CHANGED,
 	PLAYING_CHANGED,
 	PLAYING_SONG_CHANGED,
@@ -260,13 +261,13 @@ enum
 
 static GtkActionEntry rb_shell_player_actions [] =
 {
-	{ "ControlPrevious", GTK_STOCK_MEDIA_PREVIOUS, N_("P_revious"), "<control>Left",
+	{ "ControlPrevious", GTK_STOCK_MEDIA_PREVIOUS, N_("P_revious"), "<alt>Left",
 	  N_("Start playing the previous song"),
 	  G_CALLBACK (rb_shell_player_cmd_previous) },
-	{ "ControlNext", GTK_STOCK_MEDIA_NEXT, N_("_Next"), "<control>Right",
+	{ "ControlNext", GTK_STOCK_MEDIA_NEXT, N_("_Next"), "<alt>Right",
 	  N_("Start playing the next song"),
 	  G_CALLBACK (rb_shell_player_cmd_next) },
-	{ "MusicProperties", GTK_STOCK_PROPERTIES, N_("_Properties"), "<control>P",
+	{ "MusicProperties", GTK_STOCK_PROPERTIES, N_("_Properties"), "<Alt>Return",
 	  N_("Show information on the selected song"),
 	  G_CALLBACK (rb_shell_player_cmd_song_info) },
 };
@@ -295,54 +296,25 @@ static GtkActionEntry rb_shell_player_stop_action [] =
 
 static GtkToggleActionEntry rb_shell_player_toggle_entries [] =
 {
-	{ "ControlShuffle", NULL, N_("Sh_uffle"), "<control>U",
+	{ "ControlShuffle", GNOME_MEDIA_SHUFFLE, N_("Sh_uffle"), "<control>U",
 	  N_("Play songs in a random order"),
 	  G_CALLBACK (rb_shell_player_shuffle_changed_cb) },
-	{ "ControlRepeat", NULL, N_("_Repeat"), "<control>R",
+	{ "ControlRepeat", GNOME_MEDIA_REPEAT, N_("_Repeat"), "<control>R",
 	  N_("Play first song again after all songs are played"),
 	  G_CALLBACK (rb_shell_player_repeat_changed_cb) }
 };
 static guint rb_shell_player_n_toggle_entries = G_N_ELEMENTS (rb_shell_player_toggle_entries);
 
-static GObjectClass *parent_class = NULL;
-
 static guint rb_shell_player_signals[LAST_SIGNAL] = { 0 };
 
-GType
-rb_shell_player_get_type (void)
-{
-	static GType rb_shell_player_type = 0;
+G_DEFINE_TYPE (RBShellPlayer, rb_shell_player, GTK_TYPE_HBOX)
 
-	if (rb_shell_player_type == 0)
-	{
-		static const GTypeInfo our_info =
-		{
-			sizeof (RBShellPlayerClass),
-			NULL,
-			NULL,
-			(GClassInitFunc) rb_shell_player_class_init,
-			NULL,
-			NULL,
-			sizeof (RBShellPlayer),
-			0,
-			(GInstanceInitFunc) rb_shell_player_init
-		};
-
-		rb_shell_player_type = g_type_register_static (GTK_TYPE_HBOX,
-							       "RBShellPlayer",
-							       &our_info, 0);
-	}
-
-	return rb_shell_player_type;
-}
 
 static void
 rb_shell_player_class_init (RBShellPlayerClass *klass)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
-	parent_class = g_type_class_peek_parent (klass);
-
 	object_class->finalize = rb_shell_player_finalize;
 	object_class->constructor = rb_shell_player_constructor;
 
@@ -425,16 +397,16 @@ rb_shell_player_class_init (RBShellPlayerClass *klass)
 			      1,
 			      G_TYPE_STRING);
 
-	rb_shell_player_signals[DURATION_CHANGED] =
-		g_signal_new ("duration_changed",
+	rb_shell_player_signals[ELAPSED_CHANGED] =
+		g_signal_new ("elapsed_changed",
 			      G_OBJECT_CLASS_TYPE (object_class),
 			      G_SIGNAL_RUN_LAST,
-			      G_STRUCT_OFFSET (RBShellPlayerClass, duration_changed),
+			      G_STRUCT_OFFSET (RBShellPlayerClass, elapsed_changed),
 			      NULL, NULL,
-			      g_cclosure_marshal_VOID__STRING,
+			      g_cclosure_marshal_VOID__UINT,
 			      G_TYPE_NONE,
 			      1,
-			      G_TYPE_STRING);
+			      G_TYPE_UINT);
 
 	rb_shell_player_signals[PLAYING_SOURCE_CHANGED] =
 		g_signal_new ("playing-source-changed",
@@ -487,13 +459,11 @@ rb_shell_player_constructor (GType type, guint n_construct_properties,
 {
 	RBShellPlayer *player;
 	RBShellPlayerClass *klass;
-	GObjectClass *parent_class;  
-
+	
 	klass = RB_SHELL_PLAYER_CLASS (g_type_class_peek (RB_TYPE_SHELL_PLAYER));
 
-	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
-	player = RB_SHELL_PLAYER (parent_class->constructor (type, n_construct_properties,
-							     construct_properties));
+	player = RB_SHELL_PLAYER (G_OBJECT_CLASS (rb_shell_player_parent_class)->
+			constructor (type, n_construct_properties, construct_properties));
 
 	gtk_action_group_add_actions (player->priv->actiongroup,
 				      rb_shell_player_actions,
@@ -529,8 +499,11 @@ rb_shell_player_constructor (GType type, guint n_construct_properties,
 					     rb_shell_player_toggle_entries,
 					     rb_shell_player_n_toggle_entries,
 					     player);
+	player->priv->syncing_state = TRUE;
 	rb_shell_player_set_playing_source (player, NULL);
 	rb_shell_player_sync_play_order (player);
+	rb_shell_player_sync_control_state (player);
+	player->priv->syncing_state = FALSE;
 
 	return G_OBJECT (player);
 }
@@ -544,8 +517,10 @@ volume_pre_unmount_cb (GnomeVFSVolumeMonitor *monitor,
 	gchar *volume_mount_point;
 	RhythmDBEntry *entry;
 	const char *uri;
+	gboolean playing;
 
-	if (rb_shell_player_get_playing (player)) {
+	rb_shell_player_get_playing (player, &playing, NULL);
+	if (playing) {
 		return;
 	}
 
@@ -755,7 +730,7 @@ rb_shell_player_finalize (GObject *object)
 	
 	g_free (player->priv);
 
-	G_OBJECT_CLASS (parent_class)->finalize (object);
+	G_OBJECT_CLASS (rb_shell_player_parent_class)->finalize (object);
 }
 
 static void
@@ -1050,7 +1025,10 @@ rb_shell_player_open_location (RBShellPlayer *player,
 static gboolean
 rb_shell_player_open_entry (RBShellPlayer *player, RhythmDBEntry *entry, GError **error)
 {
-	return rb_shell_player_open_location (player, entry->location, error);
+	if (entry->type == RHYTHMDB_ENTRY_TYPE_PODCAST_POST)
+		return rb_shell_player_open_location (player, rb_refstring_get (entry->mountpoint), error);
+	else
+		return rb_shell_player_open_location (player, entry->location, error);
 }
 
 static gboolean
@@ -1094,13 +1072,9 @@ do_next_idle (RBShellPlayer *player)
 	GError *error = NULL;
 	
 	if (!rb_shell_player_do_next (player, &error)) {
-		if (error->domain == RB_SHELL_PLAYER_ERROR
-		    && error->code == RB_SHELL_PLAYER_ERROR_END_OF_PLAYLIST) {
-			rb_debug ("No next entry, stopping playback");
-			rb_shell_player_set_playing_source (player, NULL);
-			g_object_notify (G_OBJECT (player), "playing");
-		}
-		g_warning ("do_next_idle: Unhandled error: %s", error->message);
+		if (error->domain != RB_SHELL_PLAYER_ERROR ||
+		    error->code != RB_SHELL_PLAYER_ERROR_END_OF_PLAYLIST)
+			g_warning ("do_next_idle: Unhandled error: %s", error->message);
 	}
 
 	return FALSE;
@@ -1148,6 +1122,11 @@ rb_shell_player_set_playing_entry (RBShellPlayer *player, RhythmDBEntry *entry,
  lose:
 	/* Ignore errors, shutdown the player */
 	rb_player_close (player->priv->mmplayer, NULL);
+	if (tmp_error == NULL)
+		tmp_error = g_error_new (RB_SHELL_PLAYER_ERROR,
+					 RB_SHELL_PLAYER_ERROR_NOT_PLAYING,
+					 "Problem occurred without error being set. "
+					 "This is a bug in Rhythmbox or GStreamer.");
 	/* Mark this song as failed */
 	rb_shell_player_set_entry_playback_error (player, entry, tmp_error->message);
 	g_propagate_error (error, tmp_error);
@@ -1276,6 +1255,7 @@ rb_shell_player_do_previous (RBShellPlayer *player, GError **error)
 				     RB_SHELL_PLAYER_ERROR,
 				     RB_SHELL_PLAYER_ERROR_END_OF_PLAYLIST,
 				     _("No previous song"));
+			rb_shell_player_set_playing_source (player, NULL);
 			return FALSE;
 		}
 		
@@ -1305,6 +1285,9 @@ rb_shell_player_do_next (RBShellPlayer *player, GError **error)
 				     RB_SHELL_PLAYER_ERROR,
 				     RB_SHELL_PLAYER_ERROR_END_OF_PLAYLIST,
 				     _("No next song"));
+			rb_debug ("No previous entry, stopping playback");
+			rb_shell_player_set_playing_source (player, NULL);
+			g_object_notify (G_OBJECT (player), "playing");
 			return FALSE;
 		}
 		rb_shell_player_jump_to_current (player);
@@ -1312,7 +1295,7 @@ rb_shell_player_do_next (RBShellPlayer *player, GError **error)
 	return TRUE;
 }
 
-static void
+static gboolean
 rb_shell_player_do_previous_or_seek (RBShellPlayer *player, GError **error)
 {
 	rb_debug ("previous");
@@ -1326,37 +1309,63 @@ rb_shell_player_do_previous_or_seek (RBShellPlayer *player, GError **error)
 		rb_player_set_time (player->priv->mmplayer, 0);
 		rb_header_sync_time (player->priv->header_widget);
 	} else {
-		rb_shell_player_do_previous (player, NULL);
+		return rb_shell_player_do_previous (player, error);
 	}
+
+	return TRUE;
 }
 
 static void
 rb_shell_player_previous_button_cb (GtkButton *button,
 				    RBShellPlayer *player)
 {
-	rb_shell_player_do_previous_or_seek (player, NULL);
+	GError *error = NULL;
+	
+	if (!rb_shell_player_do_previous_or_seek (player, &error)) {
+		if (error->domain != RB_SHELL_PLAYER_ERROR ||
+		    error->code != RB_SHELL_PLAYER_ERROR_END_OF_PLAYLIST)
+			g_warning ("previos_button_cb: Unhandled error: %s", error->message);
+	}
 }
 
 static void
 rb_shell_player_next_button_cb (GtkButton *button,
 				RBShellPlayer *player)
 {
-	rb_shell_player_do_next (player, NULL);
+	GError *error = NULL;
+	
+	if (!rb_shell_player_do_next (player, &error)) {
+		if (error->domain != RB_SHELL_PLAYER_ERROR ||
+		    error->code != RB_SHELL_PLAYER_ERROR_END_OF_PLAYLIST)
+			g_warning ("next_button_cb: Unhandled error: %s", error->message);
+	}
+
 }
 
 static void
 rb_shell_player_cmd_previous (GtkAction *action,
 			      RBShellPlayer *player)
 {
-	rb_shell_player_do_previous_or_seek (player, NULL);
+	GError *error = NULL;
+	
+	if (!rb_shell_player_do_previous_or_seek (player, &error)) {
+		if (error->domain != RB_SHELL_PLAYER_ERROR ||
+		    error->code != RB_SHELL_PLAYER_ERROR_END_OF_PLAYLIST)
+			g_warning ("cmd_previous: Unhandled error: %s", error->message);
+	}
 }
 
 static void
 rb_shell_player_cmd_next (GtkAction *action,
 			  RBShellPlayer *player)
 {
-	rb_debug ("next");
-	rb_shell_player_do_next (player, NULL);
+	GError *error = NULL;
+	
+	if (!rb_shell_player_do_next (player, &error)) {
+		if (error->domain != RB_SHELL_PLAYER_ERROR ||
+		    error->code != RB_SHELL_PLAYER_ERROR_END_OF_PLAYLIST)
+			g_warning ("cmd_next: Unhandled error: %s", error->message);
+	}
 }
 
 void
@@ -1638,6 +1647,11 @@ rb_shell_player_entry_activated_cb (RBEntryView *view,
 	g_return_if_fail (entry != NULL);
 
 	rb_debug  ("got entry %p activated", entry);
+
+	/* ensure the podcast has been downloaded */
+	if (rhythmdb_entry_get_ulong (entry, RHYTHMDB_PROP_TYPE) == RHYTHMDB_ENTRY_TYPE_PODCAST_POST)
+		if (rhythmdb_entry_get_ulong (entry, RHYTHMDB_PROP_STATUS) != RHYTHMDB_PODCAST_STATUS_COMPLETE)
+			return;
 	
 	rb_shell_player_set_playing_source (playa, playa->priv->selected_source);
 
@@ -1751,7 +1765,7 @@ rb_shell_player_sync_with_source (RBShellPlayer *player)
 	const char *artist = NULL;	
 	char *title;
 	RhythmDBEntry *entry;
-	char *duration;
+	glong elapsed;
 
 	entry = rb_shell_player_get_playing_entry (player);
 	rb_debug ("playing source: %p, active entry: %p", player->priv->source, entry);
@@ -1779,13 +1793,12 @@ rb_shell_player_sync_with_source (RBShellPlayer *player)
 	else
 		title = NULL;
 
-	duration = rb_header_get_elapsed_string (player->priv->header_widget);
+	elapsed = rb_player_get_time (player->priv->mmplayer);
 
 	g_signal_emit (G_OBJECT (player), rb_shell_player_signals[WINDOW_TITLE_CHANGED], 0,
 		       title);
-	g_signal_emit (G_OBJECT (player), rb_shell_player_signals[DURATION_CHANGED], 0,
-		       duration);
-	g_free (duration);
+	g_signal_emit (G_OBJECT (player), rb_shell_player_signals[ELAPSED_CHANGED], 0,
+		       (guint) elapsed);
 
 	/* Sync the player */
 	if (player->priv->song)
@@ -1975,11 +1988,14 @@ rb_shell_player_stop (RBShellPlayer *player)
 }
 
 gboolean
-rb_shell_player_get_playing (RBShellPlayer *player)
+rb_shell_player_get_playing (RBShellPlayer *player,
+			     gboolean *playing,
+			     GError **error)
 {
-	g_return_val_if_fail (RB_IS_SHELL_PLAYER (player), -1);
+	if (playing != NULL)
+		*playing = rb_player_playing (player->priv->mmplayer);
 
-	return rb_player_playing (player->priv->mmplayer);
+	return TRUE;
 }
 
 RBPlayer *
@@ -1990,21 +2006,38 @@ rb_shell_player_get_mm_player (RBShellPlayer *player)
 	return player->priv->mmplayer;
 }
 
-long
-rb_shell_player_get_playing_time (RBShellPlayer *player)
+char *
+rb_shell_player_get_playing_time_string (RBShellPlayer *player)
 {
-	g_return_val_if_fail (RB_IS_SHELL_PLAYER (player), 0);
-	
-	return rb_player_get_time (player->priv->mmplayer);
+	return rb_header_get_elapsed_string (player->priv->header_widget);
 }
 
-void
-rb_shell_player_set_playing_time (RBShellPlayer *player, long time)
+gboolean
+rb_shell_player_get_playing_time (RBShellPlayer *player,
+				  guint *time,
+				  GError **error)
 {
-	g_return_if_fail (RB_IS_SHELL_PLAYER (player));
-	
-	if (rb_player_seekable (player->priv->mmplayer))
-		rb_player_set_time (player->priv->mmplayer, time);
+	if (time != NULL)
+		*time = (guint) rb_player_get_time (player->priv->mmplayer);
+
+	return TRUE;
+}
+
+gboolean
+rb_shell_player_set_playing_time (RBShellPlayer *player,
+				  guint time,
+				  GError **error)
+{
+	if (rb_player_seekable (player->priv->mmplayer)) {
+		rb_player_set_time (player->priv->mmplayer, (long) time);
+		return TRUE;
+	} else {
+		g_set_error (error,
+			     RB_SHELL_PLAYER_ERROR,
+			     RB_SHELL_PLAYER_ERROR_NOT_SEEKABLE,
+			     _("Current song is not seekable"));
+		return FALSE;
+	}
 }
 
 void
@@ -2087,7 +2120,15 @@ eos_cb (RBPlayer *mmplayer, gpointer data)
 		}
 			break;
 		case RB_SOURCE_EOF_NEXT:
-			rb_shell_player_do_next (player, NULL);
+			{
+				GError *error = NULL;
+	
+				if (!rb_shell_player_do_next (player, &error)) {
+					if (error->domain != RB_SHELL_PLAYER_ERROR ||
+					    error->code != RB_SHELL_PLAYER_ERROR_END_OF_PLAYLIST)
+						g_warning ("eos_cb: Unhandled error: %s", error->message);
+				}
+			}
 			break;
 		}
 
@@ -2127,7 +2168,7 @@ rb_shell_player_error (RBShellPlayer *player, gboolean async, const GError *err)
 		rb_entry_view_set_playing_entry (songs, entry);
 	}
 
-	g_printerr ("playback error: %s", err->message);
+	rb_debug ("playback error while playing: %s", err->message);
 	/* For synchronous errors the entry playback error has already been set */
 	if (entry && async)
 		rb_shell_player_set_entry_playback_error (player, entry, err->message);
@@ -2171,16 +2212,11 @@ tick_cb (RBPlayer *mmplayer, long elapsed, gpointer data)
 	rb_header_sync_time (player->priv->header_widget);
 
 	if (rb_player_playing (mmplayer)) {
-		static int callback_runs = 0;
-		callback_runs++;
-		if (callback_runs >= RB_PLAYER_TICK_HZ) {
-			gchar *duration;
-
-			duration = rb_header_get_elapsed_string (player->priv->header_widget);
-			g_signal_emit (G_OBJECT (player), rb_shell_player_signals[DURATION_CHANGED],
-				       0, duration);
-			g_free (duration);
-			callback_runs = 0;
+		static long last_elapsed = -1;
+		if (last_elapsed != elapsed) {
+			g_signal_emit (G_OBJECT (player), rb_shell_player_signals[ELAPSED_CHANGED],
+				       0, (guint) elapsed);
+			last_elapsed = elapsed;
 		}
 	}
 
@@ -2294,6 +2330,20 @@ info_available_cb (RBPlayer *mmplayer,
 		}
 		break;
 	}
+	case RB_METADATA_FIELD_BITRATE:
+		if (!rhythmdb_entry_get_ulong (entry, RHYTHMDB_PROP_BITRATE)) {
+			gulong bitrate;
+
+			/* GStreamer sends us bitrate in bps, but we need it in kbps*/
+			bitrate = g_value_get_ulong (value);
+			g_value_set_ulong (value, bitrate/1000);
+			
+			rb_debug ("setting bitrate of iradio station to %d", 
+				  g_value_get_ulong (value));
+			entry_field = RHYTHMDB_PROP_BITRATE;
+			set_field = TRUE;
+		}
+		break;
 	default:
 		break;
 	}
@@ -2321,10 +2371,21 @@ buffering_cb (RBPlayer *mmplayer, guint progress, gpointer data)
 }
 
 
-const char *
-rb_shell_player_get_playing_path (RBShellPlayer *shell_player)
+gboolean
+rb_shell_player_get_playing_path (RBShellPlayer *shell_player,
+				  const gchar **path,
+				  GError **error)
 {
-	return shell_player->priv->url;
+	RhythmDBEntry *entry;
+
+	entry = rb_play_order_get_playing_entry (shell_player->priv->play_order);
+	if (entry != NULL) {
+		*path = rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_LOCATION);
+	} else {
+		*path = NULL;
+	}
+
+	return TRUE;
 }
 
 #ifdef HAVE_MMKEYS
@@ -2391,7 +2452,9 @@ filter_mmkeys (GdkXEvent *xevent, GdkEvent *event, gpointer data)
 		rb_shell_player_playpause (player, TRUE, NULL);
 		return GDK_FILTER_REMOVE;
 	} else if (XKeysymToKeycode (GDK_DISPLAY (), XF86XK_AudioPause) == key->keycode) {	
-		if (rb_shell_player_get_playing	(player)) {
+		gboolean playing;
+		rb_shell_player_get_playing (player, &playing, NULL);
+		if (playing) {
 			rb_shell_player_playpause (player, TRUE, NULL);
 		}
 		return GDK_FILTER_REMOVE;
