@@ -220,6 +220,8 @@ rb_tray_icon_constructor (GType type, guint n_construct_properties,
 							construct_properties));
 
 	tray->priv->actiongroup = gtk_action_group_new ("TrayActions");
+	gtk_action_group_set_translation_domain (tray->priv->actiongroup,
+						 GETTEXT_PACKAGE);
 	gtk_action_group_add_toggle_actions (tray->priv->actiongroup,
 					     rb_tray_icon_toggle_entries,
 					     rb_tray_icon_n_toggle_entries,
@@ -376,7 +378,14 @@ rb_tray_icon_button_press_event_cb (GtkWidget *ebox, GdkEventButton *event,
 	switch (event->button) {
 	case 1: {
 		gboolean visible = rb_remote_proxy_get_visibility (icon->priv->proxy);
-		rb_remote_proxy_set_visibility (icon->priv->proxy, visible ? FALSE : TRUE);
+		rb_remote_proxy_set_visibility (icon->priv->proxy, !visible);
+		break;
+	}
+	case 2: {
+		if (rb_remote_proxy_playing (icon->priv->proxy))
+			rb_remote_proxy_pause (icon->priv->proxy);
+		else
+			rb_remote_proxy_play (icon->priv->proxy);
 		break;
 	}
 		
@@ -450,7 +459,7 @@ rb_tray_icon_drop_cb (GtkWidget *widget,
 	uri_list = NULL;
 
 	for (i = list; i != NULL; i = g_list_next (i))
-		uri_list = g_list_append (uri_list, gnome_vfs_uri_to_string ((const GnomeVFSURI *) i->data, 0));
+		uri_list = g_list_prepend (uri_list, gnome_vfs_uri_to_string ((const GnomeVFSURI *) i->data, 0));
 
 	gnome_vfs_uri_list_free (list);
 
