@@ -53,6 +53,9 @@ static void rb_daap_source_get_property  (GObject *object,
 					  GValue *value,
 				 	  GParamSpec *pspec);
 static void rb_daap_source_activate (RBSource *source);
+static void rb_daap_source_connection_cb (RBDAAPConnection *connection,
+					  gboolean result,
+					  RBSource *source);
 static gboolean rb_daap_source_disconnect (RBSource *source);
 static gboolean rb_daap_source_show_popup (RBSource *source);
 static const gchar * rb_daap_source_get_browser_key (RBSource *source);
@@ -125,6 +128,8 @@ rb_daap_source_class_init (RBDAAPSourceClass *klass)
 	source_class->impl_get_browser_key = rb_daap_source_get_browser_key;
 
 	library_source_class->impl_get_paned_key = rb_daap_source_get_paned_key;
+	library_source_class->impl_has_first_added_column = (RBLibrarySourceFeatureFunc) rb_false_function;
+	library_source_class->impl_has_drop_support = (RBLibrarySourceFeatureFunc) rb_false_function;
 
 
 	g_object_class_install_property (object_class,
@@ -173,9 +178,10 @@ rb_daap_source_dispose (GObject *object)
 {
 	RBDAAPSource *source = RB_DAAP_SOURCE (object);
 
-	rb_daap_source_disconnect (RB_SOURCE (source));
-
 	if (source->priv) {
+		/* we should already have been disconnected */
+		g_assert (source->priv->connection == NULL);
+	
 		g_free (source->priv->service_name);
 		g_free (source->priv->host);
 		g_free (source->priv);
@@ -519,58 +525,92 @@ static void
 rb_daap_source_activate (RBSource *source)
 {
 	RBDAAPSource *daap_source = RB_DAAP_SOURCE (source);
+	RBShell *shell = NULL;
+	RhythmDB *db = NULL;
+	gchar *name = NULL;
+	RhythmDBEntryType type;
 
+	if (daap_source->priv->connection != NULL) 
+		return;
+
+	g_object_get (G_OBJECT (daap_source), 
+		      "shell", &shell, 
+		      "entry-type", &type, 
+		      "name", &name, 
+		      NULL);
+	g_object_get (G_OBJECT (shell), "db", &db, NULL);
+
+	daap_source->priv->connection =
+		rb_daap_connection_new (name,
+					daap_source->priv->host,
+					daap_source->priv->port,
+					daap_source->priv->password_protected,
+					db,
+					type,
+					(RBDAAPConnectionCallback) rb_daap_source_connection_cb,
+					source);
+	g_object_unref (G_OBJECT (db));
+	g_object_unref (G_OBJECT (shell));
 	if (daap_source->priv->connection == NULL) {
-		RBShell *shell = NULL;
-		RhythmDB *db = NULL;
-		gchar *name = NULL;
-		RhythmDBEntryType type;
-
-		g_object_get (G_OBJECT (daap_source), "shell", &shell, "entry-type", &type, "name", &name, NULL);
-		g_object_get (G_OBJECT (shell), "db", &db, NULL);
-
-		/* FIXME FIXME FIXME
-		 * this is the call that takes a long time
-		 */
-		daap_source->priv->connection =
-			rb_daap_connection_new (name,
-						daap_source->priv->host,
-						daap_source->priv->port,
-						daap_source->priv->password_protected,
-						db,
-						type);
-
-		if (daap_source->priv->connection != NULL) {
-			GSList *playlists = rb_daap_connection_get_playlists (daap_source->priv->connection);
-			GSList *l;
-
-			for (l = playlists; l != NULL; l = l->next) {
-				RBDAAPPlaylist *playlist = l->data;
-				RBSource *playlist_source;
-
-				playlist_source = RB_SOURCE (g_object_new (RB_TYPE_PLAYLIST_SOURCE,
-									   "name", playlist->name,
-									   "shell", shell,
-									   "visibility", TRUE,
-									   "is-local", FALSE,
-									   NULL));
-				/* this is set here instead of in construction so that
-				 * rb_playlist_source_constructor has a chance to be run to set things up */
-				rb_playlist_source_add_locations (RB_PLAYLIST_SOURCE (playlist_source), playlist->uris);
-
-				rb_shell_append_source (shell, playlist_source, RB_SOURCE (daap_source));
-				daap_source->priv->playlist_sources = g_slist_prepend (daap_source->priv->playlist_sources, playlist_source);
-			}
-		} else {
-			daap_source->priv->playlist_sources = NULL;
-		}
-
-		g_free (name);
-		g_object_unref (G_OBJECT (db));
-		g_object_unref (G_OBJECT (shell));
+		/* XXX can this still happen? */
+		daap_source->priv->playlist_sources = NULL;
+		return;
 	}
 }
 
+static void
+rb_daap_source_connection_cb (RBDAAPConnection *connection,
+			      gboolean result,
+			      RBSource *source)
+{
+	RBDAAPSource *daap_source = RB_DAAP_SOURCE (source);
+	RBShell *shell = NULL;
+	GSList *playlists;
+	GSList *l;
+	RhythmDBEntryType entry_type;
+
+	if (result == FALSE) {
+		/* FIXME display error?  should get more info from the connection.. */
+		return;
+	}
+
+	g_object_get (G_OBJECT (daap_source), 
+		      "shell", &shell, 
+		      "entry-type", &entry_type,
+		      NULL);
+	playlists = rb_daap_connection_get_playlists (daap_source->priv->connection);
+	for (l = playlists; l != NULL; l = g_slist_next (l)) {
+		RBDAAPPlaylist *playlist = l->data;
+		RBSource *playlist_source;
+
+		playlist_source = RB_SOURCE (g_object_new (RB_TYPE_PLAYLIST_SOURCE,
+							   "name", playlist->name,
+							   "shell", shell,
+							   "visibility", TRUE,
+							   "is-local", FALSE,
+							   "entry-type", entry_type,
+							   NULL));
+		/* this is set here instead of in construction so that
+		 * rb_playlist_source_constructor has a chance to be run to set things up */
+		rb_playlist_source_add_locations (RB_PLAYLIST_SOURCE (playlist_source), playlist->uris);
+
+		rb_shell_append_source (shell, playlist_source, RB_SOURCE (daap_source));
+		daap_source->priv->playlist_sources = g_slist_prepend (daap_source->priv->playlist_sources, playlist_source);
+	}
+	g_object_unref (G_OBJECT (shell));
+}
+
+static void
+rb_daap_source_disconnect_cb (RBDAAPConnection *connection,
+			      gboolean result,
+			      RBSource *source)
+{
+	RBDAAPSource *daap_source = RB_DAAP_SOURCE (source);
+	g_object_unref (G_OBJECT (connection));
+	daap_source->priv->connection = NULL;
+	g_object_unref (source);
+}
+
 static gboolean
 rb_daap_source_disconnect (RBSource *source)
 {
@@ -594,17 +634,18 @@ rb_daap_source_disconnect (RBSource *source)
 
 		for (l = daap_source->priv->playlist_sources; l!= NULL; l = l->next) {
 			RBSource *playlist_source = RB_SOURCE (l->data);
-			gchar *s;
 
-			g_object_get (G_OBJECT (playlist_source), "name", &s, NULL);
 			rb_source_delete_thyself (playlist_source);
 		}
 
 		g_slist_free (daap_source->priv->playlist_sources);
 		daap_source->priv->playlist_sources = NULL;
 
-		rb_daap_connection_destroy (daap_source->priv->connection);
-		daap_source->priv->connection = NULL;
+		/* keep the source alive until the disconnect completes */
+		g_object_ref (daap_source);
+		rb_daap_connection_logout (daap_source->priv->connection,
+					   (RBDAAPConnectionCallback) rb_daap_source_disconnect_cb,
+					   daap_source);
 	}
 
 	return TRUE;
