@@ -1,3 +1,4 @@
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
  *  arch-tag: Implementation of playlist source object
  *
@@ -24,9 +25,8 @@
 #include <gtk/gtk.h>
 #include <libgnome/gnome-i18n.h>
 #include <libgnomevfs/gnome-vfs-uri.h>
+#include <totem-pl-parser.h>
 #include <libxml/tree.h>
-#include <bonobo/bonobo-ui-component.h>
-#include <bonobo/bonobo-window.h>
 #include <unistd.h>
 #include <string.h>
 
@@ -34,26 +34,35 @@
 #include "rb-entry-view.h"
 #include "rb-search-entry.h"
 #include "rb-file-helpers.h"
-#include "rb-playlist.h"
-#include "rb-thread-helpers.h"
 #include "rb-preferences.h"
 #include "rb-dialog.h"
 #include "rb-util.h"
 #include "rb-playlist-source.h"
-#include "rb-volume.h"
-#include "rb-bonobo-helpers.h"
+#include "rb-playlist-source-recorder.h"
 #include "rb-debug.h"
 #include "eel-gconf-extensions.h"
 #include "rb-song-info.h"
-#include "rb-tree-view-column.h"
 
 #define RB_PLAYLIST_XML_VERSION "1.0"
 
+#define RB_PLAYLIST_PLAYLIST (xmlChar *) "playlist"
+#define RB_PLAYLIST_TYPE (xmlChar *) "type"
+#define RB_PLAYLIST_AUTOMATIC (xmlChar *) "automatic"
+#define RB_PLAYLIST_STATIC (xmlChar *) "static"
+#define RB_PLAYLIST_NAME (xmlChar *) "name"
+#define RB_PLAYLIST_LIMIT_COUNT (xmlChar *) "limit-count"
+#define RB_PLAYLIST_LIMIT_SIZE (xmlChar *) "limit-size"
+#define RB_PLAYLIST_LIMIT_TIME (xmlChar *) "limit-time"
+#define RB_PLAYLIST_SORT_KEY (xmlChar *) "sort-key"
+#define RB_PLAYLIST_SORT_DIRECTION (xmlChar *) "sort-direction"
+#define RB_PLAYLIST_LIMIT (xmlChar *) "limit"
+#define RB_PLAYLIST_LOCATION (xmlChar *) "location"
+
 static void rb_playlist_source_class_init (RBPlaylistSourceClass *klass);
 static void rb_playlist_source_init (RBPlaylistSource *source);
 static GObject *rb_playlist_source_constructor (GType type, guint n_construct_properties,
 						GObjectConstructParam *construct_properties);
-static void rb_playlist_source_finalize (GObject *object);
+static void rb_playlist_source_dispose (GObject *object);
 static void rb_playlist_source_set_property (GObject *object,
 			                  guint prop_id,
 			                  const GValue *value,
@@ -64,7 +73,7 @@ static void rb_playlist_source_get_property (GObject *object,
 			                  GParamSpec *pspec);
 
 /* source methods */
-static const char *impl_get_status (RBSource *source);
+static char *impl_get_status (RBSource *source);
 static const char *impl_get_browser_key (RBSource *source);
 static GdkPixbuf *impl_get_pixbuf (RBSource *source);
 static RBEntryView *impl_get_entry_view (RBSource *source);
@@ -77,6 +86,8 @@ static gboolean impl_receive_drag (RBSource *source, GtkSelectionData *data);
 static gboolean impl_show_popup (RBSource *source);
 static void rb_playlist_source_entry_added_cb (RhythmDB *db, RhythmDBEntry *entry,
 					       RBPlaylistSource *source);
+static void rb_playlist_source_songs_sort_order_changed_cb (RBEntryView *view,
+							    RBPlaylistSource *source);
 
 static void rb_playlist_source_songs_show_popup_cb (RBEntryView *view, RBPlaylistSource *playlist_view);
 static void rb_playlist_source_drop_cb (GtkWidget *widget,
@@ -89,19 +100,35 @@ static void rb_playlist_source_drop_cb (GtkWidget *widget,
 				     gpointer user_data);
 static void rb_playlist_source_add_list_uri (RBPlaylistSource *source,
 					  GList *list);
+static void rb_playlist_source_do_query (RBPlaylistSource *source,
+					 GPtrArray *query,
+					 guint limit_count,
+					 guint limit_mb,
+					 guint limit_time);
+static void rb_playlist_source_row_inserted (GtkTreeModel *treemodel,
+					     GtkTreePath *path,
+					     GtkTreeIter *iter,
+					     RBPlaylistSource *source);
+static void rb_playlist_source_non_entry_dropped (GtkTreeModel *model,
+						  const char *uri,
+						  RBPlaylistSource *source);
 
-#define PLAYLIST_SOURCE_SONGS_POPUP_PATH "/popups/PlaylistSongsList"
-#define PLAYLIST_SOURCE_POPUP_PATH "/popups/PlaylistSourceList"
-#define PLAYLIST_SOURCE_AUTOMATIC_POPUP_PATH "/popups/AutomaticPlaylistSourceList"
+#define PLAYLIST_SOURCE_SONGS_POPUP_PATH "/PlaylistViewPopup"
+#define PLAYLIST_SOURCE_POPUP_PATH "/PlaylistSourcePopup"
+#define PLAYLIST_SOURCE_AUTOMATIC_POPUP_PATH "/SmartPlaylistSourcePopup"
 
 struct RBPlaylistSourcePrivate
 {
+	gboolean disposed;
+	
 	RhythmDB *db;
 
 	gboolean automatic;
 	GHashTable *entries;
 
+	RhythmDBEntryType entry_type;
 	RhythmDBQueryModel *model;
+	gboolean query_resetting;
 
 	GtkWidget *vbox;
 	GdkPixbuf *normal_pixbuf;
@@ -109,49 +136,26 @@ struct RBPlaylistSourcePrivate
 
 	RBEntryView *songs;
 
+	gboolean dirty;
+	gboolean is_local;
+
 	char *title;
-	char *status;
 };
 
 enum
 {
 	PROP_0,
-	PROP_LIBRARY,
-	PROP_DB,
 	PROP_AUTOMATIC,
+	PROP_QUERY_MODEL,
+	PROP_DIRTY,
+	PROP_LOCAL,
+	PROP_ENTRY_TYPE
 };
 
-static GObjectClass *parent_class = NULL;
-
 static const GtkTargetEntry target_uri [] = { { "text/uri-list", 0, 0 } };
 
-GType
-rb_playlist_source_get_type (void)
-{
-	static GType rb_playlist_source_type = 0;
+G_DEFINE_TYPE (RBPlaylistSource, rb_playlist_source, RB_TYPE_SOURCE);
 
-	if (rb_playlist_source_type == 0)
-	{
-		static const GTypeInfo our_info =
-		{
-			sizeof (RBPlaylistSourceClass),
-			NULL,
-			NULL,
-			(GClassInitFunc) rb_playlist_source_class_init,
-			NULL,
-			NULL,
-			sizeof (RBPlaylistSource),
-			0,
-			(GInstanceInitFunc) rb_playlist_source_init
-		};
-
-		rb_playlist_source_type = g_type_register_static (RB_TYPE_SOURCE,
-								  "RBPlaylistSource",
-								  &our_info, 0);
-	}
-
-	return rb_playlist_source_type;
-}
 
 static void
 rb_playlist_source_class_init (RBPlaylistSourceClass *klass)
@@ -159,9 +163,7 @@ rb_playlist_source_class_init (RBPlaylistSourceClass *klass)
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
 	RBSourceClass *source_class = RB_SOURCE_CLASS (klass);
 
-	parent_class = g_type_class_peek_parent (klass);
-
-	object_class->finalize = rb_playlist_source_finalize;
+	object_class->dispose = rb_playlist_source_dispose;
 	object_class->constructor = rb_playlist_source_constructor;
 
 	object_class->set_property = rb_playlist_source_set_property;
@@ -181,19 +183,10 @@ rb_playlist_source_class_init (RBPlaylistSourceClass *klass)
 	source_class->impl_delete = impl_delete;
 	source_class->impl_song_properties = impl_song_properties;
 	source_class->impl_can_pause = (RBSourceFeatureFunc) rb_true_function;
-	source_class->impl_have_artist_album = (RBSourceFeatureFunc) rb_true_function;
 	source_class->impl_have_url = (RBSourceFeatureFunc) rb_false_function;
 	source_class->impl_receive_drag = impl_receive_drag;
 	source_class->impl_show_popup = impl_show_popup;
 
-	g_object_class_install_property (object_class,
-					 PROP_DB,
-					 g_param_spec_object ("db",
-							      "RhythmDB",
-							      "RhythmDB database",
-							      RHYTHMDB_TYPE,
-							      G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
-
 	g_object_class_install_property (object_class,
 					 PROP_AUTOMATIC,
 					 g_param_spec_boolean ("automatic",
@@ -201,6 +194,38 @@ rb_playlist_source_class_init (RBPlaylistSourceClass *klass)
 							       "whether this playlist is a smartypants",
 							       FALSE,
 							       G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+
+	g_object_class_install_property (object_class,
+					 PROP_QUERY_MODEL,
+					 g_param_spec_object ("query-model",
+							      "query-model",
+							      "query model for the playlist",
+							      RHYTHMDB_TYPE_QUERY_MODEL,
+							      G_PARAM_READABLE));
+
+	g_object_class_install_property (object_class,
+					 PROP_DIRTY,
+					 g_param_spec_boolean ("dirty",
+							       "dirty",
+							       "whether this playlist should be saved",
+							       FALSE,
+							       G_PARAM_READABLE));
+
+	g_object_class_install_property (object_class,
+					 PROP_LOCAL,
+					 g_param_spec_boolean ("is-local",
+							       "is-local",
+							       "whether this playlist is attached to the local library",
+							       FALSE,
+							       G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+	g_object_class_install_property (object_class,
+					 PROP_ENTRY_TYPE,
+					 g_param_spec_int ("entry-type",
+							   "entry-type",
+							   "The entry type this playlist accepts",
+							   -1, G_MAXINT, -1,
+							   G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+
 }
 
 static void
@@ -229,14 +254,16 @@ rb_playlist_source_constructor (GType type, guint n_construct_properties,
 {
 	RBPlaylistSource *source;
 	RBPlaylistSourceClass *klass;
-	GObjectClass *parent_class;  
-	GtkWidget *dummy = gtk_tree_view_new ();
+	RBShell *shell;
 
 	klass = RB_PLAYLIST_SOURCE_CLASS (g_type_class_peek (RB_TYPE_PLAYLIST_SOURCE));
 
-	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
-	source = RB_PLAYLIST_SOURCE (parent_class->constructor (type, n_construct_properties,
-								construct_properties));
+	source = RB_PLAYLIST_SOURCE (G_OBJECT_CLASS (rb_playlist_source_parent_class)->
+			constructor (type, n_construct_properties, construct_properties));
+
+	g_object_get (G_OBJECT (source), "shell", &shell, NULL);
+	g_object_get (RB_SHELL (shell), "db", &source->priv->db, NULL);
+	g_object_unref (G_OBJECT (shell));
 
 	g_signal_connect_object (G_OBJECT (source->priv->db), "entry_added",
 				 G_CALLBACK (rb_playlist_source_entry_added_cb),
@@ -255,10 +282,18 @@ rb_playlist_source_constructor (GType type, guint n_construct_properties,
 		
 	source->priv->songs = rb_entry_view_new (source->priv->db, NULL, TRUE, TRUE);
 
+	/* watch these to find out when things were dropping into the entry view */
+	g_signal_connect_object (G_OBJECT (source->priv->model), "row-inserted",
+			 G_CALLBACK (rb_playlist_source_row_inserted),
+			 source, 0);
+	g_signal_connect_object (G_OBJECT (source->priv->model), "non-entry-dropped",
+			 G_CALLBACK (rb_playlist_source_non_entry_dropped),
+			 source, 0);
+
 	rb_entry_view_set_model (source->priv->songs, RHYTHMDB_QUERY_MODEL (source->priv->model));
 
 	{
-		GtkTreeViewColumn *column = GTK_TREE_VIEW_COLUMN (rb_tree_view_column_new ());
+		GtkTreeViewColumn *column = gtk_tree_view_column_new ();
 		GtkCellRenderer *renderer = gtk_cell_renderer_text_new ();
 		gtk_tree_view_column_pack_start (column, renderer, TRUE);
 
@@ -268,37 +303,48 @@ rb_playlist_source_constructor (GType type, guint n_construct_properties,
 							 rb_playlist_source_track_cell_data_func,
 							 source, NULL);
 		rb_entry_view_append_column_custom (source->priv->songs, column, 
-						    _("Tra_ck"), "PlaylistTrack", NULL, NULL);
+						    _("Trac_k"), "PlaylistTrack", NULL, NULL);
 	}
 
 	rb_entry_view_append_column (source->priv->songs, RB_ENTRY_VIEW_COL_TITLE);
 	rb_entry_view_append_column (source->priv->songs, RB_ENTRY_VIEW_COL_GENRE);
 	rb_entry_view_append_column (source->priv->songs, RB_ENTRY_VIEW_COL_ARTIST);
 	rb_entry_view_append_column (source->priv->songs, RB_ENTRY_VIEW_COL_ALBUM);
+	rb_entry_view_append_column (source->priv->songs, RB_ENTRY_VIEW_COL_YEAR);
 	rb_entry_view_append_column (source->priv->songs, RB_ENTRY_VIEW_COL_DURATION);
 	rb_entry_view_append_column (source->priv->songs, RB_ENTRY_VIEW_COL_RATING);
 	rb_entry_view_append_column (source->priv->songs, RB_ENTRY_VIEW_COL_PLAY_COUNT);
 	rb_entry_view_append_column (source->priv->songs, RB_ENTRY_VIEW_COL_LAST_PLAYED);
-	rb_entry_view_set_columns_clickable (source->priv->songs, FALSE);
+	rb_entry_view_append_column (source->priv->songs, RB_ENTRY_VIEW_COL_FIRST_SEEN);
 
 	g_signal_connect_object (G_OBJECT (source->priv->songs), "show_popup",
 				 G_CALLBACK (rb_playlist_source_songs_show_popup_cb), source, 0);
+	g_signal_connect_object (G_OBJECT (source->priv->songs), "sort-order-changed",
+				 G_CALLBACK (rb_playlist_source_songs_sort_order_changed_cb), source, 0);
 		
 	g_signal_connect_object (G_OBJECT (source->priv->songs), "drag_data_received",
 				 G_CALLBACK (rb_playlist_source_drop_cb), source, 0);
 	gtk_drag_dest_set (GTK_WIDGET (source->priv->songs), GTK_DEST_DEFAULT_ALL,
 			   target_uri, G_N_ELEMENTS (target_uri), GDK_ACTION_COPY);
+
+	{
+		GtkIconTheme *theme = gtk_icon_theme_get_default();
+		gint size;
+
+		gtk_icon_size_lookup (GTK_ICON_SIZE_LARGE_TOOLBAR, &size, NULL);
+		source->priv->normal_pixbuf = gtk_icon_theme_load_icon (theme,
+									"stock_playlist",
+									size,
+									0, NULL);
+		source->priv->smartypants_pixbuf = gtk_icon_theme_load_icon (theme,
+									     "stock_smart-playlist",
+									     size,
+									     0, NULL);
+	}
 		
-	source->priv->normal_pixbuf = gtk_widget_render_icon (dummy,
-						       RB_STOCK_PLAYLIST,
-						       GTK_ICON_SIZE_LARGE_TOOLBAR,
-						       NULL);
-	source->priv->smartypants_pixbuf = gtk_widget_render_icon (dummy,
-								   RB_STOCK_AUTOMATIC_PLAYLIST,
-								   GTK_ICON_SIZE_LARGE_TOOLBAR,
-								   NULL);
-	gtk_widget_destroy (dummy);
-		
+	rb_entry_view_set_columns_clickable (source->priv->songs, FALSE);
+	source->priv->query_resetting = FALSE;
+
 	gtk_box_pack_start_defaults (GTK_BOX (source->priv->vbox), GTK_WIDGET (source->priv->songs));
 		
 	gtk_widget_show_all (GTK_WIDGET (source));
@@ -310,7 +356,8 @@ static void
 rb_playlist_source_songs_show_popup_cb (RBEntryView *view,
 					RBPlaylistSource *playlist_view)
 {
-	rb_bonobo_show_popup (GTK_WIDGET (view), PLAYLIST_SOURCE_SONGS_POPUP_PATH);
+	_rb_source_show_popup (RB_SOURCE (playlist_view), 
+			       PLAYLIST_SOURCE_SONGS_POPUP_PATH);
 }
 
 static void
@@ -320,25 +367,21 @@ rb_playlist_source_init (RBPlaylistSource *source)
 }
 
 static void
-rb_playlist_source_finalize (GObject *object)
+rb_playlist_source_dispose (GObject *object)
 {
 	RBPlaylistSource *source;
-
-	g_return_if_fail (object != NULL);
-	g_return_if_fail (RB_IS_PLAYLIST_SOURCE (object));
-
 	source = RB_PLAYLIST_SOURCE (object);
 
-	g_return_if_fail (source->priv != NULL);
+	if (source->priv) {
+		g_hash_table_destroy (source->priv->entries);
+		g_object_unref (source->priv->db);
+		g_free (source->priv->title);
+	
+		g_free (source->priv);
+		source->priv = NULL;
+	}
 
-	g_hash_table_destroy (source->priv->entries);
-
-	g_free (source->priv->title);
-	g_free (source->priv->status);
-
-	g_free (source->priv);
-
-	G_OBJECT_CLASS (parent_class)->finalize (object);
+	G_OBJECT_CLASS (rb_playlist_source_parent_class)->dispose (object);
 }
 
 static void
@@ -351,12 +394,15 @@ rb_playlist_source_set_property (GObject *object,
 
 	switch (prop_id)
 	{
-	case PROP_DB:
-		source->priv->db = g_value_get_object (value);
-		break;
 	case PROP_AUTOMATIC:
 		source->priv->automatic = g_value_get_boolean (value);
 		break;
+	case PROP_LOCAL:
+		source->priv->is_local = g_value_get_boolean (value);
+		break;
+	case PROP_ENTRY_TYPE:
+		source->priv->entry_type = g_value_get_int (value);
+		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 		break;
@@ -373,12 +419,21 @@ rb_playlist_source_get_property (GObject *object,
 
 	switch (prop_id)
 	{
-	case PROP_DB:
-		g_value_set_object (value, source->priv->db);
-		break;
 	case PROP_AUTOMATIC:
 		g_value_set_boolean (value, source->priv->automatic);
 		break;
+	case PROP_DIRTY:
+		g_value_set_boolean (value, source->priv->dirty);
+		break;
+	case PROP_QUERY_MODEL:
+		g_value_set_object (value, source->priv->model);
+		break;
+	case PROP_LOCAL:
+		g_value_set_boolean (value, source->priv->is_local);
+		break;
+	case PROP_ENTRY_TYPE:
+		g_value_set_int (value, source->priv->entry_type);
+		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 		break;
@@ -386,14 +441,16 @@ rb_playlist_source_get_property (GObject *object,
 }
 
 RBSource *
-rb_playlist_source_new (RhythmDB *db, gboolean automatic)
+rb_playlist_source_new (RBShell *shell, gboolean automatic, gboolean local, RhythmDBEntryType entry_type)
 {
 	RBSource *source;
 	
 	source = RB_SOURCE (g_object_new (RB_TYPE_PLAYLIST_SOURCE,
 					  "name", _("Unknown"),
-					  "db", db,
+					  "shell", shell,
 					  "automatic", automatic,
+					  "is-local", local,
+					  "entry-type", entry_type,
 					  NULL));
 
 	return source;
@@ -408,72 +465,67 @@ rb_playlist_source_entry_added_cb (RhythmDB *db, RhythmDBEntry *entry,
 	if (source->priv->automatic)
 		return;
 	
-	location = rhythmdb_entry_get_string (db, entry,
-					      RHYTHMDB_PROP_LOCATION);
+	location = entry->location;
 	if (g_hash_table_lookup (source->priv->entries, location)) {
 		rhythmdb_query_model_add_entry (source->priv->model, entry);
+		source->priv->dirty = TRUE;
 	}
 }
 
-RhythmDBQueryModel *
-rb_playlist_source_get_model (RBPlaylistSource *source)
-{
-	return source->priv->model;
-}
-
 void
 rb_playlist_source_set_query (RBPlaylistSource *source,
 			      GPtrArray *query,
 			      guint limit_count,
-			      guint limit_mb)
+			      guint limit_mb,
+			      guint limit_time,
+			      const char *sort_key,
+			      gint sort_direction)
 {
-	RhythmDBQueryModel *query_model;
-	GtkTreeModel *model;
-	
 	g_assert (source->priv->automatic);
 
-	source->priv->model = query_model
-		= g_object_new (RHYTHMDB_TYPE_QUERY_MODEL,
-				"db", source->priv->db,
-				"max-count", limit_count,
-				"max-size", limit_mb, 
-				NULL);
+	source->priv->query_resetting = TRUE;
 
-	model = GTK_TREE_MODEL (query_model);
+	/* playlists that aren't limited, with a particular sort order, are user-orderable */
+	rb_entry_view_set_columns_clickable (source->priv->songs, (limit_count == 0 && limit_mb == 0));
+	rb_entry_view_set_sorting_order (source->priv->songs, sort_key, sort_direction);
 
-	rb_entry_view_set_model (source->priv->songs, RHYTHMDB_QUERY_MODEL (query_model));
-
-	rhythmdb_read_lock (source->priv->db);
-	rhythmdb_do_full_query_async_parsed (source->priv->db, model, query);
-
-	g_object_unref (G_OBJECT (query_model));
-	rb_entry_view_poll_model (source->priv->songs);
+	rb_playlist_source_do_query (source, query, limit_count, limit_mb, limit_time);
+	rhythmdb_query_free (query);
+	source->priv->query_resetting = FALSE;
 }
 
 void
 rb_playlist_source_get_query (RBPlaylistSource *source,
 			      GPtrArray **query,
 			      guint *limit_count,
-			      guint *limit_mb)
+			      guint *limit_mb,
+			      guint *limit_time,
+			      const char **sort_key,
+			      gint *sort_direction)
 {
 	g_assert (source->priv->automatic);
 
 	g_object_get (G_OBJECT (source->priv->model),
 		      "query", query,
 		      "max-count", limit_count,
-		      "max-size", limit_mb, NULL);
+		      "max-size", limit_mb,
+		      "max-time", limit_time,
+		      NULL);
+
+	rb_entry_view_get_sorting_order (source->priv->songs, sort_key, sort_direction);
 }
 
-static const char *
+static char *
 impl_get_status (RBSource *asource)
 {
 
 	RBPlaylistSource *source = RB_PLAYLIST_SOURCE (asource);
-	g_free (source->priv->status);
-	source->priv->status = rhythmdb_compute_status_normal (rb_entry_view_get_num_entries (source->priv->songs),
-							       rb_entry_view_get_duration (source->priv->songs),
-							       rb_entry_view_get_total_size (source->priv->songs));
-	return source->priv->status;
+	gchar *status;
+
+	status = rhythmdb_compute_status_normal (rb_entry_view_get_num_entries (source->priv->songs),
+						 rb_entry_view_get_duration (source->priv->songs),
+						 rb_entry_view_get_total_size (source->priv->songs));
+	return status;
 }
 
 static const char *
@@ -574,9 +626,8 @@ impl_receive_drag (RBSource *asource, GtkSelectionData *data)
 	GList *list;
 	RBPlaylistSource *source = RB_PLAYLIST_SOURCE (asource);
 
-
         if (data->type == gdk_atom_intern ("text/uri-list", TRUE)) {
-                list = gnome_vfs_uri_list_parse (data->data);
+                list = gnome_vfs_uri_list_parse ((char *) data->data);
 
                 if (list != NULL)
                         rb_playlist_source_add_list_uri (source, list);
@@ -584,22 +635,46 @@ impl_receive_drag (RBSource *asource, GtkSelectionData *data)
                         return FALSE;
 
 	} else {
-		GPtrArray *query;
-		GPtrArray *subquery;
+		GPtrArray *subquery = NULL;
+		gchar **names = g_strsplit ((char *)data->data, "\r\n", 0);
+		guint propid = rb_playlist_source_drag_atom_to_prop (data->type);
+		int i;
 
-		subquery = rhythmdb_query_parse (source->priv->db,
-						 RHYTHMDB_QUERY_PROP_EQUALS,
-						 rb_playlist_source_drag_atom_to_prop (data->type),
-						 data->data,
-						 RHYTHMDB_QUERY_END);
-		query = rhythmdb_query_parse (source->priv->db,
-					      RHYTHMDB_QUERY_PROP_EQUALS,
-					      RHYTHMDB_PROP_TYPE,
-					      RHYTHMDB_ENTRY_TYPE_SONG,
-					      RHYTHMDB_QUERY_SUBQUERY,
-					      subquery,
-					      RHYTHMDB_QUERY_END);
-                rb_playlist_source_set_query (source, query, 0, 0);
+		for (i=0; names[i]; i++) {
+			if (subquery == NULL) 
+				subquery = rhythmdb_query_parse (source->priv->db,
+								 RHYTHMDB_QUERY_PROP_EQUALS,
+								 propid,
+								 names[i],
+								 RHYTHMDB_QUERY_END);
+			else
+				rhythmdb_query_append (source->priv->db,
+						       subquery,
+						       RHYTHMDB_QUERY_DISJUNCTION,
+						       RHYTHMDB_QUERY_PROP_EQUALS,
+						       propid,
+						       names[i],
+						       RHYTHMDB_QUERY_END);
+		}
+
+		g_strfreev (names);
+
+		if (subquery) {
+			RhythmDBEntryType qtype = RHYTHMDB_ENTRY_TYPE_SONG;
+			GPtrArray *query;
+			
+			if (source->priv->entry_type != -1)
+				qtype = source->priv->entry_type;
+
+			query = rhythmdb_query_parse (source->priv->db,
+						      RHYTHMDB_QUERY_PROP_EQUALS,
+						      RHYTHMDB_PROP_TYPE,
+						      qtype,
+						      RHYTHMDB_QUERY_SUBQUERY,
+						      subquery,
+						      RHYTHMDB_QUERY_END);
+			rb_playlist_source_set_query (source, query, 0, 0, 0, NULL, 0);
+		}
 	}
 
         return TRUE;
@@ -609,10 +684,12 @@ static gboolean
 impl_show_popup (RBSource *asource)
 {
 	RBPlaylistSource *source = RB_PLAYLIST_SOURCE (asource);
-	if (source->priv->automatic)
-		rb_bonobo_show_popup (GTK_WIDGET (source), PLAYLIST_SOURCE_AUTOMATIC_POPUP_PATH);
-	else
-		rb_bonobo_show_popup (GTK_WIDGET (source), PLAYLIST_SOURCE_POPUP_PATH);
+
+	if (source->priv->automatic) {
+		_rb_source_show_popup (RB_SOURCE (asource), PLAYLIST_SOURCE_AUTOMATIC_POPUP_PATH);
+	} else {
+		_rb_source_show_popup (RB_SOURCE (asource), PLAYLIST_SOURCE_POPUP_PATH);
+	}
 	return TRUE;
 }
 
@@ -642,32 +719,64 @@ rb_playlist_source_drop_cb (GtkWidget *widget,
 	gtk_drag_finish (context, TRUE, FALSE, time);
 }
 
+
+
+static void
+rb_playlist_source_add_location_swapped (const char *uri, RBPlaylistSource *source)
+{
+	rb_playlist_source_add_location (source, uri);
+}
+
 void
 rb_playlist_source_add_location (RBPlaylistSource *source,
 				 const char *location)
 {
 	RhythmDBEntry *entry;
-
-	rhythmdb_read_lock (source->priv->db);
-	if (g_hash_table_lookup (source->priv->entries, location)) {
-		rhythmdb_read_unlock (source->priv->db);
+	
+	entry = rhythmdb_entry_lookup_by_location (source->priv->db, location);
+	if (entry != NULL && 
+	    source->priv->entry_type != -1 &&
+	    rhythmdb_entry_get_ulong (entry, RHYTHMDB_PROP_TYPE) != source->priv->entry_type) {
+		rb_debug ("attempting to add an entry of the wrong type to playlist");
 		return;
 	}
 
-	g_hash_table_insert (source->priv->entries,
-			     g_strdup (location), GINT_TO_POINTER (1));
+	if (rb_uri_is_directory (location)) {
+		rb_uri_handle_recursively (location,
+					   (GFunc) rb_playlist_source_add_location_swapped,
+					   NULL,
+					   source);
+	} else {
+		if (g_hash_table_lookup (source->priv->entries, location)) {
+			return;
+		}
 
-	entry = rhythmdb_entry_lookup_by_location (source->priv->db, location);
-	if (entry)
-		rhythmdb_entry_ref_unlocked (source->priv->db, entry);
-	rhythmdb_read_unlock (source->priv->db);
+		g_hash_table_insert (source->priv->entries,
+				     g_strdup (location), GINT_TO_POINTER (1));
 
-	if (entry != NULL) {
-		rhythmdb_query_model_add_entry (source->priv->model, entry);
-		rhythmdb_entry_unref (source->priv->db, entry);
+		if (entry != NULL) {
+			rhythmdb_query_model_add_entry (source->priv->model, entry);
+
+			source->priv->dirty = TRUE;
+		}
 	}
 }
 
+void
+rb_playlist_source_add_locations (RBPlaylistSource *source,
+				  GList *locations)
+{
+	GList *l;
+
+	for (l = locations; l; l = l->next) {
+		const gchar *uri = (const gchar *)l->data;
+		
+		rb_playlist_source_add_location (RB_PLAYLIST_SOURCE (source), 
+						 uri);
+	}
+}
+
+
 void
 rb_playlist_source_remove_location (RBPlaylistSource *source,
 				    const char *location)
@@ -675,41 +784,30 @@ rb_playlist_source_remove_location (RBPlaylistSource *source,
 	RhythmDBEntry *entry;
 
 	g_return_if_fail (g_hash_table_lookup (source->priv->entries, location) != NULL);
-	g_hash_table_remove (source->priv->entries,
-			     location);
-	rhythmdb_read_lock (source->priv->db);
+
+	g_hash_table_remove (source->priv->entries, location);
 	entry = rhythmdb_entry_lookup_by_location (source->priv->db, location);
-	rhythmdb_read_unlock (source->priv->db);
-	if (entry != NULL)
-		rhythmdb_query_model_remove_entry (source->priv->model, entry);
+	if (entry != NULL) {
+		gboolean removed;
+		
+		removed = rhythmdb_query_model_remove_entry (source->priv->model, entry);
+		g_assert (removed); /* if this fails, the model and the playlist are out of sync */
+		source->priv->dirty = TRUE;
+	}
 }
 
 void
 rb_playlist_source_add_entry (RBPlaylistSource *source,
 			      RhythmDBEntry *entry)
 {
-	const char *location;
-
-	rhythmdb_read_lock (source->priv->db);
-	location = rhythmdb_entry_get_string (source->priv->db, entry,
-					      RHYTHMDB_PROP_LOCATION);
-	rhythmdb_read_unlock (source->priv->db);
-
-	rb_playlist_source_add_location (source, location);
+	rb_playlist_source_add_location (source, entry->location);
 }
 
 void
 rb_playlist_source_remove_entry (RBPlaylistSource *source,
 			         RhythmDBEntry *entry)
 {
-	const char *location;
-
-	rhythmdb_read_lock (source->priv->db);
-	location = rhythmdb_entry_get_string (source->priv->db, entry,
-					      RHYTHMDB_PROP_LOCATION);
-	rhythmdb_read_unlock (source->priv->db);
-
-	rb_playlist_source_remove_location (source, location);
+	rb_playlist_source_remove_location (source, entry->location);
 }
 
 static void
@@ -721,7 +819,8 @@ rb_playlist_source_add_list_uri (RBPlaylistSource *source,
 	g_return_if_fail (list != NULL);
 
 	for (i = list; i != NULL; i = g_list_next (i))
-		uri_list = g_list_append (uri_list, gnome_vfs_uri_to_string ((const GnomeVFSURI *) i->data, 0));
+		uri_list = g_list_prepend (uri_list, gnome_vfs_uri_to_string ((const GnomeVFSURI *) i->data, 0));
+	uri_list = g_list_reverse (uri_list);
 
 	gnome_vfs_uri_list_free (list);
 
@@ -731,9 +830,8 @@ rb_playlist_source_add_list_uri (RBPlaylistSource *source,
 	for (i = uri_list; i != NULL; i = i->next) {
 		char *uri = i->data;
 		if (uri != NULL) {
-			rhythmdb_read_lock (source->priv->db);
+			rhythmdb_add_uri (source->priv->db, uri);
 			rb_playlist_source_add_location (source, uri);
-			rhythmdb_read_unlock (source->priv->db);
 		}
 
 		g_free (uri);
@@ -743,106 +841,165 @@ rb_playlist_source_add_list_uri (RBPlaylistSource *source,
 }
 
 static void
-playlist_iter_func (GtkTreeModel *model, GtkTreeIter *iter, char **uri, char **title)
+playlist_iter_func (GtkTreeModel *model, GtkTreeIter *iter, char **uri, char **title, gpointer user_data)
 {
-	RhythmDB *db;
 	RhythmDBEntry *entry;
 
-	g_object_get (G_OBJECT (model), "db", &db, NULL);
-	
 	gtk_tree_model_get (model, iter, 0, &entry, -1);
 
-	rhythmdb_read_lock (db);
-
-	*uri = g_strdup (rhythmdb_entry_get_string (db, entry, RHYTHMDB_PROP_LOCATION));
-	*title = g_strdup (rhythmdb_entry_get_string (db, entry, RHYTHMDB_PROP_TITLE));
-
-	rhythmdb_read_unlock (db);
+	*uri = g_strdup (entry->location);
+	*title = g_strdup (rb_refstring_get (entry->title));
 }
 
 void
 rb_playlist_source_save_playlist (RBPlaylistSource *source, const char *uri)
 {
-	RBPlaylist *playlist;
+	TotemPlParser *playlist;
 	GError *error = NULL;
+	char *name;
+
+
 	rb_debug ("saving playlist");
+	playlist = totem_pl_parser_new ();
 
-	playlist = rb_playlist_new ();
+	g_object_get (G_OBJECT (source), "name", &name, NULL);
 
-	rb_playlist_write (playlist, GTK_TREE_MODEL (source->priv->model),
-			   playlist_iter_func, uri, &error);
+	totem_pl_parser_write_with_title (playlist, GTK_TREE_MODEL (source->priv->model),
+					  playlist_iter_func, uri, name,
+					  TOTEM_PL_PARSER_PLS, NULL, &error);
 	if (error != NULL)
-		rb_error_dialog ("%s", error->message);
+		rb_error_dialog (NULL, _("Couldn't save playlist"),
+				 "%s", error->message);
+}
+
+static void
+burn_playlist_iter_func (GtkTreeModel *model, GtkTreeIter *iter, char **uri, char **artist, char **title, gulong *duration)
+{
+	RhythmDBEntry *entry;
+
+	gtk_tree_model_get (model, iter, 0, &entry, -1);
+
+	*uri = g_strdup (entry->location);
+	*artist = g_strdup (rb_refstring_get (entry->artist));
+	*title = g_strdup (rb_refstring_get (entry->title));
+	*duration = entry->duration;
+}
+
+void
+rb_playlist_source_burn_playlist (RBPlaylistSource *source)
+{
+	GtkWidget *recorder;
+	char *name;
+	RBShell *shell;
+
+	/* don't burn if the playlist is empty */
+	if (gtk_tree_model_iter_n_children (GTK_TREE_MODEL (source->priv->model), NULL) == 0)
+		return;
+
+	rb_debug ("burning playlist");
+
+	g_object_get (source, "name", &name, "shell", &shell, NULL);
+
+	recorder = rb_playlist_source_recorder_new (gtk_widget_get_toplevel (GTK_WIDGET (source)),
+						    shell,
+						    name);
+	g_object_unref (shell);
+	g_free (name);
+
+	rb_playlist_source_recorder_add_from_model (RB_PLAYLIST_SOURCE_RECORDER (recorder),
+						    GTK_TREE_MODEL (source->priv->model),
+						    burn_playlist_iter_func,
+						    NULL);
+
+        g_signal_connect (recorder,
+			  "response",
+			  G_CALLBACK (gtk_widget_destroy),
+			  NULL);
+
+	gtk_widget_show (recorder);
 }
 
 RBSource *
-rb_playlist_source_new_from_xml	(RhythmDB *db,
+rb_playlist_source_new_from_xml	(RBShell *shell,
 				 xmlNodePtr node)
 {
 	RBPlaylistSource *source;
 	xmlNodePtr child;
-	char *tmp;
+	xmlChar *tmp;
 
-	source = RB_PLAYLIST_SOURCE (rb_playlist_source_new (db, FALSE));
+	source = RB_PLAYLIST_SOURCE (rb_playlist_source_new (shell, FALSE, TRUE, RHYTHMDB_ENTRY_TYPE_SONG));
 
-	tmp = xmlGetProp (node, "type");
-	if (!strcmp (tmp, "automatic"))
+	tmp = xmlGetProp (node, RB_PLAYLIST_TYPE);
+	if (!xmlStrcmp (tmp, RB_PLAYLIST_AUTOMATIC))
 		source->priv->automatic = TRUE;
 	g_free (tmp);
 
-	tmp = xmlGetProp (node, "name");
+	tmp = xmlGetProp (node, RB_PLAYLIST_NAME);
 	g_object_set (G_OBJECT (source), "name", tmp, NULL);
 	g_free (tmp);
 
-	tmp = xmlGetProp (node, "internal-name");
-	if (!tmp) {
-		GTimeVal serial;
-		/* Hm.  Upgrades. */
-		g_get_current_time (&serial);
-		tmp = g_strdup_printf ("<playlist:%ld:%ld>",
-				       serial.tv_sec, serial.tv_usec);
-	}
-	g_object_set (G_OBJECT (source), "internal-name", tmp, NULL);
-	g_free (tmp);
-
 	if (source->priv->automatic) {
 		GPtrArray *query;
-		gchar *limit_str;
-		guint limit_count = 0;
-		guint limit_mb = 0;
+		gint limit_count = 0, limit_mb = 0, limit_time = 0;
+		gchar *sort_key = NULL;
+		gint sort_direction = 0;
 
 		child = node->children;
 		while (xmlNodeIsText (child))
 			child = child->next;
 
-		query = rhythmdb_query_deserialize (db, child);
-		limit_str = xmlGetProp (node, "limit-count");
-		if (!limit_str) /* Backwards compatibility */
-			limit_str = xmlGetProp (node, "limit");
-		if (limit_str) {
-			limit_count = atoi (limit_str);
-			g_free (limit_str);
+		query = rhythmdb_query_deserialize (source->priv->db, child);
+		tmp = xmlGetProp (node, RB_PLAYLIST_LIMIT_COUNT);
+		if (!tmp) /* Backwards compatibility */
+			tmp = xmlGetProp (node, RB_PLAYLIST_LIMIT);
+		if (tmp) {
+			limit_count = atoi ((char*) tmp);
+			g_free (tmp);
 		}
-		limit_str = xmlGetProp (node, "limit-size");
-		if (limit_str) {
-			limit_mb = atoi (limit_str);
-			g_free (limit_str);
+		tmp = xmlGetProp (node, RB_PLAYLIST_LIMIT_SIZE);
+		if (tmp) {
+			limit_mb = atoi ((char*) tmp);
+			g_free (tmp);
 		}
+		tmp = xmlGetProp (node, RB_PLAYLIST_LIMIT_TIME);
+		if (tmp) {
+			limit_time = atoi ((char*) tmp);
+			g_free (tmp);
+		}
+
+		sort_key = (gchar*) xmlGetProp (node, RB_PLAYLIST_SORT_KEY);
+		if (sort_key && *sort_key) {
+			tmp = xmlGetProp (node, RB_PLAYLIST_SORT_DIRECTION);
+			if (tmp) {
+				sort_direction = atoi ((char*) tmp);
+				g_free (tmp);
+			}
+		} else {
+			g_free (sort_key);
+			sort_key = NULL;
+			sort_direction = 0;
+		}
+
 		rb_playlist_source_set_query (source, query,
 					      limit_count,
-					      limit_mb);
+					      limit_mb,
+					      limit_time,
+					      sort_key,
+					      sort_direction);
+		g_free (sort_key);
 	} else {
 		for (child = node->children; child; child = child->next) {
-			char *location;
+			xmlChar *location;
 
 			if (xmlNodeIsText (child))
 				continue;
 		
-			if (strcmp (child->name, "location"))
+			if (xmlStrcmp (child->name, RB_PLAYLIST_LOCATION))
 				continue;
 		
 			location = xmlNodeGetContent (child);
-			rb_playlist_source_add_location (source, location);
+			rb_playlist_source_add_location (source,
+							 (char *) location);
 		}
 	}
 
@@ -853,56 +1010,130 @@ void
 rb_playlist_source_save_to_xml (RBPlaylistSource *source, xmlNodePtr parent_node)
 {
 	xmlNodePtr node;
-	char *name;
-	char *internal_name;
+	xmlChar *name;
 	GtkTreeIter iter;
 
-	node = xmlNewChild (parent_node, NULL, "playlist", NULL);
-	g_object_get (G_OBJECT (source), "name", &name,
-		      "internal-name", &internal_name, NULL);
-	xmlSetProp (node, "name", name);
-	xmlSetProp (node, "internal-name", internal_name);
-	xmlSetProp (node, "type", source->priv->automatic ? "automatic" : "static");
-	
+	node = xmlNewChild (parent_node, NULL, RB_PLAYLIST_PLAYLIST, NULL);
+	g_object_get (G_OBJECT (source), "name", &name, NULL);
+	xmlSetProp (node, RB_PLAYLIST_NAME, name);
+	xmlSetProp (node, RB_PLAYLIST_TYPE, source->priv->automatic ? RB_PLAYLIST_AUTOMATIC : RB_PLAYLIST_STATIC);
+
 	if (!source->priv->automatic) {
 		if (!gtk_tree_model_get_iter_first (GTK_TREE_MODEL (source->priv->model),
 						    &iter))
 			return;
 		do { 
-			xmlNodePtr child_node = xmlNewChild (node, NULL, "location", NULL);
+			xmlNodePtr child_node = xmlNewChild (node, NULL, RB_PLAYLIST_LOCATION, NULL);
 			RhythmDBEntry *entry;
-			const char *location;
-			char *encoded;
+			xmlChar *encoded;
 
 			gtk_tree_model_get (GTK_TREE_MODEL (source->priv->model), &iter, 0, &entry, -1);
 
-			rhythmdb_read_lock (source->priv->db);
-			location = rhythmdb_entry_get_string (source->priv->db, entry,
-							      RHYTHMDB_PROP_LOCATION);
-			rhythmdb_read_unlock (source->priv->db);
-			
-			encoded = xmlEncodeEntitiesReentrant (NULL, location);			
-			
+			encoded = xmlEncodeEntitiesReentrant (NULL, BAD_CAST entry->location);
+
 			xmlNodeSetContent (child_node, encoded);
 			g_free (encoded);
 		} while (gtk_tree_model_iter_next (GTK_TREE_MODEL (source->priv->model),
 						   &iter));
 	} else {
 		GPtrArray *query;
-		guint max_count;
-		guint max_size_mb;
-		char *limit_str;
+		guint max_count, max_size_mb, max_time;
+		const gchar *sort_key;
+		gint sort_direction;
+		gchar *temp_str;
+
+		rb_playlist_source_get_query (source,
+					      &query,
+					      &max_count, &max_size_mb, &max_time,
+					      &sort_key, &sort_direction);
+		temp_str = g_strdup_printf ("%d", max_count);
+		xmlSetProp (node, RB_PLAYLIST_LIMIT_COUNT, BAD_CAST temp_str);
+		g_free (temp_str);
+		temp_str = g_strdup_printf ("%d", max_size_mb);
+		xmlSetProp (node, RB_PLAYLIST_LIMIT_SIZE, BAD_CAST temp_str);
+		g_free (temp_str);
+		temp_str = g_strdup_printf ("%d", max_time);
+		xmlSetProp (node, RB_PLAYLIST_LIMIT_TIME, BAD_CAST temp_str);
+		g_free (temp_str);
+
+		if (sort_key && *sort_key) {
+			xmlSetProp (node, RB_PLAYLIST_SORT_KEY, BAD_CAST sort_key);
+			temp_str = g_strdup_printf ("%d", sort_direction);
+			xmlSetProp (node, RB_PLAYLIST_SORT_DIRECTION, BAD_CAST temp_str);
+			g_free (temp_str);
+		}
 
-		g_object_get (G_OBJECT (source->priv->model),
-			      "max-count", &max_count,
-			      "max-size", &max_size_mb,
-			      "query", &query, NULL);
-		limit_str = g_strdup_printf ("%d", max_count);
-		xmlSetProp (node, "limit-count", limit_str);
-		g_free (limit_str);
-		limit_str = g_strdup_printf ("%d", max_size_mb);
-		xmlSetProp (node, "limit-size", limit_str);
-		g_free (limit_str);
 		rhythmdb_query_serialize (source->priv->db, query, node);
 	}
+
+	source->priv->dirty = FALSE;
+}
+
+static void
+rb_playlist_source_songs_sort_order_changed_cb (RBEntryView *view, RBPlaylistSource *source)
+{
+	GPtrArray *query;
+	guint limit_count, limit_mb, limit_time;
+
+	g_assert (source->priv->automatic);
+
+	/* don't process this if we are in the middle of setting a query */
+	if (source->priv->query_resetting)
+		return;
+	rb_debug ("sort order changed");
+
+	/* need to re-run query with the same settings*/
+	g_object_get (G_OBJECT (source->priv->model),
+		      "query", &query,
+		      "max-count", &limit_count,
+		      "max-size", &limit_mb,
+		      "max-size", &limit_time,
+		      NULL);
+
+	rb_playlist_source_do_query (source, query, limit_count, limit_mb, limit_time);
+	rhythmdb_query_free (query);
+}
+
+static void
+rb_playlist_source_do_query (RBPlaylistSource *source,
+			      GPtrArray *query,
+			      guint limit_count,
+			      guint limit_mb,
+			      guint limit_time)
+{
+	g_assert (source->priv->automatic);
+
+	source->priv->model = g_object_new (RHYTHMDB_TYPE_QUERY_MODEL,
+				    "db", source->priv->db,
+				    "max-count", limit_count,
+				    "max-size", limit_mb, 
+				    "max-time", limit_time, 
+				    NULL);
+
+	rb_entry_view_set_model (source->priv->songs, source->priv->model);
+	rhythmdb_do_full_query_async_parsed (source->priv->db, GTK_TREE_MODEL (source->priv->model), query);
+
+	/* emit notification the the property has changed */
+	g_object_notify (G_OBJECT (source), "query-model");
+}
+
+static void
+rb_playlist_source_non_entry_dropped (GtkTreeModel *model,
+				      const char *uri,
+				      RBPlaylistSource *source)
+{
+	rb_playlist_source_add_location (source, uri);
+}
+
+static void
+rb_playlist_source_row_inserted (GtkTreeModel *model,
+				 GtkTreePath *path,
+				 GtkTreeIter *iter,
+				 RBPlaylistSource *source)
+{
+	RhythmDBEntry *entry;
+
+	gtk_tree_model_get (model, iter, 0, &entry, -1);
+
+	rb_playlist_source_add_location (source, rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_LOCATION));
 }
