@@ -32,6 +32,7 @@
 #include "rb-entry-view.h"
 #include "rb-dialog.h"
 #include "rb-debug.h"
+#include "rb-util.h"
 #include "rhythmdb.h"
 #include "rhythmdb-query-model.h"
 #include "rb-cell-renderer-pixbuf.h"
@@ -79,6 +80,11 @@ static void rb_entry_view_row_changed_cb (GtkTreeModel *model,
 					  GtkTreePath *path,
 					  GtkTreeIter *iter,
 					  RBEntryView *view);
+static void rb_entry_view_rows_reordered_cb (GtkTreeModel *model,
+					     GtkTreePath *path,
+					     GtkTreeIter *iter,
+					     gint *order,
+					     RBEntryView *view);
 static gboolean emit_entry_changed (RBEntryView *view);
 static void queue_changed_sig (RBEntryView *view);
 static void rb_entry_view_sync_columns_visible (RBEntryView *view);
@@ -143,7 +149,6 @@ struct RBEntryViewPrivate
 	
 	char *sorting_key;
 	guint sorting_gconf_notification_id;
-	GList *clickable_columns;
 	GtkTreeViewColumn *sorting_column;
 	gint sorting_order;
 	char *sorting_column_name;
@@ -210,7 +215,7 @@ type_ahead_search_func (GtkTreeModel *model, gint column,
 	gboolean res;
 
 	gtk_tree_model_get (model, iter, 0, &entry, -1);
-	folded = g_utf8_casefold (key, -1);
+	folded = rb_search_fold (key);
 	res = (strstr (rb_refstring_get_folded (entry->title), folded) == NULL);
 	g_free (folded);
 	return res;
@@ -450,8 +455,6 @@ rb_entry_view_finalize (GObject *object)
 	if (view->priv->sorting_gconf_notification_id > 0)
 		eel_gconf_notification_remove (view->priv->sorting_gconf_notification_id);
 
-	g_list_free (view->priv->clickable_columns);
-
 	g_hash_table_destroy (view->priv->propid_column_map);
 	g_hash_table_destroy (view->priv->column_sort_data_map);
 	g_hash_table_destroy (view->priv->column_key_map);
@@ -491,7 +494,6 @@ rb_entry_view_set_property (GObject *object,
 		struct RBEntryViewColumnSortData *sort_data;
 		
 		if (view->priv->model) {
-			rhythmdb_query_model_cancel (view->priv->model);
 			rhythmdb_query_model_set_connected (RHYTHMDB_QUERY_MODEL (view->priv->model), FALSE);
 			g_signal_handlers_disconnect_by_func (G_OBJECT (view->priv->model),
 							      G_CALLBACK (rb_entry_view_row_inserted_cb),
@@ -502,6 +504,9 @@ rb_entry_view_set_property (GObject *object,
 			g_signal_handlers_disconnect_by_func (G_OBJECT (view->priv->model),
 							      G_CALLBACK (rb_entry_view_row_changed_cb),
 							      view);
+			g_signal_handlers_disconnect_by_func (G_OBJECT (view->priv->model),
+							      G_CALLBACK (rb_entry_view_rows_reordered_cb),
+							      view);
 		}
 		new_model = g_value_get_object (value);
 
@@ -523,6 +528,11 @@ rb_entry_view_set_property (GObject *object,
 					 G_CALLBACK (rb_entry_view_row_changed_cb),
 					 view,
 					 0);
+		g_signal_connect_object (G_OBJECT (new_model),
+					 "rows_reordered",
+					 G_CALLBACK (rb_entry_view_rows_reordered_cb),
+					 view,
+					 0);
 
 		if (view->priv->sorting_column) {
 			sort_data = g_hash_table_lookup (view->priv->column_sort_data_map,
@@ -739,6 +749,27 @@ struct RBEntryViewCellDataFuncData {
 	RhythmDBPropType propid;
 };
 
+static gint
+rb_entry_view_location_sort_func (RhythmDBEntry *a, RhythmDBEntry *b,
+				  RBEntryView *view)
+{
+	const char *a_val;
+	const char *b_val;
+
+	a_val = rhythmdb_entry_get_string (a, RHYTHMDB_PROP_LOCATION);
+	b_val = rhythmdb_entry_get_string (b, RHYTHMDB_PROP_LOCATION);
+
+	if (a_val == NULL) {
+		if (b_val == NULL)
+			return 0;
+		else
+			return -1;
+	} else if (b_val == NULL)
+		return 1;
+	else
+		return strcmp (a_val, b_val);
+}
+
 static gint
 rb_entry_view_album_sort_func (RhythmDBEntry *a, RhythmDBEntry *b,
 			       RBEntryView *view)
@@ -772,7 +803,7 @@ rb_entry_view_album_sort_func (RhythmDBEntry *a, RhythmDBEntry *b,
 	if (a->tracknum != b->tracknum)
 		return (a->tracknum < b->tracknum ? -1 : 1);
 
-	/* And finally by title */
+	/*  by title */
 	a_val = rhythmdb_entry_get_string (a, RHYTHMDB_PROP_TITLE_SORT_KEY);
 	b_val = rhythmdb_entry_get_string (b, RHYTHMDB_PROP_TITLE_SORT_KEY);
 
@@ -784,7 +815,7 @@ rb_entry_view_album_sort_func (RhythmDBEntry *a, RhythmDBEntry *b,
 	} else if (b_val == NULL)
 		return 1;
 	else
-		return strcmp (a_val, b_val);
+		return rb_entry_view_location_sort_func (a, b, view);
 }
 
 static gint
@@ -823,7 +854,7 @@ rb_entry_view_genre_sort_func (RhythmDBEntry *a, RhythmDBEntry *b,
 	gint ret;
 
 	a_val = rhythmdb_entry_get_string (a, RHYTHMDB_PROP_GENRE_SORT_KEY);
-	b_val = rhythmdb_entry_get_string (a, RHYTHMDB_PROP_GENRE_SORT_KEY);
+	b_val = rhythmdb_entry_get_string (b, RHYTHMDB_PROP_GENRE_SORT_KEY);
 
 	if (a_val == NULL) {
 		if (b_val == NULL)
@@ -853,14 +884,14 @@ rb_entry_view_double_ceiling_sort_func (RhythmDBEntry *a, RhythmDBEntry *b,
 				       struct RBEntryViewCellDataFuncData *data)
 {
 	gdouble a_val, b_val;
-	gint ret;
 
 	a_val = ceil (rhythmdb_entry_get_double (a, data->propid));
 	b_val = ceil (rhythmdb_entry_get_double (b, data->propid));
 
-	ret = (a_val == b_val ? 0 : (a_val > b_val ? 1 : -1));
-
-	return ret;
+	if (a_val != b_val)
+		return (a_val > b_val ? 1 : -1);
+	else
+		return rb_entry_view_location_sort_func (a, b, data->view);
 }
 
 static gint
@@ -868,14 +899,33 @@ rb_entry_view_ulong_sort_func (RhythmDBEntry *a, RhythmDBEntry *b,
 			       struct RBEntryViewCellDataFuncData *data)
 {
 	gulong a_val, b_val;
-	gint ret;
 
 	a_val = rhythmdb_entry_get_ulong (a, data->propid);
 	b_val = rhythmdb_entry_get_ulong (b, data->propid);
 
-	ret = (a_val == b_val ? 0 : (a_val > b_val ? 1 : -1));
+	if (a_val != b_val)
+		return (a_val > b_val ? 1 : -1);
+	else
+		return rb_entry_view_location_sort_func (a, b, data->view);
+}
 
-	return ret;
+static gint
+rb_entry_view_date_sort_func (RhythmDBEntry *a, RhythmDBEntry *b,
+			      RBEntryView *view)
+{
+	gulong a_val, b_val;
+	gint ret;
+
+	a_val = rhythmdb_entry_get_ulong (a, RHYTHMDB_PROP_DATE);
+	b_val = rhythmdb_entry_get_ulong (b, RHYTHMDB_PROP_DATE);
+
+	ret = (a_val == b_val ? 0 : (a_val > b_val ? 1 : -1));
+	if (a_val > b_val)
+		return 1;
+	else if (a_val < b_val)
+		return -1;
+	else
+		return rb_entry_view_album_sort_func (a, b, view);
 }
 
 static gint
@@ -884,19 +934,25 @@ rb_entry_view_string_sort_func (RhythmDBEntry *a, RhythmDBEntry *b,
 {
 	const char *a_val;
 	const char *b_val;
+	gint ret;
 
 	a_val = rhythmdb_entry_get_string (a, data->propid);
 	b_val = rhythmdb_entry_get_string (b, data->propid);
 
 	if (a_val == NULL) {
 		if (b_val == NULL)
-			return 0;
+			ret = 0;
 		else
-			return -1;
+			ret = -1;
 	} else if (b_val == NULL)
-		return 1;
+		ret = 1;
 	else
-		return strcmp (a_val, b_val);
+		ret = strcmp (a_val, b_val);
+
+	if (ret != 0)
+		return ret;
+	else
+		return rb_entry_view_location_sort_func (a, b, data->view);
 }
 
 static void
@@ -1005,6 +1061,29 @@ rb_entry_view_duration_cell_data_func (GtkTreeViewColumn *column, GtkCellRendere
 	g_free (str);
 }
 
+static void
+rb_entry_view_year_cell_data_func (GtkTreeViewColumn *column, GtkCellRenderer *renderer,
+				   GtkTreeModel *tree_model, GtkTreeIter *iter,
+				   struct RBEntryViewCellDataFuncData *data)
+{
+	RhythmDBEntry *entry;
+	char str[255];
+	int julian;
+	GDate *date;
+
+	entry = entry_from_tree_iter (data->view, iter);
+	julian = rhythmdb_entry_get_ulong (entry, RHYTHMDB_PROP_DATE);
+	
+	if (julian > 0) {
+		date = g_date_new_julian (julian);
+		g_date_strftime (str, sizeof (str), "%Y", date);	
+		g_object_set (G_OBJECT (renderer), "text", str, NULL);
+		g_date_free (date);
+	} else {
+		g_object_set (G_OBJECT (renderer), "text", _("Unknown"), NULL);
+	}
+}
+
 #if 0
 static void
 rb_entry_view_quality_cell_data_func (GtkTreeViewColumn *column, GtkCellRenderer *renderer,
@@ -1116,7 +1195,7 @@ rb_entry_view_set_sorting_type (RBEntryView *view, const char *sorttype)
 	char **strs;
 
 	if (!sorttype || !strchr (sorttype, ',')) {
-		g_warning ("malformed sort data");
+		rb_debug ("malformed sort data: %s", (sorttype) ? sorttype : "(null)");
 		return;
 	}
 	
@@ -1310,6 +1389,15 @@ rb_entry_view_append_column (RBEntryView *view, RBEntryViewColumn coltype)
 		title = _("Tim_e");
 		key = "Time";
 		break;
+	case RB_ENTRY_VIEW_COL_YEAR:
+		propid = RHYTHMDB_PROP_DATE;
+		cell_data->propid = propid;
+		cell_data_func = (GtkTreeCellDataFunc) rb_entry_view_year_cell_data_func;
+		sort_data->propid = cell_data->propid;
+		sort_func = (GCompareDataFunc) rb_entry_view_date_sort_func;
+		title = _("_Year");
+		key = "Year";
+		break;
 #if 0
 	case RB_ENTRY_VIEW_COL_QUALITY:
 		propid = RHYTHMDB_PROP_BITRATE;
@@ -1386,9 +1474,6 @@ rb_entry_view_append_column_custom (RBEntryView *view,
 	gtk_tree_view_column_set_title (column, title);
 	gtk_tree_view_column_set_reorderable (column, FALSE);
 
-	if (gtk_tree_view_column_get_clickable (column))
-		view->priv->clickable_columns = g_list_append (view->priv->clickable_columns, column);
-
 	g_signal_connect_object (G_OBJECT (column), "clicked",
 				 G_CALLBACK (rb_entry_view_column_clicked_cb),
 				 view, 0);
@@ -1712,7 +1797,7 @@ harvest_entries (GtkTreeModel *model,
 
 	gtk_tree_model_get (model, iter, 0, &entry, -1);
 
-	*list = g_list_append (*list, entry);
+	*list = g_list_prepend (*list, entry);
 
 	return FALSE;
 }
@@ -1905,6 +1990,49 @@ rb_entry_view_row_changed_cb (GtkTreeModel *model,
 	queue_changed_sig (view);
 }
 
+static void
+rb_entry_view_rows_reordered_cb (GtkTreeModel *model,
+				 GtkTreePath *path,
+				 GtkTreeIter *iter,
+				 gint *order,
+				 RBEntryView *view)
+{
+	GList *selected_rows;
+	GList *i;
+	gint model_size;
+	rb_debug ("rows reordered");
+
+	model_size = gtk_tree_model_iter_n_children (model, NULL);
+
+	/* check if a selected row was moved; if so, we'll
+	 * need to move the selection too.
+	 */
+	selected_rows = gtk_tree_selection_get_selected_rows (view->priv->selection,
+							      NULL);
+	for (i = selected_rows; i != NULL; i = i->next) {
+		GtkTreePath *path = (GtkTreePath *)i->data;
+		gint index = gtk_tree_path_get_indices (path)[0];
+		gint newindex;
+		if (order[index] != index) {
+			GtkTreePath *newpath;
+			gtk_tree_selection_unselect_path (view->priv->selection, path);
+
+			for (newindex = 0; newindex < model_size; newindex++) {
+				if (order[newindex] == index) {
+					newpath = gtk_tree_path_new_from_indices (newindex, -1);
+					gtk_tree_selection_select_path (view->priv->selection, newpath);
+					gtk_tree_path_free (newpath);
+					break;
+				}
+			}
+			
+		}
+	}
+
+	g_list_foreach (selected_rows, (GFunc) gtk_tree_path_free, NULL);
+	g_list_free (selected_rows);
+}
+
 guint
 rb_entry_view_get_num_entries (RBEntryView *view)
 {
@@ -2115,27 +2243,28 @@ propid_from_name (const char *name)
 }
 
 static void
-set_column_not_visible (guint propid, GtkTreeViewColumn *column, gpointer unused)
+set_column_visibility (guint propid, GtkTreeViewColumn *column, GList *visible_props)
 {
+	gboolean visible;
+
 	/* title is always visible */
 	if (propid == RHYTHMDB_PROP_TITLE)
 		return;
-	
-	gtk_tree_view_column_set_visible (column, FALSE);
+
+	visible = (g_list_find (visible_props, GINT_TO_POINTER (propid)) != NULL);
+	gtk_tree_view_column_set_visible (column, visible);
 }
 
 static void
 rb_entry_view_sync_columns_visible (RBEntryView *view)
 {
 	char **items;
-	GList *visible_properties = NULL, *tem;
+	GList *visible_properties = NULL;
 	char *config = eel_gconf_get_string (CONF_UI_COLUMNS_SETUP);
 
 	g_return_if_fail (view != NULL);
 	g_return_if_fail (config != NULL);
 
-	g_hash_table_foreach (view->priv->propid_column_map, (GHFunc) set_column_not_visible, NULL);
-
 	items = g_strsplit (config, ",", 0);
 	if (items != NULL) {
 		int i;
@@ -2143,17 +2272,12 @@ rb_entry_view_sync_columns_visible (RBEntryView *view)
 			int value = propid_from_name (items[i]);
 
 			if ((value >= 0) && (value < RHYTHMDB_NUM_PROPERTIES))
-				visible_properties = g_list_append (visible_properties, GINT_TO_POINTER (value));
+				visible_properties = g_list_prepend (visible_properties, GINT_TO_POINTER (value));
 		}
 		g_strfreev (items);
 	}
 
-	for (tem = visible_properties; tem; tem = tem->next) {
-		GtkTreeViewColumn *column
-			= g_hash_table_lookup (view->priv->propid_column_map, tem->data);
-		if (column)
-			gtk_tree_view_column_set_visible (column, TRUE);
-	}
+	g_hash_table_foreach (view->priv->propid_column_map, (GHFunc) set_column_visibility, visible_properties);
 
 	g_list_free (visible_properties);
 	g_free (config);
@@ -2178,17 +2302,6 @@ rb_entry_view_set_playing (RBEntryView *view,
 	}
 }
 
-gboolean
-rb_entry_view_poll_model (RBEntryView *view)
-{
-	GTimeVal timeout;
-
-	g_get_current_time (&timeout);
-	g_time_val_add (&timeout, G_USEC_PER_SEC*0.75);
-
-	return rhythmdb_query_model_poll (view->priv->model, &timeout);
-}
-
 static void
 rb_entry_view_grab_focus (GtkWidget *widget)
 {
