@@ -50,6 +50,7 @@ typedef enum {
 	RB_ENTRY_VIEW_COL_QUALITY,
 	RB_ENTRY_VIEW_COL_RATING,
 	RB_ENTRY_VIEW_COL_PLAY_COUNT,
+	RB_ENTRY_VIEW_COL_YEAR,
 	RB_ENTRY_VIEW_COL_LAST_PLAYED,
 	RB_ENTRY_VIEW_COL_FIRST_SEEN
 } RBEntryViewColumn;
@@ -153,8 +154,6 @@ const char *	rb_entry_view_get_sorting_type		(RBEntryView *view);
 /** Should be called by sources before resorting the query model. */
 void		rb_entry_view_set_resorting		(RBEntryView *view);
 
-gboolean	rb_entry_view_poll_model		(RBEntryView *view);
-
 G_END_DECLS
 
 #endif /* __RB_ENTRY_VIEW_H */
