@@ -71,8 +71,6 @@ struct RBPropertyViewPrivate
 
 	RhythmDBPropType propid;
 
-	GPtrArray *query;
-	
 	RhythmDBPropertyModel *prop_model;
 
 	char *title;
@@ -80,9 +78,8 @@ struct RBPropertyViewPrivate
 	GtkWidget *treeview;
 	GtkTreeSelection *selection;
 
+	gboolean draggable;
 	gboolean handling_row_deletion;
-
-	guint refresh_idle_id;
 };
 
 enum
@@ -101,10 +98,9 @@ enum
 	PROP_PROP,
 	PROP_TITLE,
 	PROP_MODEL,
+	PROP_DRAGGABLE,
 };
 
-static GObjectClass *parent_class = NULL;
-
 static guint rb_property_view_signals[LAST_SIGNAL] = { 0 };
 
 enum {
@@ -127,41 +123,14 @@ static const GtkTargetEntry targets_artist [] = {
 	{ "text/uri-list", 0, TARGET_URIS },
 };
 
-GType
-rb_property_view_get_type (void)
-{
-	static GType rb_property_view_type = 0;
+G_DEFINE_TYPE (RBPropertyView, rb_property_view, GTK_TYPE_SCROLLED_WINDOW)
 
-	if (rb_property_view_type == 0)
-	{
-		static const GTypeInfo our_info =
-		{
-			sizeof (RBPropertyViewClass),
-			NULL,
-			NULL,
-			(GClassInitFunc) rb_property_view_class_init,
-			NULL,
-			NULL,
-			sizeof (RBPropertyView),
-			0,
-			(GInstanceInitFunc) rb_property_view_init
-		};
-		
-		rb_property_view_type = g_type_register_static (GTK_TYPE_SCROLLED_WINDOW,
-								"RBPropertyView",
-								&our_info, 0);
-	}
-
-	return rb_property_view_type;
-}
 
 static void
 rb_property_view_class_init (RBPropertyViewClass *klass)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
-	parent_class = g_type_class_peek_parent (klass);
-
 	object_class->finalize = rb_property_view_finalize;
 	object_class->constructor = rb_property_view_constructor;
 
@@ -199,6 +168,13 @@ rb_property_view_class_init (RBPropertyViewClass *klass)
 							      "RhythmDBPropertyModel",
 							      RHYTHMDB_TYPE_PROPERTY_MODEL,
 							      G_PARAM_READWRITE));
+	g_object_class_install_property (object_class,
+					 PROP_DRAGGABLE,
+					 g_param_spec_boolean ("draggable",
+						 	       "draggable",
+							       "is a drag source",
+							       TRUE,
+							       G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
 
 	rb_property_view_signals[PROPERTY_ACTIVATED] =
 		g_signal_new ("property-activated",
@@ -242,6 +218,7 @@ rb_property_view_class_init (RBPropertyViewClass *klass)
 			      g_cclosure_marshal_VOID__VOID,
 			      G_TYPE_NONE,
 			      0); 
+	
 }
 
 static void
@@ -265,7 +242,7 @@ rb_property_view_finalize (GObject *object)
 	g_free (view->priv->title);
 	g_free (view->priv);
 
-	G_OBJECT_CLASS (parent_class)->finalize (object);
+	G_OBJECT_CLASS (rb_property_view_parent_class)->finalize (object);
 }
 
 
@@ -299,6 +276,7 @@ rb_property_view_set_property (GObject *object,
 			g_signal_handlers_disconnect_by_func (G_OBJECT (view->priv->prop_model),
 							      G_CALLBACK (rb_property_view_post_row_deleted_cb),
 							      view);
+			g_object_unref (G_OBJECT (view->priv->prop_model));
 		}
 
 		view->priv->prop_model = g_value_get_object (value);
@@ -309,6 +287,7 @@ rb_property_view_set_property (GObject *object,
 		gtk_tree_view_set_model (GTK_TREE_VIEW (view->priv->treeview),
 					 GTK_TREE_MODEL (view->priv->prop_model));
 
+		g_object_ref (G_OBJECT (view->priv->prop_model));
 		g_signal_connect_object (G_OBJECT (view->priv->prop_model),
 					 "pre-row-deletion",
 					 G_CALLBACK (rb_property_view_pre_row_deleted_cb),
@@ -333,6 +312,9 @@ rb_property_view_set_property (GObject *object,
 
 	}
 		break;
+	case PROP_DRAGGABLE:
+		view->priv->draggable = g_value_get_boolean (value);
+		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 		break;
@@ -361,6 +343,9 @@ rb_property_view_get_property (GObject *object,
 	case PROP_MODEL:
 		g_value_set_object (value, view->priv->prop_model);
 		break;
+	case PROP_DRAGGABLE:
+		g_value_set_boolean (value, view->priv->draggable);
+		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 		break;
@@ -381,6 +366,7 @@ rb_property_view_new (RhythmDB *db, guint propid, const char *title)
 					       "db", db,
 					       "prop", propid,
 					       "title", title,
+					       "draggable", TRUE,
 					       NULL));
 
 	g_return_val_if_fail (view->priv != NULL, NULL);
@@ -487,25 +473,25 @@ rb_property_view_constructor (GType type, guint n_construct_properties,
 
 	RBPropertyView *view;
 	RBPropertyViewClass *klass;
-	GObjectClass *parent_class; 
 
 	klass = RB_PROPERTY_VIEW_CLASS (g_type_class_peek (RB_TYPE_PROPERTY_VIEW));
 
-	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
-	view = RB_PROPERTY_VIEW (parent_class->constructor (type, n_construct_properties,
-							    construct_properties));
+	view = RB_PROPERTY_VIEW (G_OBJECT_CLASS (rb_property_view_parent_class)->
+			constructor (type, n_construct_properties, construct_properties));
 
 	view->priv->prop_model = rhythmdb_property_model_new (view->priv->db, view->priv->propid);
 	view->priv->treeview = GTK_WIDGET (gtk_tree_view_new_with_model (GTK_TREE_MODEL (view->priv->prop_model)));
 
-	rhythmdb_property_model_enable_drag (view->priv->prop_model,
-					     GTK_TREE_VIEW (view->priv->treeview));
+	if (view->priv->draggable)
+		rhythmdb_property_model_enable_drag (view->priv->prop_model,
+						     GTK_TREE_VIEW (view->priv->treeview));
 
 	g_signal_connect_object (G_OBJECT (view->priv->treeview),
 			         "row_activated",
 			         G_CALLBACK (rb_property_view_row_activated_cb),
 			         view,
 				 0);
+	
 	view->priv->selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (view->priv->treeview));
 	g_signal_connect_object (G_OBJECT (view->priv->selection),
 			         "changed",
@@ -526,7 +512,6 @@ rb_property_view_constructor (GType type, guint n_construct_properties,
 						 view, NULL);
 	gtk_tree_view_column_set_title (column, view->priv->title);
 	gtk_tree_view_column_set_sizing (column, GTK_TREE_VIEW_COLUMN_FIXED);
-
 	gtk_tree_view_append_column (GTK_TREE_VIEW (view->priv->treeview),
 				     column);
 
@@ -609,7 +594,7 @@ rb_property_view_selection_changed_cb (GtkTreeSelection *selection,
 				selected_properties = NULL;
 				break;
 			}
-			selected_properties = g_list_append (selected_properties,
+			selected_properties = g_list_prepend (selected_properties,
 							     g_strdup (selected_prop));
 		}
 
@@ -638,3 +623,9 @@ rb_property_view_selection_changed_cb (GtkTreeSelection *selection,
 	}
 }
 
+GtkWidget*
+rb_property_view_get_treeview	(RBPropertyView *view)
+{
+	return view->priv->treeview;
+}
+
